@@ -1,6 +1,6 @@
 #!/bin/bash
 
-VERSION="2.2.0"
+VERSION="2.3.0-alpha1"
 
 # ---------------------------------------------------------------------------------------------------------------------
 # check dependencies
@@ -30,8 +30,8 @@ source common.env
 CHROOT_CARLA_DIR="/tmp/carla-src"
 
 # used for downloading packages from kxstudio repos, in order to get lv2-gtk3 and windows bridges
-CARLA_GIT_VER="2.3~alpha1+git20200925"
-PKGS_NUM="20200925"
+CARLA_GIT_VER="2.2~rc1+git20200718"
+PKGS_NUM="20200718"
 
 # ---------------------------------------------------------------------------------------------------------------------
 # function to remove old stuff
@@ -150,7 +150,7 @@ if [ ! -f /tmp/setup-aria2 ]; then
 fi
 
 if [ ! -d ${CHROOT_CARLA_DIR} ]; then
-  git clone --depth=1 -b master git://github.com/falkTX/Carla ${CHROOT_CARLA_DIR}
+  git clone --depth=1 -b develop git://github.com/falkTX/Carla ${CHROOT_CARLA_DIR}
 fi
 
 if [ ! -f ${CHROOT_CARLA_DIR}/source/native-plugins/external/README.md ]; then
