@@ -1,6 +1,6 @@
 ﻿/*
  * Carla Plugin Host
- * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,10 +28,14 @@
 
 #define STR_MAX 0xFF
 
+#ifndef CARLA_BACKEND_NAMESPACE
+# define CARLA_BACKEND_NAMESPACE CarlaBackend
+#endif
+
 #ifdef __cplusplus
-# define CARLA_BACKEND_START_NAMESPACE namespace CarlaBackend {
+# define CARLA_BACKEND_START_NAMESPACE namespace CARLA_BACKEND_NAMESPACE {
 # define CARLA_BACKEND_END_NAMESPACE }
-# define CARLA_BACKEND_USE_NAMESPACE using namespace CarlaBackend;
+# define CARLA_BACKEND_USE_NAMESPACE using namespace CARLA_BACKEND_NAMESPACE;
 # include <algorithm>
 # include <cmath>
 # include <limits>
@@ -275,6 +279,24 @@ static const uint PLUGIN_OPTIONS_NULL = 0x10000;
 
 /** @} */
 
+/* ------------------------------------------------------------------------------------------------------------
+ * Audio Port Hints */
+
+/*!
+ * @defgroup AudioPortHints Audio Port Hints
+ *
+ * Various audio port hints.
+ * @see CarlaPlugin::getAudioPortHints() and carla_get_audio_port_hints()
+ * @{
+ */
+
+/*!
+ * Audio port should be used as sidechan.
+ */
+static const uint AUDIO_PORT_IS_SIDECHAIN = 0x1;
+
+/** @} */
+
 /* ------------------------------------------------------------------------------------------------------------
  * Parameter Hints */
 
@@ -309,9 +331,12 @@ static const uint PARAMETER_IS_LOGARITHMIC = 0x004;
 static const uint PARAMETER_IS_ENABLED = 0x010;
 
 /*!
- * Parameter is automable (real-time safe).
+ * Parameter is automatable (real-time safe).
  */
-static const uint PARAMETER_IS_AUTOMABLE = 0x020;
+static const uint PARAMETER_IS_AUTOMATABLE = 0x020;
+
+// for backwards compatibility
+static const uint PARAMETER_IS_AUTOMABLE = PARAMETER_IS_AUTOMATABLE;
 
 /*!
  * Parameter is read-only.
@@ -341,6 +366,31 @@ static const uint PARAMETER_USES_CUSTOM_TEXT = 0x400;
  */
 static const uint PARAMETER_CAN_BE_CV_CONTROLLED = 0x800;
 
+/*!
+ * Parameter should not be saved as part of the project/session.
+ * @note only valid for parameter inputs.
+ */
+static const uint PARAMETER_IS_NOT_SAVED = 0x1000;
+
+/** @} */
+
+/* ------------------------------------------------------------------------------------------------------------
+ * Mapped Parameter Flags */
+
+/*!
+ * @defgroup MappedParameterFlags Mapped Parameter Flags
+ *
+ * Various flags for parameter mappings.
+ * @see ParameterData::mappedFlags
+ * @{
+ */
+
+/*!
+ * Parameter mapping uses delta value instead of full scale.
+ * Only relevant for MIDI CC mappings.
+ */
+static const uint PARAMETER_MAPPING_MIDI_DELTA = 0x001;
+
 /** @} */
 
 /* ------------------------------------------------------------------------------------------------------------
@@ -435,6 +485,11 @@ static const char* const CUSTOM_DATA_TYPE_BOOLEAN = "http://kxstudio.sf.net/ns/c
  */
 static const char* const CUSTOM_DATA_TYPE_CHUNK = "http://kxstudio.sf.net/ns/carla/chunk";
 
+/*!
+ * Path type URI.
+ */
+static const char* const CUSTOM_DATA_TYPE_PATH = "http://kxstudio.sf.net/ns/carla/path";
+
 /*!
  * Property type URI.
  */
@@ -611,7 +666,12 @@ typedef enum {
     /*!
      * JACK application.
      */
-    PLUGIN_JACK = 12
+    PLUGIN_JACK = 12,
+
+    /*!
+     * JSFX plugin.
+     */
+    PLUGIN_JSFX = 13
 
 } PluginType;
 
@@ -816,6 +876,7 @@ typedef enum {
     /*!
      * A plugin has been added.
      * @a pluginId Plugin Id
+     * @a value1   Plugin type
      * @a valueStr Plugin name
      */
     ENGINE_CALLBACK_PLUGIN_ADDED = 1,
@@ -1178,7 +1239,7 @@ typedef enum {
      * @a value1   New width
      * @a value2   New height
      */
-    ENGINE_CALLBACK_EMBED_UI_RESIZED = 48,
+    ENGINE_CALLBACK_EMBED_UI_RESIZED = 48
 
 } EngineCallbackOpcode;
 
@@ -1233,7 +1294,12 @@ typedef enum {
     /*!
      * Hide-optional-gui message.
      */
-    NSM_CALLBACK_HIDE_OPTIONAL_GUI = 7
+    NSM_CALLBACK_HIDE_OPTIONAL_GUI = 7,
+
+    /*!
+     * Set client name id message.
+     */
+    NSM_CALLBACK_SET_CLIENT_NAME_ID = 8
 
 } NsmCallbackOpcode;
 
@@ -1338,6 +1404,7 @@ typedef enum {
      */
     ENGINE_OPTION_AUDIO_DEVICE = 14,
 
+#ifndef BUILD_BRIDGE
     /*!
      * Wherever to enable OSC support in the engine.
      */
@@ -1359,6 +1426,7 @@ typedef enum {
      * @note Valid ports begin at 1024 and end at 32767 (inclusive)
      */
     ENGINE_OPTION_OSC_PORT_UDP = 17,
+#endif
 
     /*!
      * Set path used for a specific file type.
@@ -1445,17 +1513,24 @@ typedef enum {
     ENGINE_OPTION_WINE_SERVER_RT_PRIO = 32,
 #endif
 
+#ifndef BUILD_BRIDGE
     /*!
      * Capture console output into debug callbacks.
      */
     ENGINE_OPTION_DEBUG_CONSOLE_OUTPUT = 33,
+#endif
 
     /*!
      * A prefix to give to all plugin clients created by Carla.
      * Mostly useful for JACK multi-client mode.
      * @note MUST include at least one "." (dot).
      */
-    ENGINE_OPTION_CLIENT_NAME_PREFIX = 34
+    ENGINE_OPTION_CLIENT_NAME_PREFIX = 34,
+
+    /*!
+     * Treat loaded plugins as standalone (that is, there is no host UI to manage them)
+     */
+    ENGINE_OPTION_PLUGINS_ARE_STANDALONE = 35
 
 } EngineOption;
 
@@ -1669,6 +1744,12 @@ typedef struct {
      */
     float mappedMaximum;
 
+    /*!
+     * Flags related to the current mapping of this parameter.
+     * @see MappedParameterFlags
+     */
+    uint mappedFlags;
+
 } ParameterData;
 
 /*!
@@ -1829,7 +1910,7 @@ typedef struct {
 /*!
  * Custom data, used for saving key:value 'dictionaries'.
  */
-typedef struct {
+typedef struct _CustomData {
     /*!
      * Value type, in URI form.
      * @see CustomDataTypes
