@@ -1,6 +1,6 @@
 /*
  * Carla Plugin Host
- * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,6 +19,7 @@
 #define CARLA_ENGINE_HPP_INCLUDED
 
 #include "CarlaBackend.h"
+#include "CarlaPluginPtr.hpp"
 
 namespace water {
 class MemoryOutputStream;
@@ -176,7 +177,9 @@ static const uint8_t kEngineEventNonMidiChannel = 0x30;
 struct CARLA_API EngineControlEvent {
     EngineControlEventType type; //!< Control-Event type.
     uint16_t param;              //!< Parameter Id, midi bank or midi program.
-    float    value;              //!< Parameter value, normalized to 0.0f<->1.0f.
+    int8_t   midiValue;          //!< Raw midi value, >= 0 if applicable, -1 otherwise.
+    float    normalizedValue;    //!< Parameter value, normalized to 0.0f<->1.0f.
+    bool     handled;            //!< Indicates that event was handled/received at least once.
 
     /*!
      * Convert this control event into MIDI data.
@@ -239,6 +242,7 @@ struct CARLA_API EngineOptions {
     bool preferPluginBridges;
     bool preferUiBridges;
     bool uisAlwaysOnTop;
+    bool pluginsAreStandalone;
     uint bgColor;
     uint fgColor;
     float uiScale;
@@ -267,9 +271,11 @@ struct CARLA_API EngineOptions {
     const char* pathVST3;
     const char* pathSF2;
     const char* pathSFZ;
+    const char* pathJSFX;
 
     const char* binaryDir;
     const char* resourceDir;
+    const char* clientNamePrefix;
 
     bool preventBadBehaviour;
     uintptr_t frontendWinId;
@@ -573,12 +579,12 @@ public:
      * Get the event at @a index.
      * @note You must only call this for input ports.
      */
-    virtual const EngineEvent& getEvent(uint32_t index) const noexcept;
+    virtual EngineEvent& getEvent(uint32_t index) const noexcept;
 
     /*!
      * Get the event at @a index, faster unchecked version.
      */
-    virtual const EngineEvent& getEventUnchecked(uint32_t index) const noexcept;
+    virtual EngineEvent& getEventUnchecked(uint32_t index) const noexcept;
 
     /*!
      * Write a control event into the buffer.
@@ -591,7 +597,8 @@ public:
      * Arguments are the same as in the EngineControlEvent struct.
      * @note You must only call this for output ports.
      */
-    virtual bool writeControlEvent(uint32_t time, uint8_t channel, EngineControlEventType type, uint16_t param, float value = 0.0f) noexcept;
+    virtual bool writeControlEvent(uint32_t time, uint8_t channel, EngineControlEventType type,
+                                   uint16_t param, int8_t midiValue, float normalizedValue) noexcept;
 
     /*!
      * Write a MIDI event into the buffer.
@@ -640,7 +647,7 @@ public:
     /*!
      * Add a CV port as a source of events.
      */
-    virtual bool addCVSource(CarlaEngineCVPort* port, uint32_t portIndexOffset);
+    virtual bool addCVSource(CarlaEngineCVPort* port, uint32_t portIndexOffset, bool reconfigureNow);
 
     /*!
      * Remove a CV port as a source of events.
@@ -704,7 +711,7 @@ public:
      * Deactivate this client.
      * Client must be activated before calling this function.
      */
-    virtual void deactivate() noexcept;
+    virtual void deactivate(bool willClose) noexcept;
 
     /*!
      * Check if the client is activated.
@@ -925,7 +932,7 @@ public:
      * Add new engine client.
      * @note This function must only be called within a plugin class.
      */
-    virtual CarlaEngineClient* addClient(CarlaPlugin* plugin);
+    virtual CarlaEngineClient* addClient(CarlaPluginPtr plugin);
 
     /*!
      * Get the current CPU load estimated by the engine.
@@ -1024,12 +1031,12 @@ public:
     /*!
      * Get plugin with id @a id.
      */
-    CarlaPlugin* getPlugin(uint id) const noexcept;
+    CarlaPluginPtr getPlugin(uint id) const noexcept;
 
     /*!
      * Get plugin with id @a id, faster unchecked version.
      */
-    CarlaPlugin* getPluginUnchecked(uint id) const noexcept;
+    CarlaPluginPtr getPluginUnchecked(uint id) const noexcept;
 
     /*!
      * Get a unique plugin name within the engine.
@@ -1059,8 +1066,15 @@ public:
     bool saveProject(const char* filename, bool setAsCurrentProject);
 
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+    /*!
+     * Get the currently set project folder.
+     * @note Valid for both standalone and plugin versions.
+     */
+    virtual const char* getCurrentProjectFolder() const noexcept;
+
     /*!
      * Get the currently set project filename.
+     * @note Valid only for both standalone version.
      */
     const char* getCurrentProjectFilename() const noexcept;
 
@@ -1145,7 +1159,8 @@ public:
      * Call the file callback, if set.
      * May be called by plugins.
      */
-    const char* runFileCallback(FileCallbackOpcode action, bool isDir, const char* title, const char* filter) noexcept;
+    virtual const char* runFileCallback(FileCallbackOpcode action,
+                                        bool isDir, const char* title, const char* filter) noexcept;
 
     /*!
      * Set the file callback to @a func.
@@ -1168,6 +1183,12 @@ public:
      */
     virtual bool patchbayDisconnect(bool external, uint connectionId);
 
+    /*!
+     * Set the position of a group.
+     */
+    virtual bool patchbaySetGroupPos(bool sendHost, bool sendOSC, bool external,
+                                     uint groupId, int x1, int y1, int x2, int y2);
+
     /*!
      * Force the engine to resend all patchbay clients, ports and connections again.
      */
@@ -1321,6 +1342,7 @@ protected:
      */
     void setPluginPeaksRT(uint pluginId, float const inPeaks[2], float const outPeaks[2]) noexcept;
 
+public:
     /*!
      * Common save project function for main engine and plugin.
      */
@@ -1329,8 +1351,9 @@ protected:
     /*!
      * Common load project function for main engine and plugin.
      */
-    bool loadProjectInternal(water::XmlDocument& xmlDoc);
+    bool loadProjectInternal(water::XmlDocument& xmlDoc, bool alwaysLoadConnections);
 
+protected:
     // -------------------------------------------------------------------
     // Helper functions
 
@@ -1348,10 +1371,12 @@ protected:
      * Virtual functions for handling patchbay state.
      * Do not free returned data.
      */
+    struct PatchbayPosition { const char* name; int x1, y1, x2, y2, pluginId; bool dealloc; };
     virtual const char* const* getPatchbayConnections(bool external) const;
-    virtual void restorePatchbayConnection(bool external,
-                                           const char* sourcePort,
-                                           const char* targetPort);
+    virtual const PatchbayPosition* getPatchbayPositions(bool external, uint& count) const;
+    virtual void restorePatchbayConnection(bool external, const char* sourcePort, const char* targetPort);
+    // returns true if plugin name mapping found, ppos.name updated to its converted name
+    virtual bool restorePatchbayGroupPosition(bool external, PatchbayPosition& ppos);
 
     /*!
      * Virtual functions for handling external graph ports.
@@ -1362,62 +1387,6 @@ protected:
 
     // -------------------------------------------------------------------
 
-public:
-    /*!
-     * Native audio APIs.
-     */
-    enum AudioApi {
-        AUDIO_API_NULL,
-        // common
-        AUDIO_API_JACK,
-        AUDIO_API_OSS,
-        // linux
-        AUDIO_API_ALSA,
-        AUDIO_API_PULSEAUDIO,
-        // macos
-        AUDIO_API_COREAUDIO,
-        // windows
-        AUDIO_API_ASIO,
-        AUDIO_API_DIRECTSOUND,
-        AUDIO_API_WASAPI
-    };
-
-    // -------------------------------------------------------------------
-    // Engine initializers
-
-    // JACK
-    static CarlaEngine* newJack();
-
-#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
-    // Dummy
-    static CarlaEngine* newDummy();
-#endif
-
-#ifdef BUILD_BRIDGE
-    // Bridge
-    static CarlaEngine* newBridge(const char* audioPoolBaseName,
-                                  const char* rtClientBaseName,
-                                  const char* nonRtClientBaseName,
-                                  const char* nonRtServerBaseName);
-#else
-# ifdef USING_JUCE
-    // Juce
-    static CarlaEngine*       newJuce(AudioApi api);
-    static uint               getJuceApiCount();
-    static const char*        getJuceApiName(uint index);
-    static const char* const* getJuceApiDeviceNames(uint index);
-    static const EngineDriverDeviceInfo* getJuceDeviceInfo(uint index, const char* deviceName);
-    static bool               showJuceDeviceControlPanel(uint index, const char* deviceName);
-# else
-    // RtAudio
-    static CarlaEngine*       newRtAudio(AudioApi api);
-    static uint               getRtAudioApiCount();
-    static const char*        getRtAudioApiName(uint index);
-    static const char* const* getRtAudioApiDeviceNames(uint index);
-    static const EngineDriverDeviceInfo* getRtAudioDeviceInfo(uint index, const char* deviceName);
-# endif
-#endif
-
     CARLA_DECLARE_NON_COPY_CLASS(CarlaEngine)
 };
 
