@@ -1,6 +1,6 @@
 ﻿/*
  * Carla Plugin Host
- * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -237,6 +237,11 @@ public:
      */
     int32_t getCurrentMidiProgram() const noexcept;
 
+    /*!
+     * Get hints about an audio port.
+     */
+    virtual uint getAudioPortHints(bool isOutput, uint32_t portIndex) const noexcept;
+
     /*!
      * Get the parameter data of @a parameterId.
      */
@@ -447,7 +452,7 @@ public:
     bool loadStateFromFile(const char* filename);
 
     /*!
-     * Export this plugin as LV2.
+     * Export this plugin as its own LV2 plugin, using a carla wrapper around it for the LV2 functionality.
      */
     bool exportAsLV2(const char* lv2path);
 
