@@ -1,6 +1,6 @@
 /*
  * Carla Standalone
- * Copyright (C) 2011-2021 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -146,7 +146,7 @@ void _CarlaTransportInfo::clear() noexcept
 
 // --------------------------------------------------------------------------------------------------------------------
 
-using CarlaBackend::CarlaPluginPtr;
+using CARLA_BACKEND_NAMESPACE::CarlaPluginPtr;
 
 // --------------------------------------------------------------------------------------------------------------------
 
@@ -283,6 +283,9 @@ static void carla_engine_init_common(const CarlaHostStandalone& standalone, Carl
     if (const char* const pathSFZ = std::getenv("ENGINE_OPTION_PLUGIN_PATH_SFZ"))
         engine->setOption(CB::ENGINE_OPTION_PLUGIN_PATH, CB::PLUGIN_SFZ, pathSFZ);
 
+    if (const char* const pathJSFX = std::getenv("ENGINE_OPTION_PLUGIN_PATH_JSFX"))
+        engine->setOption(CB::ENGINE_OPTION_PLUGIN_PATH, CB::PLUGIN_JSFX, pathJSFX);
+
     if (const char* const binaryDir = std::getenv("ENGINE_OPTION_PATH_BINARIES"))
         engine->setOption(CB::ENGINE_OPTION_PATH_BINARIES, 0, binaryDir);
     else
@@ -347,6 +350,9 @@ static void carla_engine_init_common(const CarlaHostStandalone& standalone, Carl
     if (standalone.engineOptions.pathSFZ != nullptr)
         engine->setOption(CB::ENGINE_OPTION_PLUGIN_PATH,       CB::PLUGIN_SFZ, standalone.engineOptions.pathSFZ);
 
+    if (standalone.engineOptions.pathJSFX != nullptr)
+        engine->setOption(CB::ENGINE_OPTION_PLUGIN_PATH,       CB::PLUGIN_JSFX, standalone.engineOptions.pathJSFX);
+
     if (standalone.engineOptions.binaryDir != nullptr && standalone.engineOptions.binaryDir[0] != '\0')
         engine->setOption(CB::ENGINE_OPTION_PATH_BINARIES, 0, standalone.engineOptions.binaryDir);
     else
@@ -851,7 +857,7 @@ void carla_set_engine_option(CarlaHostHandle handle, EngineOption option, int va
 
         case CB::ENGINE_OPTION_PLUGIN_PATH:
             CARLA_SAFE_ASSERT_RETURN(value > CB::PLUGIN_NONE,);
-            CARLA_SAFE_ASSERT_RETURN(value <= CB::PLUGIN_SFZ,);
+            CARLA_SAFE_ASSERT_RETURN(value <= CB::PLUGIN_JSFX,);
             CARLA_SAFE_ASSERT_RETURN(valueStr != nullptr,);
 
             switch (value)
@@ -891,6 +897,11 @@ void carla_set_engine_option(CarlaHostHandle handle, EngineOption option, int va
                     delete[] shandle.engineOptions.pathSFZ;
                 shandle.engineOptions.pathSFZ = carla_strdup_safe(valueStr);
                 break;
+            case CB::PLUGIN_JSFX:
+                if (shandle.engineOptions.pathJSFX != nullptr)
+                    delete[] shandle.engineOptions.pathJSFX;
+                shandle.engineOptions.pathJSFX = carla_strdup_safe(valueStr);
+                break;
             }
             break;
 
@@ -1573,6 +1584,22 @@ const CarlaScalePointInfo* carla_get_parameter_scalepoint_info(CarlaHostHandle h
 
 // --------------------------------------------------------------------------------------------------------------------
 
+uint carla_get_audio_port_hints(CarlaHostHandle handle, uint pluginId, bool isOutput, uint32_t portIndex)
+{
+    CARLA_SAFE_ASSERT_RETURN(handle->engine != nullptr, 0x0);
+
+    if (const CarlaPluginPtr plugin = handle->engine->getPlugin(pluginId))
+    {
+        CARLA_SAFE_ASSERT_RETURN(portIndex < (isOutput ? plugin->getAudioOutCount() : plugin->getAudioInCount()), 0x0);
+
+        return plugin->getAudioPortHints(isOutput, portIndex);
+    }
+
+    return 0x0;
+}
+
+// --------------------------------------------------------------------------------------------------------------------
+
 const ParameterData* carla_get_parameter_data(CarlaHostHandle handle, uint pluginId, uint32_t parameterId)
 {
     static ParameterData retParamData;
@@ -1960,6 +1987,18 @@ float carla_get_internal_parameter_value(CarlaHostHandle handle, uint pluginId,
 
 // --------------------------------------------------------------------------------------------------------------------
 
+uint32_t carla_get_plugin_latency(CarlaHostHandle handle, uint pluginId)
+{
+    CARLA_SAFE_ASSERT_RETURN(handle->engine != nullptr, 0);
+
+    if (const CarlaPluginPtr plugin = handle->engine->getPlugin(pluginId))
+         return plugin->getLatencyInFrames();
+
+    return 0;
+}
+
+// --------------------------------------------------------------------------------------------------------------------
+
 const float* carla_get_peak_values(CarlaHostHandle handle, uint pluginId)
 {
     CARLA_SAFE_ASSERT_RETURN(handle->engine != nullptr, nullptr);
