@@ -21,6 +21,8 @@
 #include "CarlaBackend.h"
 #include "CarlaThread.hpp"
 
+#include "CarlaJuceUtils.hpp"
+
 CARLA_BACKEND_START_NAMESPACE
 
 // -----------------------------------------------------------------------
@@ -29,7 +31,7 @@ CARLA_BACKEND_START_NAMESPACE
 class CarlaEngineThread : public CarlaThread
 {
 public:
-    CarlaEngineThread(CarlaEngine* const engine) noexcept;
+    CarlaEngineThread(CarlaEngine* engine) noexcept;
     ~CarlaEngineThread() noexcept override;
 
 protected:
