@@ -1,6 +1,6 @@
 ﻿/*
  * Carla Plugin
- * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -242,6 +242,11 @@ int32_t CarlaPlugin::getCurrentMidiProgram() const noexcept
     return pData->midiprog.current;
 }
 
+uint CarlaPlugin::getAudioPortHints(bool, uint32_t) const noexcept
+{
+    return 0x0;
+}
+
 const ParameterData& CarlaPlugin::getParameterData(const uint32_t parameterId) const noexcept
 {
     CARLA_SAFE_ASSERT_RETURN(parameterId < pData->param.count, kParameterDataNull);
@@ -1230,7 +1235,7 @@ bool CarlaPlugin::exportAsLV2(const char* const lv2path)
             if (paramData.hints & PARAMETER_IS_INTEGER)
                 mainStream << "        lv2:portProperty lv2:integer ;\n";
 
-            // TODO logarithmic, enabled (not on gui), automable, samplerate, scalepoints
+            // TODO logarithmic, enabled (not on gui), automatable, samplerate, scalepoints
 
             if (! getParameterName(i, strBufName))
                 strBufName[0] = '\0';
