@@ -1433,7 +1433,7 @@ public:
                     }
                 }
 
-                // Volume (and buffer copy)
+                // Volume
                 if (doVolume)
                 {
                     for (uint32_t k=0; k < frames; ++k)
@@ -1533,6 +1533,8 @@ public:
             case kPluginBridgeNonRtServerMidiProgramData:
             case kPluginBridgeNonRtServerSetCustomData:
             case kPluginBridgeNonRtServerVersion:
+            case kPluginBridgeNonRtServerRespEmbedUI:
+            case kPluginBridgeNonRtServerResizeEmbedUI:
                 break;
 
             case kPluginBridgeNonRtServerSetChunkDataFile:
