@@ -24,7 +24,8 @@
 #include "CarlaMathUtils.hpp"
 #include "CarlaProcessUtils.hpp"
 #include "CarlaScopeUtils.hpp"
-#include "CarlaVstUtils.hpp"
+#include "CarlaVst2Utils.hpp"
+#include "CarlaVst3Utils.hpp"
 
 #if defined(__clang__)
 # pragma clang diagnostic push
@@ -495,7 +496,16 @@ public:
                         uiName += " (GUI)";
                     }
 
-                    fWindow = new JucePluginWindow(opts.frontendWinId, opts.pluginsAreStandalone);
+                    AEffect* const vst2effect = fDesc.pluginFormatName == "VST" || fDesc.pluginFormatName == "VST2"
+                                              ? (AEffect*)fInstance->getPlatformSpecificData()
+                                              : nullptr;
+
+                    v3_plugin_view** const vst3view = fDesc.pluginFormatName == "VST3"
+                                                    ? (v3_plugin_view**)editor->getPlatformSpecificData()
+                                                    : nullptr;
+
+                    fWindow = new JucePluginWindow(opts.frontendWinId, opts.pluginsAreStandalone,
+                                                   vst2effect, vst3view);
                     fWindow->setName(uiName);
                 }
 
@@ -557,6 +567,7 @@ public:
 
         const bool isAU = fDesc.pluginFormatName == "AU" || fDesc.pluginFormatName == "AudioUnit";
         const bool isVST2 = fDesc.pluginFormatName == "VST" || fDesc.pluginFormatName == "VST2";
+        const bool isVST3 = fDesc.pluginFormatName == "VST3";
         findMaxTotalChannels(fInstance.get(), isAU, aIns, aOuts);
         fInstance->refreshParameterList();
 
@@ -659,11 +670,12 @@ public:
             pData->param.data[j].hints |= PARAMETER_USES_CUSTOM_TEXT;
 
             if (parameter->isAutomatable())
-                pData->param.data[j].hints |= PARAMETER_IS_AUTOMABLE;
+                pData->param.data[j].hints |= PARAMETER_IS_AUTOMATABLE;
 
             const float min = 0.0f;
             const float max = 1.0f;
             float def, step, stepSmall, stepLarge;
+            bool hasDetails = false;
 
             if (isVST2)
             {
@@ -674,6 +686,8 @@ public:
 
                 if (effect != nullptr && effect->dispatcher(effect, effGetParameterProperties, ij, 0, &prop, 0.0f) == 1)
                 {
+                    hasDetails = true;
+
                     /**/ if (prop.flags & kVstParameterIsSwitch)
                     {
                         step = max - min;
@@ -706,30 +720,74 @@ public:
                     if (prop.flags & kVstParameterCanRamp)
                         pData->param.data[j].hints |= PARAMETER_IS_LOGARITHMIC;
                 }
+            }
+            else if (isVST3)
+            {
+                v3_component** const component = (v3_component**)fInstance->getPlatformSpecificData();
+
+                v3_edit_controller** controller = nullptr;
+                v3_cpp_obj_query_interface(component, v3_edit_controller_iid, &controller);
+
+                if (controller != nullptr)
+                {
+                    v3_param_info info;
+                    if (v3_cpp_obj(controller)->get_parameter_info(controller, static_cast<int32_t>(j), &info) == V3_OK)
+                    {
+                        hasDetails = true;
+
+                        if (info.step_count == 1)
+                        {
+                            step = stepSmall = stepLarge = 1;
+                            pData->param.data[j].hints |= PARAMETER_IS_BOOLEAN;
+                        }
+                        else if (info.step_count != 0)
+                        {
+                            step = 1.0f / static_cast<float>(info.step_count);
+                            stepSmall = step/10.0f;
+                            stepLarge = std::min(1.0f, step*10.0f);
+                        }
+                        else
+                        {
+                            const float range = max - min;
+                            step = range/100.0f;
+                            stepSmall = range/1000.0f;
+                            stepLarge = range/10.0f;
+                        }
+
+                        if (info.flags & V3_PARAM_READ_ONLY)
+                            pData->param.data[j].type = PARAMETER_OUTPUT;
+                        // TODO V3_PARAM_IS_LIST
+                        if (info.flags & (V3_PARAM_IS_HIDDEN|V3_PARAM_PROGRAM_CHANGE))
+                            pData->param.data[j].hints &= ~PARAMETER_IS_ENABLED;
+
+                        if ((info.flags & (V3_PARAM_IS_LIST|V3_PARAM_IS_HIDDEN|V3_PARAM_PROGRAM_CHANGE)) == 0x0)
+                            if (info.flags & V3_PARAM_CAN_AUTOMATE)
+                                pData->param.data[j].hints |= PARAMETER_CAN_BE_CV_CONTROLLED;
+                    }
+
+                    v3_cpp_obj_unref(controller);
+                }
+            }
+
+            if (! hasDetails)
+            {
+                if (parameter->isBoolean())
+                {
+                    step = max - min;
+                    stepSmall = step;
+                    stepLarge = step;
+                    pData->param.data[j].hints |= PARAMETER_IS_BOOLEAN;
+                }
                 else
                 {
                     const float range = max - min;
                     step = range/100.0f;
                     stepSmall = range/1000.0f;
                     stepLarge = range/10.0f;
-                }
-            }
-            else if (parameter->isBoolean())
-            {
-                step = max - min;
-                stepSmall = step;
-                stepLarge = step;
-                pData->param.data[j].hints |= PARAMETER_IS_BOOLEAN;
-            }
-            else
-            {
-                const float range = max - min;
-                step = range/100.0f;
-                stepSmall = range/1000.0f;
-                stepLarge = range/10.0f;
 
-                if (! parameter->isMetaParameter())
-                    pData->param.data[j].hints |= PARAMETER_CAN_BE_CV_CONTROLLED;
+                    if (! parameter->isMetaParameter())
+                        pData->param.data[j].hints |= PARAMETER_CAN_BE_CV_CONTROLLED;
+                }
             }
 
             def = parameter->getDefaultValue();
@@ -1080,7 +1138,7 @@ public:
                                 continue;
                             if (pData->param.data[k].type != PARAMETER_INPUT)
                                 continue;
-                            if ((pData->param.data[k].hints & PARAMETER_IS_AUTOMABLE) == 0)
+                            if ((pData->param.data[k].hints & PARAMETER_IS_AUTOMATABLE) == 0)
                                 continue;
 
                             ctrlEvent.handled = true;
@@ -1304,6 +1362,75 @@ public:
         for (uint32_t i=0; i < pData->audioOut.count; ++i)
             carla_copyFloats(outBuffer[i], fAudioBuffer.getReadPointer(static_cast<int>(i)), frames);
 
+#ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
+        // --------------------------------------------------------------------------------------------------------
+        // Post-processing (dry/wet, volume and balance)
+
+        {
+            const bool doVolume  = (pData->hints & PLUGIN_CAN_VOLUME) != 0 && carla_isNotEqual(pData->postProc.volume, 1.0f);
+            const bool doDryWet  = (pData->hints & PLUGIN_CAN_DRYWET) != 0 && carla_isNotEqual(pData->postProc.dryWet, 1.0f);
+            const bool doBalance = (pData->hints & PLUGIN_CAN_BALANCE) != 0 && ! (carla_isEqual(pData->postProc.balanceLeft, -1.0f) && carla_isEqual(pData->postProc.balanceRight, 1.0f));
+            const bool isMono    = (pData->audioIn.count == 1);
+
+            bool isPair;
+            float bufValue, oldBufLeft[doBalance ? frames : 1];
+
+            for (uint32_t i=0; i < pData->audioOut.count; ++i)
+            {
+                // Dry/Wet
+                if (doDryWet)
+                {
+                    const uint32_t c = isMono ? 0 : i;
+
+                    for (uint32_t k=0; k < frames; ++k)
+                    {
+                        bufValue = inBuffer[c][k];
+                        outBuffer[i][k] = (outBuffer[i][k] * pData->postProc.dryWet) + (bufValue * (1.0f - pData->postProc.dryWet));
+                    }
+                }
+
+                // Balance
+                if (doBalance)
+                {
+                    isPair = (i % 2 == 0);
+
+                    if (isPair)
+                    {
+                        CARLA_ASSERT(i+1 < pData->audioOut.count);
+                        carla_copyFloats(oldBufLeft, outBuffer[i], frames);
+                    }
+
+                    float balRangeL = (pData->postProc.balanceLeft  + 1.0f)/2.0f;
+                    float balRangeR = (pData->postProc.balanceRight + 1.0f)/2.0f;
+
+                    for (uint32_t k=0; k < frames; ++k)
+                    {
+                        if (isPair)
+                        {
+                            // left
+                            outBuffer[i][k]  = oldBufLeft[k]     * (1.0f - balRangeL);
+                            outBuffer[i][k] += outBuffer[i+1][k] * (1.0f - balRangeR);
+                        }
+                        else
+                        {
+                            // right
+                            outBuffer[i][k]  = outBuffer[i][k] * balRangeR;
+                            outBuffer[i][k] += oldBufLeft[k]   * balRangeL;
+                        }
+                    }
+                }
+
+                // Volume
+                if (doVolume)
+                {
+                    for (uint32_t k=0; k < frames; ++k)
+                        outBuffer[i][k] *= pData->postProc.volume;
+                }
+            }
+
+        } // End of Post-processing
+#endif
+
         // --------------------------------------------------------------------------------------------------------
         // Midi out
 
