@@ -1,6 +1,6 @@
 /*
  * Carla Plugin Host
- * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,7 +30,7 @@
 #include "water/containers/Array.h"
 #include "water/files/File.h"
 
-namespace CB = CarlaBackend;
+namespace CB = CARLA_BACKEND_NAMESPACE;
 
 // -------------------------------------------------------------------------------------------------------------------
 
@@ -44,6 +44,7 @@ static bool isCachedPluginType(const CB::PluginType ptype)
     case CB::PLUGIN_LV2:
     case CB::PLUGIN_AU:
     case CB::PLUGIN_SFZ:
+    case CB::PLUGIN_JSFX:
         return true;
     default:
         return false;
@@ -178,15 +179,23 @@ static const CarlaCachedPluginInfo* get_cached_plugin_lv2(Lv2WorldClass& lv2Worl
             lilv_node_free(nameNode);
         }
 
-        if (const char* const author = lilvPlugin.get_author_name().as_string())
-            smaker = author;
+        if (LilvNode* const authorNode = lilv_plugin_get_author_name(lilvPlugin.me))
+        {
+            if (const char* const author = lilv_node_as_string(authorNode))
+                smaker = author;
+            lilv_node_free(authorNode);
+        }
 
         Lilv::Nodes licenseNodes(lilvPlugin.get_value(lv2World.doap_license));
 
         if (licenseNodes.size() > 0)
         {
-            if (const char* const license = licenseNodes.get_first().as_string())
-                slicense = license;
+            if (LilvNode* const licenseNode = lilv_nodes_get_first(licenseNodes.me))
+            {
+                if (const char* const license = lilv_node_as_string(licenseNode))
+                    slicense = license;
+                // lilv_node_free(licenseNode);
+            }
         }
 
         lilv_nodes_free(const_cast<LilvNodes*>(licenseNodes.me));
