@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Common Carla code
-# Copyright (C) 2011-2021 Filipe Coelho <falktx@falktx.com>
+# Copyright (C) 2011-2022 Filipe Coelho <falktx@falktx.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -71,7 +71,7 @@ if WINDOWS:
 # ------------------------------------------------------------------------------------------------------------
 # Set Version
 
-VERSION = "2.4.0"
+VERSION = "2.4.2"
 
 # ------------------------------------------------------------------------------------------------------------
 # Set TMP
@@ -299,7 +299,7 @@ if WINDOWS:
     CARLA_DEFAULT_AUDIO_DRIVER = "DirectSound"
 elif MACOS:
     CARLA_DEFAULT_AUDIO_DRIVER = "CoreAudio"
-elif os.path.exists("/usr/bin/jackd") or os.path.exists("/usr/bin/jackdbus"):
+elif os.path.exists("/usr/bin/jackd") or os.path.exists("/usr/bin/jackdbus") or os.path.exists("/usr/bin/pw-jack"):
     CARLA_DEFAULT_AUDIO_DRIVER = "JACK"
 else:
     CARLA_DEFAULT_AUDIO_DRIVER = "PulseAudio"
@@ -357,6 +357,7 @@ if WINDOWS:
     splitter = ";"
 
     APPDATA = os.getenv("APPDATA")
+    LOCALAPPDATA = os.getenv("LOCALAPPDATA", APPDATA)
     PROGRAMFILES = os.getenv("PROGRAMFILES")
     PROGRAMFILESx86 = os.getenv("PROGRAMFILES(x86)")
     COMMONPROGRAMFILES = os.getenv("COMMONPROGRAMFILES")
@@ -391,6 +392,7 @@ if WINDOWS:
         DEFAULT_VST2_PATH  += ";" + COMMONPROGRAMFILES + "\\VST2"
 
     DEFAULT_VST3_PATH    = COMMONPROGRAMFILES + "\\VST3"
+    DEFAULT_VST3_PATH   += ";" + LOCALAPPDATA + "\\Programs\\Common\\VST3"
 
     DEFAULT_SF2_PATH     = APPDATA + "\\SF2"
     DEFAULT_SFZ_PATH     = APPDATA + "\\SFZ"
@@ -470,7 +472,7 @@ else:
     DEFAULT_VST3_PATH   += ":/usr/local/lib/vst3"
 
     DEFAULT_SF2_PATH     = HOME + "/.sounds/sf2"
-    DEFAULT_SF2_PATH    += HOME + ":/.sounds/sf3"
+    DEFAULT_SF2_PATH    += ":" + HOME + "/.sounds/sf3"
     DEFAULT_SF2_PATH    += ":/usr/share/sounds/sf2"
     DEFAULT_SF2_PATH    += ":/usr/share/sounds/sf3"
     DEFAULT_SF2_PATH    += ":/usr/share/soundfonts"
