@@ -1,8 +1,8 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-# Pixmap Keyboard, a custom Qt4 widget
-# Copyright (C) 2011-2018 Filipe Coelho <falktx@falktx.com>
+# Pixmap Keyboard, a custom Qt widget
+# Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -16,22 +16,17 @@
 #
 # For a full copy of the GNU General Public License see the doc/GPL.txt file.
 
-# ------------------------------------------------------------------------------------------------------------
-# Imports (Config)
-
-from carla_config import *
-
 # ------------------------------------------------------------------------------------------------------------
 # Imports (Global)
 
-if config_UseQt5:
-    from PyQt5.QtCore import pyqtSignal, pyqtSlot, qCritical, Qt, QPointF, QRectF, QTimer, QSettings, QSize
-    from PyQt5.QtGui import QColor, QFont, QPainter, QPixmap
-    from PyQt5.QtWidgets import QMenu, QScrollArea, QWidget
-else:
-    from PyQt4.QtCore import pyqtSignal, pyqtSlot, qCritical, Qt, QPointF, QRectF, QTimer, QSettings, QSize
-    from PyQt4.QtGui import QColor, QFont, QPainter, QPixmap
-    from PyQt4.QtGui import QMenu, QScrollArea, QWidget
+from PyQt5.QtCore import pyqtSignal, pyqtSlot, qCritical, Qt, QPointF, QRectF, QTimer, QSize
+from PyQt5.QtGui import QColor, QFont, QPainter, QPixmap
+from PyQt5.QtWidgets import QActionGroup, QMenu, QScrollArea, QWidget
+
+# ------------------------------------------------------------------------------------------------------------
+# Imports (Custom)
+
+from carla_shared import QSafeSettings
 
 # ------------------------------------------------------------------------------------------------------------
 
@@ -227,17 +222,17 @@ class PixmapKeyboard(QWidget):
         if self.fInitalizing:
             return
 
-        settings = QSettings("falkTX", "CarlaKeyboard")
+        settings = QSafeSettings("falkTX", "CarlaKeyboard")
         settings.setValue("PcKeyboardLayout", self.fkPcKeyLayout)
         settings.setValue("PcKeyboardOffset", self.fPcKeybOffset)
         settings.setValue("HighlightColor", self.fHighlightColor)
         del settings
 
     def loadSettings(self):
-        settings = QSettings("falkTX", "CarlaKeyboard")
-        self.setPcKeyboardLayout(settings.value("PcKeyboardLayout", self.fkPcKeyLayout, type=str))
-        self.setPcKeyboardOffset(settings.value("PcKeyboardOffset", self.fPcKeybOffset, type=int))
-        self.setColor(settings.value("HighlightColor", self.fHighlightColor, type=str))
+        settings = QSafeSettings("falkTX", "CarlaKeyboard")
+        self.setPcKeyboardLayout(settings.value("PcKeyboardLayout", self.fkPcKeyLayout, str))
+        self.setPcKeyboardOffset(settings.value("PcKeyboardOffset", self.fPcKeybOffset, int))
+        self.setColor(settings.value("HighlightColor", self.fHighlightColor, str))
         del settings
 
     def allNotesOff(self, sendSignal=True):
@@ -378,34 +373,38 @@ class PixmapKeyboard(QWidget):
         event.accept()
         menu = QMenu()
 
-        menu.addAction("Note: restart carla to apply globally").setEnabled(False)
-        menu.addSeparator()
+        menu.addAction(self.tr("Note: restart carla to apply globally")).setEnabled(False)
+        menu.addAction(self.tr("Color")).setSeparator(True)
 
-        menuColor  = QMenu("Highlight color", menu)
-        menuLayout = QMenu("PC Keyboard layout", menu)
-        actColors  = []
-        actLayouts = []
+        groupColor  = QActionGroup(menu)
+        groupLayout = QActionGroup(menu)
+        actColors   = []
+        actLayouts  = []
+
+        menu.addAction(self.tr("Highlight color")).setSeparator(True)
 
         for color in kValidColors:
-            act = menuColor.addAction(color)
+            act = menu.addAction(color)
+            act.setActionGroup(groupColor)
             act.setCheckable(True)
             if self.fHighlightColor == color:
                 act.setChecked(True)
             actColors.append(act)
 
+        menu.addAction(self.tr("PC Keyboard layout")).setSeparator(True)
+
         for pcKeyLayout in kPcKeysLayouts.keys():
-            act = menuLayout.addAction(pcKeyLayout)
+            act = menu.addAction(pcKeyLayout)
+            act.setActionGroup(groupLayout)
             act.setCheckable(True)
             if self.fkPcKeyLayout == pcKeyLayout:
                 act.setChecked(True)
             actLayouts.append(act)
 
-        menu.addMenu(menuColor)
-        menu.addMenu(menuLayout)
-        menu.addSeparator()
+        menu.addAction(self.tr("PC Keyboard base octave (%i)" % self.fPcKeybOffset)).setSeparator(True)
 
-        actOctaveUp   = menu.addAction("PC Keyboard octave up")
-        actOctaveDown = menu.addAction("PC Keyboard octave down")
+        actOctaveUp   = menu.addAction(self.tr("Octave up"))
+        actOctaveDown = menu.addAction(self.tr("Octave down"))
 
         if self.fPcKeybOffset == 0:
             actOctaveDown.setEnabled(False)
@@ -629,7 +628,7 @@ class PixmapKeyboardHArea(QScrollArea):
         self.setWidget(self.keyboard)
 
         self.setEnabled(False)
-        self.setFixedHeight(self.keyboard.height() + self.horizontalScrollBar().height()/2 + 2)
+        self.setFixedHeight(int(self.keyboard.height() + self.horizontalScrollBar().height()/2 + 2))
 
         QTimer.singleShot(0, self.slot_initScrollbarValue)
 
@@ -640,7 +639,7 @@ class PixmapKeyboardHArea(QScrollArea):
 
     @pyqtSlot()
     def slot_initScrollbarValue(self):
-        self.horizontalScrollBar().setValue(self.horizontalScrollBar().maximum()/2)
+        self.horizontalScrollBar().setValue(int(self.horizontalScrollBar().maximum()/2))
 
 # ------------------------------------------------------------------------------------------------------------
 # Main Testing
