@@ -1431,12 +1431,6 @@ void iOSAudioIODeviceType::handleAsyncUpdate()
     callDeviceChangeListeners();
 }
 
-//==============================================================================
-AudioIODeviceType* AudioIODeviceType::createAudioIODeviceType_iOSAudio()
-{
-    return new iOSAudioIODeviceType();
-}
-
 //==============================================================================
 AudioSessionHolder::AudioSessionHolder()    { nativeSession = [[iOSAudioSessionNative alloc] init: this]; }
 AudioSessionHolder::~AudioSessionHolder()   { [nativeSession release]; }
