@@ -2234,12 +2234,6 @@ private:
 
 };
 
-//==============================================================================
-AudioIODeviceType* AudioIODeviceType::createAudioIODeviceType_CoreAudio()
-{
-    return new CoreAudioClasses::CoreAudioIODeviceType();
-}
-
 #undef JUCE_COREAUDIOLOG
 
 } // namespace juce
