@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -32,6 +31,8 @@
 namespace juce
 {
 
+#ifndef DOXYGEN
+
 struct AudioUnitHelpers
 {
     class ChannelRemapper
@@ -213,7 +214,7 @@ struct AudioUnitHelpers
         }
 
         //==============================================================================
-        AudioSampleBuffer& getBuffer (UInt32 frames) noexcept
+        AudioBuffer<float>& getBuffer (UInt32 frames) noexcept
         {
             jassert (pushIdx == scratch.getNumChannels());
 
@@ -240,9 +241,7 @@ struct AudioUnitHelpers
         }
 
         //==============================================================================
-        AudioSampleBuffer scratch;
-        AudioSampleBuffer mutableBuffer;
-
+        AudioBuffer<float> scratch, mutableBuffer;
         HeapBlock<float*> channels;
         int pushIdx, popIdx;
     };
@@ -304,7 +303,7 @@ struct AudioUnitHelpers
         }
     }
 
-    template <int numLayouts>
+    template <size_t numLayouts>
     static bool isLayoutSupported (const AudioProcessor& processor,
                                    bool isInput, int busIdx,
                                    int numChannels,
@@ -532,4 +531,6 @@ struct AudioUnitHelpers
     }
 };
 
+#endif // ! DOXYGEN
+
 } // namespace juce
