@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -40,8 +39,11 @@ namespace juce
 
     To play back a graph through an audio device, you might want to use an
     AudioProcessorPlayer object.
+
+    @tags{Audio}
 */
 class JUCE_API  AudioProcessorGraph   : public AudioProcessor,
+                                        public ChangeBroadcaster,
                                         private AsyncUpdater
 {
 public:
@@ -52,7 +54,43 @@ public:
     /** Destructor.
         Any processor objects that have been added to the graph will also be deleted.
     */
-    ~AudioProcessorGraph();
+    ~AudioProcessorGraph() override;
+
+    /** Each node in the graph has a UID of this type. */
+    struct NodeID
+    {
+        NodeID() {}
+        explicit NodeID (uint32 i) : uid (i) {}
+
+        uint32 uid = 0;
+
+        bool operator== (const NodeID& other) const noexcept    { return uid == other.uid; }
+        bool operator!= (const NodeID& other) const noexcept    { return uid != other.uid; }
+        bool operator<  (const NodeID& other) const noexcept    { return uid <  other.uid; }
+    };
+
+    //==============================================================================
+    /** A special index that represents the midi channel of a node.
+
+        This is used as a channel index value if you want to refer to the midi input
+        or output instead of an audio channel.
+    */
+    enum { midiChannelIndex = 0x1000 };
+
+    //==============================================================================
+    /**
+        Represents an input or output channel of a node in an AudioProcessorGraph.
+    */
+    struct NodeAndChannel
+    {
+        NodeID nodeID;
+        int channelIndex;
+
+        bool isMIDI() const noexcept                                    { return channelIndex == midiChannelIndex; }
+
+        bool operator== (const NodeAndChannel& other) const noexcept    { return nodeID == other.nodeID && channelIndex == other.channelIndex; }
+        bool operator!= (const NodeAndChannel& other) const noexcept    { return ! operator== (other); }
+    };
 
     //==============================================================================
     /** Represents one of the nodes, or processors, in an AudioProcessorGraph.
@@ -66,10 +104,10 @@ public:
         /** The ID number assigned to this node.
             This is assigned by the graph that owns it, and can't be changed.
         */
-        const uint32 nodeId;
+        const NodeID nodeID;
 
         /** The actual processor object that this node represents. */
-        AudioProcessor* getProcessor() const noexcept           { return processor; }
+        AudioProcessor* getProcessor() const noexcept           { return processor.get(); }
 
         /** A set of user-definable properties that are associated with this node.
 
@@ -79,23 +117,58 @@ public:
         */
         NamedValueSet properties;
 
+        //==============================================================================
+        /** Returns if the node is bypassed or not. */
+        bool isBypassed() const noexcept;
+
+        /** Tell this node to bypass processing. */
+        void setBypassed (bool shouldBeBypassed) noexcept;
+
         //==============================================================================
         /** A convenient typedef for referring to a pointer to a node object. */
-        typedef ReferenceCountedObjectPtr<Node> Ptr;
+        using Ptr = ReferenceCountedObjectPtr<Node>;
 
     private:
         //==============================================================================
         friend class AudioProcessorGraph;
+        template <typename Float>
+        friend struct GraphRenderSequence;
 
-        const ScopedPointer<AudioProcessor> processor;
-        bool isPrepared;
+        struct Connection
+        {
+            Node* otherNode;
+            int otherChannel, thisChannel;
 
-        Node (uint32 nodeId, AudioProcessor*) noexcept;
+            bool operator== (const Connection&) const noexcept;
+        };
+
+        std::unique_ptr<AudioProcessor> processor;
+        Array<Connection> inputs, outputs;
+        bool isPrepared = false;
+        std::atomic<bool> bypassed { false };
+
+        Node (NodeID, std::unique_ptr<AudioProcessor>) noexcept;
 
         void setParentGraph (AudioProcessorGraph*) const;
         void prepare (double newSampleRate, int newBlockSize, AudioProcessorGraph*, ProcessingPrecision);
         void unprepare();
 
+        template <typename Sample>
+        void processBlock (AudioBuffer<Sample>& audio, MidiBuffer& midi)
+        {
+            const ScopedLock lock (processorLock);
+            processor->processBlock (audio, midi);
+        }
+
+        template <typename Sample>
+        void processBlockBypassed (AudioBuffer<Sample>& audio, MidiBuffer& midi)
+        {
+            const ScopedLock lock (processorLock);
+            processor->processBlockBypassed (audio, midi);
+        }
+
+        CriticalSection processorLock;
+
         JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Node)
     };
 
@@ -107,41 +180,22 @@ public:
     struct JUCE_API  Connection
     {
         //==============================================================================
-        Connection (uint32 sourceNodeId, int sourceChannelIndex,
-                    uint32 destNodeId, int destChannelIndex) noexcept;
+        Connection() = default;
+        Connection (NodeAndChannel source, NodeAndChannel destination) noexcept;
+
+        Connection (const Connection&) = default;
+        Connection& operator= (const Connection&) = default;
+
+        bool operator== (const Connection&) const noexcept;
+        bool operator!= (const Connection&) const noexcept;
+        bool operator<  (const Connection&) const noexcept;
 
         //==============================================================================
-        /** The ID number of the node which is the input source for this connection.
-            @see AudioProcessorGraph::getNodeForId
-        */
-        uint32 sourceNodeId;
+        /** The channel and node which is the input source for this connection. */
+        NodeAndChannel source { {}, 0 };
 
-        /** The index of the output channel of the source node from which this
-            connection takes its data.
-
-            If this value is the special number AudioProcessorGraph::midiChannelIndex, then
-            it is referring to the source node's midi output. Otherwise, it is the zero-based
-            index of an audio output channel in the source node.
-        */
-        int sourceChannelIndex;
-
-        /** The ID number of the node which is the destination for this connection.
-            @see AudioProcessorGraph::getNodeForId
-        */
-        uint32 destNodeId;
-
-        /** The index of the input channel of the destination node to which this
-            connection delivers its data.
-
-            If this value is the special number AudioProcessorGraph::midiChannelIndex, then
-            it is referring to the destination node's midi input. Otherwise, it is the zero-based
-            index of an audio input channel in the destination node.
-        */
-        int destChannelIndex;
-
-    private:
-        //==============================================================================
-        JUCE_LEAK_DETECTOR (Connection)
+        /** The channel and node which is the input source for this connection. */
+        NodeAndChannel destination { {}, 0 };
     };
 
     //==============================================================================
@@ -150,6 +204,9 @@ public:
     */
     void clear();
 
+    /** Returns the array of nodes in the graph. */
+    const ReferenceCountedArray<Node>& getNodes() const noexcept    { return nodes; }
+
     /** Returns the number of nodes in the graph. */
     int getNumNodes() const noexcept                                { return nodes.size(); }
 
@@ -157,13 +214,13 @@ public:
         This will return nullptr if the index is out of range.
         @see getNodeForId
     */
-    Node* getNode (const int index) const noexcept                  { return nodes [index]; }
+    Node::Ptr getNode (int index) const noexcept                    { return nodes[index]; }
 
     /** Searches the graph for a node with the given ID number and returns it.
         If no such node was found, this returns nullptr.
         @see getNode
     */
-    Node* getNodeForId (const uint32 nodeId) const;
+    Node* getNodeForId (NodeID) const;
 
     /** Adds a node to the graph.
 
@@ -171,74 +228,61 @@ public:
         added a processor to the graph, the graph owns it and will delete it later when
         it is no longer needed.
 
-        The optional nodeId parameter lets you specify an ID to use for the node, but
-        if the value is already in use, this new node will overwrite the old one.
+        The optional nodeId parameter lets you specify a unique ID to use for the node.
+        If the value is already in use, this method will fail and return an empty node.
 
         If this succeeds, it returns a pointer to the newly-created node.
     */
-    Node* addNode (AudioProcessor* newProcessor, uint32 nodeId = 0);
+    Node::Ptr addNode (std::unique_ptr<AudioProcessor> newProcessor, NodeID nodeId = {});
 
     /** Deletes a node within the graph which has the specified ID.
         This will also delete any connections that are attached to this node.
     */
-    bool removeNode (uint32 nodeId);
+    Node::Ptr removeNode (NodeID);
 
     /** Deletes a node within the graph.
         This will also delete any connections that are attached to this node.
     */
-    bool removeNode (Node* node);
+    Node::Ptr removeNode (Node*);
 
-    //==============================================================================
-    /** Returns the number of connections in the graph. */
-    int getNumConnections() const                                       { return connections.size(); }
+    /** Returns the list of connections in the graph. */
+    std::vector<Connection> getConnections() const;
 
-    /** Returns a pointer to one of the connections in the graph. */
-    const Connection* getConnection (int index) const                   { return connections [index]; }
+    /** Returns true if the given connection exists. */
+    bool isConnected (const Connection&) const noexcept;
 
-    /** Searches for a connection between some specified channels.
-        If no such connection is found, this returns nullptr.
-    */
-    const Connection* getConnectionBetween (uint32 sourceNodeId,
-                                            int sourceChannelIndex,
-                                            uint32 destNodeId,
-                                            int destChannelIndex) const;
-
-    /** Returns true if there is a connection between any of the channels of
+    /** Returns true if there is a direct connection between any of the channels of
         two specified nodes.
     */
-    bool isConnected (uint32 possibleSourceNodeId,
-                      uint32 possibleDestNodeId) const;
+    bool isConnected (NodeID possibleSourceNodeID, NodeID possibleDestNodeID) const noexcept;
+
+    /** Does a recursive check to see if there's a direct or indirect series of connections
+        between these two nodes.
+    */
+    bool isAnInputTo (Node& source, Node& destination) const noexcept;
 
     /** Returns true if it would be legal to connect the specified points. */
-    bool canConnect (uint32 sourceNodeId, int sourceChannelIndex,
-                     uint32 destNodeId, int destChannelIndex) const;
+    bool canConnect (const Connection&) const;
 
     /** Attempts to connect two specified channels of two nodes.
 
         If this isn't allowed (e.g. because you're trying to connect a midi channel
         to an audio one or other such nonsense), then it'll return false.
     */
-    bool addConnection (uint32 sourceNodeId, int sourceChannelIndex,
-                        uint32 destNodeId, int destChannelIndex);
+    bool addConnection (const Connection&);
 
-    /** Deletes the connection with the specified index. */
-    void removeConnection (int index);
-
-    /** Deletes any connection between two specified points.
-        Returns true if a connection was actually deleted.
-    */
-    bool removeConnection (uint32 sourceNodeId, int sourceChannelIndex,
-                           uint32 destNodeId, int destChannelIndex);
+    /** Deletes the given connection. */
+    bool removeConnection (const Connection&);
 
     /** Removes all connections from the specified node. */
-    bool disconnectNode (uint32 nodeId);
+    bool disconnectNode (NodeID);
 
     /** Returns true if the given connection's channel numbers map on to valid
         channels at each end.
         Even if a connection is valid when created, its status could change if
         a node changes its channel config.
     */
-    bool isConnectionLegal (const Connection* connection) const;
+    bool isConnectionLegal (const Connection&) const;
 
     /** Performs a sanity checks of all the connections.
 
@@ -247,15 +291,6 @@ public:
     */
     bool removeIllegalConnections();
 
-    //==============================================================================
-    /** A special number that represents the midi channel of a node.
-
-        This is used as a channel index value if you want to refer to the midi input
-        or output instead of an audio channel.
-    */
-    static const int midiChannelIndex;
-
-
     //==============================================================================
     /** A special type of AudioProcessor that can live inside an AudioProcessorGraph
         in order to use the audio that comes into and out of the graph itself.
@@ -305,8 +340,8 @@ public:
         bool isOutput() const noexcept;
 
         //==============================================================================
-        AudioGraphIOProcessor (const IODeviceType type);
-        ~AudioGraphIOProcessor();
+        AudioGraphIOProcessor (IODeviceType);
+        ~AudioGraphIOProcessor() override;
 
         const String getName() const override;
         void fillInPluginDescription (PluginDescription&) const override;
@@ -339,11 +374,7 @@ public:
 
     private:
         const IODeviceType type;
-        AudioProcessorGraph* graph;
-
-        //==============================================================================
-        template <typename floatType>
-        void processAudio (AudioBuffer<floatType>& buffer, MidiBuffer& midiMessages);
+        AudioProcessorGraph* graph = nullptr;
 
         JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (AudioGraphIOProcessor)
     };
@@ -358,7 +389,6 @@ public:
 
     void reset() override;
     void setNonRealtime (bool) noexcept override;
-    void setPlayHead (AudioPlayHead*) override;
 
     double getTailLengthSeconds() const override;
     bool acceptsMidi() const override;
@@ -377,33 +407,29 @@ public:
     void setStateInformation (const void* data, int sizeInBytes) override;
 
 private:
-    //==============================================================================
-    template <typename floatType>
-    void processAudio (AudioBuffer<floatType>& buffer, MidiBuffer& midiMessages);
-
-    template <typename floatType>
-    void sliceAndProcess (AudioBuffer<floatType>& buffer, MidiBuffer& midiMessages);
-
     //==============================================================================
     ReferenceCountedArray<Node> nodes;
-    OwnedArray<Connection> connections;
-    uint32 lastNodeId;
-    OwnedArray<MidiBuffer> midiBuffers;
-    Array<void*> renderingOps;
+    NodeID lastNodeID = {};
+
+    struct RenderSequenceFloat;
+    struct RenderSequenceDouble;
+    std::unique_ptr<RenderSequenceFloat> renderSequenceFloat;
+    std::unique_ptr<RenderSequenceDouble> renderSequenceDouble;
 
     friend class AudioGraphIOProcessor;
-    struct AudioProcessorGraphBufferHelpers;
-    ScopedPointer<AudioProcessorGraphBufferHelpers> audioBuffers;
 
-    MidiBuffer* currentMidiInputBuffer;
-    MidiBuffer currentMidiOutputBuffer;
-
-    bool isPrepared;
+    std::atomic<bool> isPrepared { false };
 
+    void topologyChanged();
     void handleAsyncUpdate() override;
     void clearRenderingSequence();
     void buildRenderingSequence();
-    bool isAnInputTo (uint32 possibleInputId, uint32 possibleDestinationId, int recursionCheck) const;
+    bool anyNodesNeedPreparing() const noexcept;
+    bool isConnected (Node* src, int sourceChannel, Node* dest, int destChannel) const noexcept;
+    bool isAnInputTo (Node& src, Node& dst, int recursionCheck) const noexcept;
+    bool canConnect (Node* src, int sourceChannel, Node* dest, int destChannel) const noexcept;
+    bool isLegal (Node* src, int sourceChannel, Node* dest, int destChannel) const noexcept;
+    static void getNodeConnections (Node&, std::vector<Connection>&);
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (AudioProcessorGraph)
 };
