@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -31,11 +31,13 @@ void MACAddress::findAllAddresses (Array<MACAddress>& result)
     {
         for (const ifaddrs* cursor = addrs; cursor != nullptr; cursor = cursor->ifa_next)
         {
-            auto sto = (sockaddr_storage*) cursor->ifa_addr;
+            // Required to avoid misaligned pointer access
+            sockaddr sto;
+            std::memcpy (&sto, cursor->ifa_addr, sizeof (sockaddr));
 
-            if (sto->ss_family == AF_LINK)
+            if (sto.sa_family == AF_LINK)
             {
-                auto sadd = (const sockaddr_dl*) cursor->ifa_addr;
+                auto sadd = reinterpret_cast<const sockaddr_dl*> (cursor->ifa_addr);
 
                #ifndef IFT_ETHER
                 enum { IFT_ETHER = 6 };
@@ -108,7 +110,7 @@ bool JUCE_CALLTYPE Process::openEmailWithAttachments (const String& targetEmailA
 
 //==============================================================================
 // Unfortunately, we need to have this ugly ifdef here as long as some older OS X versions do not support NSURLSession
-#if JUCE_IOS || (defined (__MAC_OS_X_VERSION_MIN_REQUIRED) && defined (__MAC_10_10) && __MAC_OS_X_VERSION_MIN_REQUIRED >= __MAC_10_10)
+#if JUCE_IOS || (defined (MAC_OS_X_VERSION_10_10) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_10)
 
 //==============================================================================
 class URLConnectionState   : private Thread
@@ -125,7 +127,7 @@ public:
         DelegateClass::setState (delegate, this);
     }
 
-    ~URLConnectionState()
+    ~URLConnectionState() override
     {
         signalThreadShouldExit();
 
@@ -190,11 +192,11 @@ public:
 
         while (numBytes > 0)
         {
-            const int available = jmin (numBytes, (int) [data length]);
+            const ScopedLock sl (dataLock);
+            auto available = jmin (numBytes, (int) [data length]);
 
             if (available > 0)
             {
-                const ScopedLock sl (dataLock);
                 [data getBytes: dest length: (NSUInteger) available];
                 [data replaceBytesInRange: NSMakeRange (0, (NSUInteger) available) withBytes: nil length: 0];
 
@@ -207,6 +209,7 @@ public:
                 if (hasFailed || hasFinished)
                     break;
 
+                const ScopedUnlock ul (dataLock);
                 Thread::sleep (1);
             }
         }
@@ -329,7 +332,8 @@ public:
     NSMutableData* data = nil;
     NSDictionary* headers = nil;
     int statusCode = 0;
-    bool initialised = false, hasFailed = false, hasFinished = false, isBeingDeleted = false;
+    std::atomic<bool> initialised { false }, hasFailed { false }, hasFinished { false };
+    bool isBeingDeleted = false;
     const int numRedirectsToFollow;
     int numRedirects = 0;
     int64 latestTotalBytes = 0;
@@ -411,9 +415,10 @@ struct BackgroundDownloadTask  : public URL::DownloadTask
                             String extraHeadersToUse,
                             URL::DownloadTask::Listener* listenerToUse,
                             bool shouldUsePostRequest)
-         : targetLocation (targetLocationToUse), listener (listenerToUse),
+         : listener (listenerToUse),
            uniqueIdentifier (String (urlToUse.toString (true).hashCode64()) + String (Random().nextInt64()))
     {
+        targetLocation = targetLocationToUse;
         downloaded = -1;
 
         static DelegateClass cls;
@@ -421,7 +426,8 @@ struct BackgroundDownloadTask  : public URL::DownloadTask
         DelegateClass::setState (delegate, this);
 
         activeSessions.set (uniqueIdentifier, this);
-        NSMutableURLRequest* request = [[NSMutableURLRequest alloc] initWithURL:[NSURL URLWithString:juceStringToNS (urlToUse.toString (true))]];
+        auto nsUrl = [NSURL URLWithString: juceStringToNS (urlToUse.toString (true))];
+        NSMutableURLRequest* request = [[NSMutableURLRequest alloc] initWithURL: nsUrl];
 
         if (shouldUsePostRequest)
             [request setHTTPMethod: @"POST"];
@@ -447,6 +453,9 @@ struct BackgroundDownloadTask  : public URL::DownloadTask
         if (session != nullptr)
             downloadTask = [session downloadTaskWithRequest:request];
 
+        // Workaround for an Apple bug. See https://github.com/AFNetworking/AFNetworking/issues/2334
+        [request HTTPBody];
+
         [request release];
     }
 
@@ -483,7 +492,6 @@ struct BackgroundDownloadTask  : public URL::DownloadTask
     }
 
     //==============================================================================
-    File targetLocation;
     URL::DownloadTask::Listener* listener;
     NSObject<NSURLSessionDelegate>* delegate = nil;
     NSURLSession* session = nil;
@@ -593,9 +601,9 @@ struct BackgroundDownloadTask  : public URL::DownloadTask
     }
 
     //==============================================================================
-    struct DelegateClass  : public ObjCClass<NSObject<NSURLSessionDelegate> >
+    struct DelegateClass  : public ObjCClass<NSObject<NSURLSessionDelegate>>
     {
-        DelegateClass()  : ObjCClass<NSObject<NSURLSessionDelegate> > ("JUCE_URLDelegate_")
+        DelegateClass()  : ObjCClass<NSObject<NSURLSessionDelegate>> ("JUCE_URLDelegate_")
         {
             addIvar<BackgroundDownloadTask*> ("state");
 
@@ -640,12 +648,12 @@ struct BackgroundDownloadTask  : public URL::DownloadTask
 
 HashMap<String, BackgroundDownloadTask*, DefaultHashFunctions, CriticalSection> BackgroundDownloadTask::activeSessions;
 
-URL::DownloadTask* URL::downloadToFile (const File& targetLocation, String extraHeaders, DownloadTask::Listener* listener, bool usePostRequest)
+std::unique_ptr<URL::DownloadTask> URL::downloadToFile (const File& targetLocation, String extraHeaders, DownloadTask::Listener* listener, bool usePostRequest)
 {
-    ScopedPointer<BackgroundDownloadTask> downloadTask = new BackgroundDownloadTask (*this, targetLocation, extraHeaders, listener, usePostRequest);
+    std::unique_ptr<BackgroundDownloadTask> downloadTask (new BackgroundDownloadTask (*this, targetLocation, extraHeaders, listener, usePostRequest));
 
     if (downloadTask->initOK() && downloadTask->connect())
-        return downloadTask.release();
+        return downloadTask;
 
     return nullptr;
 }
@@ -655,7 +663,7 @@ void URL::DownloadTask::juce_iosURLSessionNotify (const String& identifier)
     BackgroundDownloadTask::invokeNotify (identifier);
 }
 #else
-URL::DownloadTask* URL::downloadToFile (const File& targetLocation, String extraHeaders, DownloadTask::Listener* listener, bool usePost)
+std::unique_ptr<URL::DownloadTask> URL::downloadToFile (const File& targetLocation, String extraHeaders, DownloadTask::Listener* listener, bool usePost)
 {
     return URL::DownloadTask::createFallbackDownloader (*this, targetLocation, extraHeaders, listener, usePost);
 }
@@ -668,8 +676,7 @@ URL::DownloadTask* URL::downloadToFile (const File& targetLocation, String extra
 // so we'll turn off deprecation warnings. This code will be removed at some point
 // in the future.
 
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wdeprecated"
+JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wdeprecated")
 
 //==============================================================================
 class URLConnectionState   : public Thread
@@ -686,7 +693,7 @@ public:
         DelegateClass::setState (delegate, this);
     }
 
-    ~URLConnectionState()
+    ~URLConnectionState() override
     {
         stop();
 
@@ -740,11 +747,11 @@ public:
 
         while (numBytes > 0)
         {
-            const int available = jmin (numBytes, (int) [data length]);
+            const ScopedLock sl (dataLock);
+            auto available = jmin (numBytes, (int) [data length]);
 
             if (available > 0)
             {
-                const ScopedLock sl (dataLock);
                 [data getBytes: dest length: (NSUInteger) available];
                 [data replaceBytesInRange: NSMakeRange (0, (NSUInteger) available) withBytes: nil length: 0];
 
@@ -757,6 +764,7 @@ public:
                 if (hasFailed || hasFinished)
                     break;
 
+                const ScopedUnlock sul (dataLock);
                 Thread::sleep (1);
             }
         }
@@ -803,7 +811,8 @@ public:
     {
         DBG (nsStringToJuce ([error description])); ignoreUnused (error);
         nsUrlErrorCode = [error code];
-        hasFailed = initialised = true;
+        hasFailed = true;
+        initialised = true;
         signalThreadShouldExit();
     }
 
@@ -821,7 +830,8 @@ public:
 
     void finishedLoading()
     {
-        hasFinished = initialised = true;
+        hasFinished = true;
+        initialised = true;
         signalThreadShouldExit();
     }
 
@@ -855,7 +865,7 @@ public:
     NSDictionary* headers = nil;
     NSInteger nsUrlErrorCode = 0;
     int statusCode = 0;
-    bool initialised = false, hasFailed = false, hasFinished = false;
+    std::atomic<bool> initialised { false }, hasFailed { false }, hasFinished { false };
     const int numRedirectsToFollow;
     int numRedirects = 0;
     int latestTotalBytes = 0;
@@ -919,12 +929,12 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (URLConnectionState)
 };
 
-URL::DownloadTask* URL::downloadToFile (const File& targetLocation, String extraHeaders, DownloadTask::Listener* listener, bool shouldUsePost)
+std::unique_ptr<URL::DownloadTask> URL::downloadToFile (const File& targetLocation, String extraHeaders, DownloadTask::Listener* listener, bool shouldUsePost)
 {
     return URL::DownloadTask::createFallbackDownloader (*this, targetLocation, extraHeaders, listener, shouldUsePost);
 }
 
-#pragma clang diagnostic pop
+JUCE_END_IGNORE_WARNINGS_GCC_LIKE
 
 #endif
 
@@ -941,7 +951,7 @@ public:
 
     ~Pimpl()
     {
-        connection = nullptr;
+        connection.reset();
     }
 
     bool connect (WebInputStream::Listener* webInputListener, int numRetries = 0)
@@ -957,22 +967,25 @@ public:
             createConnection();
         }
 
+        if (connection == nullptr)
+            return false;
+
         if (! connection->start (owner, webInputListener))
         {
             // Workaround for deployment targets below 10.10 where HTTPS POST requests with keep-alive fail with the NSURLErrorNetworkConnectionLost error code.
-           #if ! (JUCE_IOS || (defined (__MAC_OS_X_VERSION_MIN_REQUIRED) && defined (__MAC_10_10) && __MAC_OS_X_VERSION_MIN_REQUIRED >= __MAC_10_10))
+           #if ! (JUCE_IOS || (defined (MAC_OS_X_VERSION_10_10) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_10))
             if (numRetries == 0 && connection->nsUrlErrorCode == NSURLErrorNetworkConnectionLost)
             {
-                connection = nullptr;
+                connection.reset();
                 return connect (webInputListener, ++numRetries);
             }
            #endif
 
-            connection = nullptr;
+            connection.reset();
             return false;
         }
 
-        if (connection != nullptr && connection->headers != nil)
+        if (connection->headers != nil)
         {
             statusCode = connection->statusCode;
 
@@ -1055,9 +1068,9 @@ public:
             if (wantedPos < position)
                 return false;
 
-            int64 numBytesToSkip = wantedPos - position;
-            const int skipBufferSize = (int) jmin (numBytesToSkip, (int64) 16384);
-            HeapBlock<char> temp ((size_t) skipBufferSize);
+            auto numBytesToSkip = wantedPos - position;
+            auto skipBufferSize = (int) jmin (numBytesToSkip, (int64) 16384);
+            HeapBlock<char> temp (skipBufferSize);
 
             while (numBytesToSkip > 0 && ! isExhausted())
                 numBytesToSkip -= read (temp, (int) jmin (numBytesToSkip, (int64) skipBufferSize));
@@ -1071,7 +1084,7 @@ public:
 private:
     WebInputStream& owner;
     URL url;
-    ScopedPointer<URLConnectionState> connection;
+    std::unique_ptr<URLConnectionState> connection;
     String headers;
     MemoryBlock postData;
     int64 position = 0;
@@ -1088,35 +1101,44 @@ private:
     {
         jassert (connection == nullptr);
 
-        if (NSMutableURLRequest* req = [NSMutableURLRequest requestWithURL: [NSURL URLWithString: juceStringToNS (url.toString (! isPost))]
-                                                               cachePolicy: NSURLRequestReloadIgnoringLocalCacheData
-                                                           timeoutInterval: timeOutMs <= 0 ? 60.0 : (timeOutMs / 1000.0)])
+        if (NSURL* nsURL = [NSURL URLWithString: juceStringToNS (url.toString (! isPost))])
         {
-            [req setHTTPMethod: [NSString stringWithUTF8String: httpRequestCmd.toRawUTF8()]];
-
-            if (isPost)
+            if (NSMutableURLRequest* req = [NSMutableURLRequest requestWithURL: nsURL
+                                                                   cachePolicy: NSURLRequestReloadIgnoringLocalCacheData
+                                                               timeoutInterval: timeOutMs <= 0 ? 60.0 : (timeOutMs / 1000.0)])
             {
-                WebInputStream::createHeadersAndPostData (url, headers, postData);
+                if (NSString* httpMethod = [NSString stringWithUTF8String: httpRequestCmd.toRawUTF8()])
+                {
+                    [req setHTTPMethod: httpMethod];
 
-                if (postData.getSize() > 0)
-                    [req setHTTPBody: [NSData dataWithBytes: postData.getData()
-                                                     length: postData.getSize()]];
+                    if (isPost)
+                    {
+                        WebInputStream::createHeadersAndPostData (url, headers, postData);
+
+                        if (postData.getSize() > 0)
+                            [req setHTTPBody: [NSData dataWithBytes: postData.getData()
+                                                             length: postData.getSize()]];
+                    }
+
+                    StringArray headerLines;
+                    headerLines.addLines (headers);
+                    headerLines.removeEmptyStrings (true);
+
+                    for (int i = 0; i < headerLines.size(); ++i)
+                    {
+                        auto key   = headerLines[i].upToFirstOccurrenceOf (":", false, false).trim();
+                        auto value = headerLines[i].fromFirstOccurrenceOf (":", false, false).trim();
+
+                        if (key.isNotEmpty() && value.isNotEmpty())
+                            [req addValue: juceStringToNS (value) forHTTPHeaderField: juceStringToNS (key)];
+                    }
+
+                    // Workaround for an Apple bug. See https://github.com/AFNetworking/AFNetworking/issues/2334
+                    [req HTTPBody];
+
+                    connection.reset (new URLConnectionState (req, numRedirectsToFollow));
+                }
             }
-
-            StringArray headerLines;
-            headerLines.addLines (headers);
-            headerLines.removeEmptyStrings (true);
-
-            for (int i = 0; i < headerLines.size(); ++i)
-            {
-                String key   = headerLines[i].upToFirstOccurrenceOf (":", false, false).trim();
-                String value = headerLines[i].fromFirstOccurrenceOf (":", false, false).trim();
-
-                if (key.isNotEmpty() && value.isNotEmpty())
-                    [req addValue: juceStringToNS (value) forHTTPHeaderField: juceStringToNS (key)];
-            }
-
-            connection = new URLConnectionState (req, numRedirectsToFollow);
         }
     }
 
