@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -88,9 +88,9 @@ public:
 
                 for (;;)
                 {
-                    HeapBlock<char> buffer ((size_t) bufferSizeBytes);
+                    HeapBlock<char> buffer (bufferSizeBytes);
 
-                    if (HttpQueryInfo (request, HTTP_QUERY_RAW_HEADERS_CRLF, buffer.getData(), &bufferSizeBytes, 0))
+                    if (HttpQueryInfo (request, HTTP_QUERY_RAW_HEADERS_CRLF, buffer.getData(), &bufferSizeBytes, nullptr))
                     {
                         StringArray headersArray;
                         headersArray.addLines (String (reinterpret_cast<const WCHAR*> (buffer.getData())));
@@ -116,7 +116,7 @@ public:
                 DWORD status = 0;
                 DWORD statusSize = sizeof (status);
 
-                if (HttpQueryInfo (request, HTTP_QUERY_STATUS_CODE | HTTP_QUERY_FLAG_NUMBER, &status, &statusSize, 0))
+                if (HttpQueryInfo (request, HTTP_QUERY_STATUS_CODE | HTTP_QUERY_FLAG_NUMBER, &status, &statusSize, nullptr))
                 {
                     statusCode = (int) status;
 
@@ -151,10 +151,10 @@ public:
             break;
         }
 
-        return (request != 0);
+        return (request != nullptr);
     }
 
-    bool isError() const        { return request == 0; }
+    bool isError() const        { return request == nullptr; }
     bool isExhausted()          { return finished; }
     int64 getPosition()         { return position; }
 
@@ -207,7 +207,7 @@ public:
         if (wantedPos != position)
         {
             finished = false;
-            position = (int64) InternetSetFilePointer (request, (LONG) wantedPos, 0, FILE_BEGIN, 0);
+            position = (int64) InternetSetFilePointer (request, (LONG) wantedPos, nullptr, FILE_BEGIN, 0);
 
             if (position == wantedPos)
                 return true;
@@ -216,8 +216,8 @@ public:
                 return false;
 
             int64 numBytesToSkip = wantedPos - position;
-            const int skipBufferSize = (int) jmin (numBytesToSkip, (int64) 16384);
-            HeapBlock<char> temp ((size_t) skipBufferSize);
+            auto skipBufferSize = (int) jmin (numBytesToSkip, (int64) 16384);
+            HeapBlock<char> temp (skipBufferSize);
 
             while (numBytesToSkip > 0 && ! isExhausted())
                 numBytesToSkip -= read (temp, (int) jmin (numBytesToSkip, (int64) skipBufferSize));
@@ -232,7 +232,7 @@ private:
     //==============================================================================
     WebInputStream& owner;
     const URL url;
-    HINTERNET connection = 0, request = 0;
+    HINTERNET connection = nullptr, request = nullptr;
     String headers;
     MemoryBlock postData;
     int64 position = 0;
@@ -249,25 +249,25 @@ private:
     {
         HINTERNET requestCopy = request;
 
-        request = 0;
+        request = nullptr;
 
-        if (requestCopy != 0)
+        if (requestCopy != nullptr)
             InternetCloseHandle (requestCopy);
 
-        if (connection != 0)
+        if (connection != nullptr)
         {
             InternetCloseHandle (connection);
-            connection = 0;
+            connection = nullptr;
         }
     }
 
     void createConnection (const String& address, WebInputStream::Listener* listener)
     {
-        static HINTERNET sessionHandle = InternetOpen (_T("juce"), INTERNET_OPEN_TYPE_PRECONFIG, 0, 0, 0);
+        static HINTERNET sessionHandle = InternetOpen (_T("juce"), INTERNET_OPEN_TYPE_PRECONFIG, nullptr, nullptr, 0);
 
         closeConnection();
 
-        if (sessionHandle != 0)
+        if (sessionHandle != nullptr)
         {
             // break up the url..
             const int fileNumChars = 65536;
@@ -277,7 +277,7 @@ private:
             HeapBlock<TCHAR> file (fileNumChars), server (serverNumChars),
                              username (usernameNumChars), password (passwordNumChars);
 
-            URL_COMPONENTS uc = { 0 };
+            URL_COMPONENTS uc = {};
             uc.dwStructSize = sizeof (uc);
             uc.lpszUrlPath = file;
             uc.dwUrlPathLength = fileNumChars;
@@ -319,7 +319,7 @@ private:
         {
             const ScopedLock lock (createConnectionLock);
 
-            connection = hasBeenCancelled ? 0
+            connection = hasBeenCancelled ? nullptr
                                           : InternetConnect (sessionHandle,
                                                              uc.lpszHostName, uc.nPort,
                                                              uc.lpszUserName, uc.lpszPassword,
@@ -328,7 +328,7 @@ private:
                                                              0, 0);
         }
 
-        if (connection != 0)
+        if (connection != nullptr)
         {
             if (isFtp)
                 request = FtpOpenFile (connection, uc.lpszUrlPath, GENERIC_READ,
@@ -343,6 +343,35 @@ private:
         InternetSetOption (sessionHandle, option, &timeOutMs, sizeof (timeOutMs));
     }
 
+    void sendHTTPRequest (INTERNET_BUFFERS& buffers, WebInputStream::Listener* listener)
+    {
+        if (! HttpSendRequestEx (request, &buffers, nullptr, HSR_INITIATE, 0))
+            return;
+
+        int totalBytesSent = 0;
+
+        while (totalBytesSent < (int) postData.getSize())
+        {
+            auto bytesToSend = jmin (1024, (int) postData.getSize() - totalBytesSent);
+            DWORD bytesSent = 0;
+
+            if (bytesToSend == 0
+                || ! InternetWriteFile (request, static_cast<const char*> (postData.getData()) + totalBytesSent,
+                                        (DWORD) bytesToSend, &bytesSent))
+            {
+                return;
+            }
+
+            totalBytesSent += (int) bytesSent;
+
+            if (listener != nullptr
+                && ! listener->postDataSendProgress (owner, totalBytesSent, (int) postData.getSize()))
+            {
+                return;
+            }
+        }
+    }
+
     void openHTTPConnection (URL_COMPONENTS& uc, const String& address, WebInputStream::Listener* listener)
     {
         const TCHAR* mimeTypes[] = { _T("*/*"), nullptr };
@@ -357,51 +386,37 @@ private:
         {
             const ScopedLock lock (createConnectionLock);
 
-            request = hasBeenCancelled ? 0
+            request = hasBeenCancelled ? nullptr
                                        : HttpOpenRequest (connection, httpRequestCmd.toWideCharPointer(),
-                                                          uc.lpszUrlPath, 0, 0, mimeTypes, flags, 0);
+                                                          uc.lpszUrlPath, nullptr, nullptr, mimeTypes, flags, 0);
         }
 
-        if (request != 0)
+        if (request != nullptr)
         {
-            INTERNET_BUFFERS buffers = { 0 };
-            buffers.dwStructSize = sizeof (INTERNET_BUFFERS);
-            buffers.lpcszHeader = headers.toWideCharPointer();
+            INTERNET_BUFFERS buffers = {};
+            buffers.dwStructSize    = sizeof (INTERNET_BUFFERS);
+            buffers.lpcszHeader     = headers.toWideCharPointer();
             buffers.dwHeadersLength = (DWORD) headers.length();
-            buffers.dwBufferTotal = (DWORD) postData.getSize();
+            buffers.dwBufferTotal   = (DWORD) postData.getSize();
 
-            if (HttpSendRequestEx (request, &buffers, 0, HSR_INITIATE, 0))
+            auto sendRequestAndTryEnd = [this, &buffers, &listener]() -> bool
             {
-                int bytesSent = 0;
+                sendHTTPRequest (buffers, listener);
 
-                for (;;)
-                {
-                    const int bytesToDo = jmin (1024, (int) postData.getSize() - bytesSent);
-                    DWORD bytesDone = 0;
+                if (HttpEndRequest (request, nullptr, 0, 0))
+                    return true;
 
-                    if (bytesToDo > 0
-                         && ! InternetWriteFile (request,
-                                                 static_cast<const char*> (postData.getData()) + bytesSent,
-                                                 (DWORD) bytesToDo, &bytesDone))
-                    {
-                        break;
-                    }
+                return false;
+            };
 
-                    if (bytesToDo == 0 || (int) bytesDone < bytesToDo)
-                    {
-                        if (HttpEndRequest (request, 0, 0, 0))
-                            return;
+            auto closed = sendRequestAndTryEnd();
 
-                        break;
-                    }
+            // N.B. this is needed for some authenticated HTTP connections
+            if (! closed && GetLastError() == ERROR_INTERNET_FORCE_RETRY)
+                closed = sendRequestAndTryEnd();
 
-                    bytesSent += bytesDone;
-
-                    if (listener != nullptr
-                          && ! listener->postDataSendProgress (owner, bytesSent, (int) postData.getSize()))
-                        break;
-                }
-            }
+            if (closed)
+                return;
         }
 
         closeConnection();
@@ -425,10 +440,10 @@ struct GetAdaptersAddressesHelper
         adaptersAddresses.malloc (1);
         ULONG len = sizeof (IP_ADAPTER_ADDRESSES);
 
-        if (getAdaptersAddresses (AF_UNSPEC, GAA_FLAG_INCLUDE_PREFIX, NULL, adaptersAddresses, &len) == ERROR_BUFFER_OVERFLOW)
+        if (getAdaptersAddresses (AF_UNSPEC, GAA_FLAG_INCLUDE_PREFIX, nullptr, adaptersAddresses, &len) == ERROR_BUFFER_OVERFLOW)
             adaptersAddresses.malloc (len, 1);
 
-        return getAdaptersAddresses (AF_UNSPEC, GAA_FLAG_INCLUDE_PREFIX, NULL, adaptersAddresses, &len) == NO_ERROR;
+        return getAdaptersAddresses (AF_UNSPEC, GAA_FLAG_INCLUDE_PREFIX, nullptr, adaptersAddresses, &len) == NO_ERROR;
     }
 
     HeapBlock<IP_ADAPTER_ADDRESSES> adaptersAddresses;
@@ -461,12 +476,12 @@ namespace MACAddressHelpers
         DynamicLibrary dll ("netapi32.dll");
         JUCE_LOAD_WINAPI_FUNCTION (dll, Netbios, NetbiosCall, UCHAR, (PNCB))
 
-        if (NetbiosCall != 0)
+        if (NetbiosCall != nullptr)
         {
-            LANA_ENUM enums = { 0 };
+            LANA_ENUM enums = {};
 
             {
-                NCB ncb = { 0 };
+                NCB ncb = {};
                 ncb.ncb_command = NCBENUM;
                 ncb.ncb_buffer = (unsigned char*) &enums;
                 ncb.ncb_length = sizeof (LANA_ENUM);
@@ -475,13 +490,13 @@ namespace MACAddressHelpers
 
             for (int i = 0; i < enums.length; ++i)
             {
-                NCB ncb2 = { 0 };
+                NCB ncb2 = {};
                 ncb2.ncb_command = NCBRESET;
                 ncb2.ncb_lana_num = enums.lana[i];
 
                 if (NetbiosCall (&ncb2) == 0)
                 {
-                    NCB ncb = { 0 };
+                    NCB ncb = {};
                     memcpy (ncb.ncb_callname, "*                   ", NCBNAMSZ);
                     ncb.ncb_command = NCBASTAT;
                     ncb.ncb_lana_num = enums.lana[i];
@@ -514,6 +529,37 @@ namespace MACAddressHelpers
         split[1] = sa_in6->sin6_addr.u.Byte[off];
        #endif
     }
+
+    static IPAddress createAddress (const sockaddr_in6* sa_in6)
+    {
+        IPAddressByteUnion temp;
+        uint16 arr[8];
+
+        for (int i = 0; i < 8; ++i)
+        {
+            split (sa_in6, i * 2, temp.split);
+            arr[i] = temp.combined;
+        }
+
+        return IPAddress (arr);
+    }
+
+    static IPAddress createAddress (const sockaddr_in* sa_in)
+    {
+        return IPAddress ((uint8*) &sa_in->sin_addr.s_addr, false);
+    }
+
+    template <typename Type>
+    static void findAddresses (Array<IPAddress>& result, bool includeIPv6, Type start)
+    {
+        for (auto addr = start; addr != nullptr; addr = addr->Next)
+        {
+            if (addr->Address.lpSockaddr->sa_family == AF_INET)
+                result.addIfNotAlreadyThere (createAddress ((sockaddr_in*) addr->Address.lpSockaddr));
+            else if (addr->Address.lpSockaddr->sa_family == AF_INET6 && includeIPv6)
+                result.addIfNotAlreadyThere (createAddress ((sockaddr_in6*) addr->Address.lpSockaddr));
+        }
+    }
 }
 
 void MACAddress::findAllAddresses (Array<MACAddress>& result)
@@ -530,94 +576,25 @@ void IPAddress::findAllAddresses (Array<IPAddress>& result, bool includeIPv6)
         result.addIfNotAlreadyThere (IPAddress::local (true));
 
     GetAdaptersAddressesHelper addressesHelper;
+
     if (addressesHelper.callGetAdaptersAddresses())
     {
         for (PIP_ADAPTER_ADDRESSES adapter = addressesHelper.adaptersAddresses; adapter != nullptr; adapter = adapter->Next)
         {
-            PIP_ADAPTER_UNICAST_ADDRESS pUnicast = nullptr;
-            for (pUnicast = adapter->FirstUnicastAddress; pUnicast != nullptr; pUnicast = pUnicast->Next)
-            {
-                if (pUnicast->Address.lpSockaddr->sa_family == AF_INET)
-                {
-                    const sockaddr_in* sa_in = (sockaddr_in*)pUnicast->Address.lpSockaddr;
-                    IPAddress ip ((uint8*)&sa_in->sin_addr.s_addr, false);
-                    result.addIfNotAlreadyThere (ip);
-                }
-                else if (pUnicast->Address.lpSockaddr->sa_family == AF_INET6 && includeIPv6)
-                {
-                    const sockaddr_in6* sa_in6 = (sockaddr_in6*)pUnicast->Address.lpSockaddr;
-
-                    ByteUnion temp;
-                    uint16 arr[8];
-
-                    for (int i = 0; i < 8; ++i)
-                    {
-                        MACAddressHelpers::split (sa_in6, i * 2, temp.split);
-                        arr[i] = temp.combined;
-                    }
-
-                    IPAddress ip (arr);
-                    result.addIfNotAlreadyThere (ip);
-                }
-            }
-
-            PIP_ADAPTER_ANYCAST_ADDRESS   pAnycast = nullptr;
-            for (pAnycast = adapter->FirstAnycastAddress; pAnycast != nullptr; pAnycast = pAnycast->Next)
-            {
-                if (pAnycast->Address.lpSockaddr->sa_family == AF_INET)
-                {
-                    const sockaddr_in* sa_in = (sockaddr_in*)pAnycast->Address.lpSockaddr;
-                    IPAddress ip ((uint8*)&sa_in->sin_addr.s_addr, false);
-                    result.addIfNotAlreadyThere (ip);
-                }
-                else if (pAnycast->Address.lpSockaddr->sa_family == AF_INET6 && includeIPv6)
-                {
-                    const sockaddr_in6* sa_in6 = (sockaddr_in6*)pAnycast->Address.lpSockaddr;
-
-                    ByteUnion temp;
-                    uint16 arr[8];
-
-                    for (int i = 0; i < 8; ++i)
-                    {
-                        MACAddressHelpers::split (sa_in6, i * 2, temp.split);
-                        arr[i] = temp.combined;
-                    }
-
-                    IPAddress ip (arr);
-                    result.addIfNotAlreadyThere (ip);
-                }
-            }
-
-            PIP_ADAPTER_MULTICAST_ADDRESS pMulticast = nullptr;
-            for (pMulticast = adapter->FirstMulticastAddress; pMulticast != nullptr; pMulticast = pMulticast->Next)
-            {
-                if (pMulticast->Address.lpSockaddr->sa_family == AF_INET)
-                {
-                    const sockaddr_in* sa_in = (sockaddr_in*)pMulticast->Address.lpSockaddr;
-                    IPAddress ip ((uint8*)&sa_in->sin_addr.s_addr, false);
-                    result.addIfNotAlreadyThere (ip);
-                }
-                else if (pMulticast->Address.lpSockaddr->sa_family == AF_INET6 && includeIPv6)
-                {
-                    const sockaddr_in6* sa_in6 = (sockaddr_in6*)pMulticast->Address.lpSockaddr;
-
-                    ByteUnion temp;
-                    uint16 arr[8];
-
-                    for (int i = 0; i < 8; ++i)
-                    {
-                        MACAddressHelpers::split (sa_in6, i * 2, temp.split);
-                        arr[i] = temp.combined;
-                    }
-
-                    IPAddress ip (arr);
-                    result.addIfNotAlreadyThere (ip);
-                }
-            }
+            MACAddressHelpers::findAddresses (result, includeIPv6, adapter->FirstUnicastAddress);
+            MACAddressHelpers::findAddresses (result, includeIPv6, adapter->FirstAnycastAddress);
+            MACAddressHelpers::findAddresses (result, includeIPv6, adapter->FirstMulticastAddress);
         }
     }
 }
 
+IPAddress IPAddress::getInterfaceBroadcastAddress (const IPAddress&)
+{
+    // TODO
+    return {};
+}
+
+
 //==============================================================================
 bool JUCE_CALLTYPE Process::openEmailWithAttachments (const String& targetEmailAddress,
                                                       const String& emailSubject,
@@ -631,11 +608,11 @@ bool JUCE_CALLTYPE Process::openEmailWithAttachments (const String& targetEmailA
     if (mapiSendMail == nullptr)
         return false;
 
-    MapiMessage message = { 0 };
+    MapiMessage message = {};
     message.lpszSubject = (LPSTR) emailSubject.toRawUTF8();
     message.lpszNoteText = (LPSTR) bodyText.toRawUTF8();
 
-    MapiRecipDesc recip = { 0 };
+    MapiRecipDesc recip = {};
     recip.ulRecipClass = MAPI_TO;
     String targetEmailAddress_ (targetEmailAddress);
     if (targetEmailAddress_.isEmpty())
@@ -645,7 +622,7 @@ bool JUCE_CALLTYPE Process::openEmailWithAttachments (const String& targetEmailA
     message.lpRecips = &recip;
 
     HeapBlock<MapiFileDesc> files;
-    files.calloc ((size_t) filesToAttach.size());
+    files.calloc (filesToAttach.size());
 
     message.nFileCount = (ULONG) filesToAttach.size();
     message.lpFiles = files;
@@ -659,7 +636,7 @@ bool JUCE_CALLTYPE Process::openEmailWithAttachments (const String& targetEmailA
     return mapiSendMail (0, 0, &message, MAPI_DIALOG | MAPI_LOGON_UI, 0) == SUCCESS_SUCCESS;
 }
 
-URL::DownloadTask* URL::downloadToFile (const File& targetLocation, String extraHeaders, DownloadTask::Listener* listener, bool shouldUsePost)
+std::unique_ptr<URL::DownloadTask> URL::downloadToFile (const File& targetLocation, String extraHeaders, DownloadTask::Listener* listener, bool shouldUsePost)
 {
     return URL::DownloadTask::createFallbackDownloader (*this, targetLocation, extraHeaders, listener, shouldUsePost);
 }
