@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -26,13 +26,13 @@ namespace juce
 struct FallbackDownloadTask  : public URL::DownloadTask,
                                public Thread
 {
-    FallbackDownloadTask (FileOutputStream* outputStreamToUse,
+    FallbackDownloadTask (std::unique_ptr<FileOutputStream> outputStreamToUse,
                           size_t bufferSizeToUse,
-                          WebInputStream* streamToUse,
+                          std::unique_ptr<WebInputStream> streamToUse,
                           URL::DownloadTask::Listener* listenerToUse)
         : Thread ("DownloadTask thread"),
-          fileStream (outputStreamToUse),
-          stream (streamToUse),
+          fileStream (std::move (outputStreamToUse)),
+          stream (std::move (streamToUse)),
           bufferSize (bufferSizeToUse),
           buffer (bufferSize),
           listener (listenerToUse)
@@ -40,13 +40,14 @@ struct FallbackDownloadTask  : public URL::DownloadTask,
         jassert (fileStream != nullptr);
         jassert (stream != nullptr);
 
-        contentLength = stream->getTotalLength();
-        httpCode      = stream->getStatusCode();
+        targetLocation = fileStream->getFile();
+        contentLength  = stream->getTotalLength();
+        httpCode       = stream->getStatusCode();
 
         startThread();
     }
 
-    ~FallbackDownloadTask()
+    ~FallbackDownloadTask() override
     {
         signalThreadShouldExit();
         stream->cancel();
@@ -61,10 +62,10 @@ struct FallbackDownloadTask  : public URL::DownloadTask,
             if (listener != nullptr)
                 listener->progress (this, downloaded, contentLength);
 
-            const int max = jmin ((int) bufferSize, contentLength < 0 ? std::numeric_limits<int>::max()
-                                                                      : static_cast<int> (contentLength - downloaded));
+            auto max = (int) jmin ((int64) bufferSize, contentLength < 0 ? std::numeric_limits<int64>::max()
+                                                                         : static_cast<int64> (contentLength - downloaded));
 
-            const int actual = stream->read (buffer.get(), max);
+            auto actual = stream->read (buffer.get(), max);
 
             if (actual < 0 || threadShouldExit() || stream->isError())
                 break;
@@ -81,7 +82,7 @@ struct FallbackDownloadTask  : public URL::DownloadTask,
                 break;
         }
 
-        fileStream->flush();
+        fileStream.reset();
 
         if (threadShouldExit() || stream->isError())
             error = true;
@@ -96,8 +97,8 @@ struct FallbackDownloadTask  : public URL::DownloadTask,
     }
 
     //==============================================================================
-    const ScopedPointer<FileOutputStream> fileStream;
-    const ScopedPointer<WebInputStream> stream;
+    std::unique_ptr<FileOutputStream> fileStream;
+    const std::unique_ptr<WebInputStream> stream;
     const size_t bufferSize;
     HeapBlock<char> buffer;
     URL::DownloadTask::Listener* const listener;
@@ -109,22 +110,25 @@ void URL::DownloadTask::Listener::progress (DownloadTask*, int64, int64) {}
 URL::DownloadTask::Listener::~Listener() {}
 
 //==============================================================================
-URL::DownloadTask* URL::DownloadTask::createFallbackDownloader (const URL& urlToUse,
-                                                                const File& targetFileToUse,
-                                                                const String& extraHeadersToUse,
-                                                                Listener* listenerToUse,
-                                                                bool usePostRequest)
+std::unique_ptr<URL::DownloadTask> URL::DownloadTask::createFallbackDownloader (const URL& urlToUse,
+                                                                                const File& targetFileToUse,
+                                                                                const String& extraHeadersToUse,
+                                                                                Listener* listenerToUse,
+                                                                                bool usePostRequest)
 {
     const size_t bufferSize = 0x8000;
     targetFileToUse.deleteFile();
 
-    if (ScopedPointer<FileOutputStream> outputStream = targetFileToUse.createOutputStream (bufferSize))
+    if (auto outputStream = targetFileToUse.createOutputStream (bufferSize))
     {
-        ScopedPointer<WebInputStream> stream = new WebInputStream (urlToUse, usePostRequest);
+        auto stream = std::make_unique<WebInputStream> (urlToUse, usePostRequest);
         stream->withExtraHeaders (extraHeadersToUse);
 
         if (stream->connect (nullptr))
-            return new FallbackDownloadTask (outputStream.release(), bufferSize, stream.release(), listenerToUse);
+            return std::make_unique<FallbackDownloadTask> (std::move (outputStream),
+                                                           bufferSize,
+                                                           std::move (stream),
+                                                           listenerToUse);
     }
 
     return nullptr;
@@ -134,20 +138,57 @@ URL::DownloadTask::DownloadTask() {}
 URL::DownloadTask::~DownloadTask() {}
 
 //==============================================================================
-URL::URL()
-{
-}
+URL::URL() {}
 
 URL::URL (const String& u)  : url (u)
 {
-    int i = url.indexOfChar ('?');
+    init();
+}
+
+URL::URL (File localFile)
+{
+    if (localFile == File())
+        return;
+
+   #if JUCE_WINDOWS
+    bool isUncPath = localFile.getFullPathName().startsWith ("\\\\");
+   #endif
+
+    while (! localFile.isRoot())
+    {
+        url = "/" + addEscapeChars (localFile.getFileName(), false) + url;
+        localFile = localFile.getParentDirectory();
+    }
+
+    url = addEscapeChars (localFile.getFileName(), false) + url;
+
+   #if JUCE_WINDOWS
+    if (isUncPath)
+    {
+        url = url.fromFirstOccurrenceOf ("/", false, false);
+    }
+    else
+   #endif
+    {
+        if (! url.startsWithChar (L'/'))
+            url = "/" + url;
+    }
+
+    url = "file://" + url;
+
+    jassert (isWellFormed());
+}
+
+void URL::init()
+{
+    auto i = url.indexOfChar ('?');
 
     if (i >= 0)
     {
         do
         {
-            const int nextAmp   = url.indexOfChar (i + 1, '&');
-            const int equalsPos = url.indexOfChar (i + 1, '=');
+            auto nextAmp   = url.indexOfChar (i + 1, '&');
+            auto equalsPos = url.indexOfChar (i + 1, '=');
 
             if (nextAmp < 0)
             {
@@ -175,26 +216,6 @@ URL URL::createWithoutParsing (const String& u)
     return URL (u, 0);
 }
 
-URL::URL (const URL& other)
-    : url (other.url),
-      postData (other.postData),
-      parameterNames (other.parameterNames),
-      parameterValues (other.parameterValues),
-      filesToUpload (other.filesToUpload)
-{
-}
-
-URL& URL::operator= (const URL& other)
-{
-    url = other.url;
-    postData = other.postData;
-    parameterNames = other.parameterNames;
-    parameterValues = other.parameterValues;
-    filesToUpload = other.filesToUpload;
-
-    return *this;
-}
-
 bool URL::operator== (const URL& other) const
 {
     return url == other.url
@@ -209,10 +230,6 @@ bool URL::operator!= (const URL& other) const
     return ! operator== (other);
 }
 
-URL::~URL()
-{
-}
-
 namespace URLHelpers
 {
     static String getMangledParameters (const URL& url)
@@ -272,6 +289,20 @@ namespace URLHelpers
         else
             path += suffix;
     }
+
+    static String removeLastPathSection (const String& url)
+    {
+        auto startOfPath = findStartOfPath (url);
+        auto lastSlash = url.lastIndexOfChar ('/');
+
+        if (lastSlash > startOfPath && lastSlash == url.length() - 1)
+            return removeLastPathSection (url.dropLastCharacters (1));
+
+        if (lastSlash < 0)
+            return url;
+
+        return url.substring (0, std::max (startOfPath, lastSlash));
+    }
 }
 
 void URL::addParameter (const String& name, const String& value)
@@ -280,10 +311,10 @@ void URL::addParameter (const String& name, const String& value)
     parameterValues.add (value);
 }
 
-String URL::toString (const bool includeGetParameters) const
+String URL::toString (bool includeGetParameters) const
 {
-    if (includeGetParameters && parameterNames.size() > 0)
-        return url + "?" + URLHelpers::getMangledParameters (*this);
+    if (includeGetParameters)
+        return url + getQueryString();
 
     return url;
 }
@@ -301,22 +332,27 @@ bool URL::isWellFormed() const
 
 String URL::getDomain() const
 {
-    const int start = URLHelpers::findStartOfNetLocation (url);
-    const int end1 = url.indexOfChar (start, '/');
-    const int end2 = url.indexOfChar (start, ':');
-
-    const int end = (end1 < 0 && end2 < 0) ? std::numeric_limits<int>::max()
-                                           : ((end1 < 0 || end2 < 0) ? jmax (end1, end2)
-                                                                     : jmin (end1, end2));
-    return url.substring (start, end);
+    return getDomainInternal (false);
 }
 
-String URL::getSubPath() const
+String URL::getSubPath (bool includeGetParameters) const
 {
-    const int startOfPath = URLHelpers::findStartOfPath (url);
+    auto startOfPath = URLHelpers::findStartOfPath (url);
+    auto subPath = startOfPath <= 0 ? String()
+                                    : url.substring (startOfPath);
 
-    return startOfPath <= 0 ? String()
-        : url.substring (startOfPath);
+    if (includeGetParameters)
+        subPath += getQueryString();
+
+    return subPath;
+}
+
+String URL::getQueryString() const
+{
+    if (parameterNames.size() > 0)
+        return "?" + URLHelpers::getMangledParameters (*this);
+
+    return {};
 }
 
 String URL::getScheme() const
@@ -324,6 +360,52 @@ String URL::getScheme() const
     return url.substring (0, URLHelpers::findEndOfScheme (url) - 1);
 }
 
+#if ! JUCE_ANDROID
+bool URL::isLocalFile() const
+{
+    return getScheme() == "file";
+}
+
+File URL::getLocalFile() const
+{
+    return fileFromFileSchemeURL (*this);
+}
+
+String URL::getFileName() const
+{
+    return toString (false).fromLastOccurrenceOf ("/", false, true);
+}
+#endif
+
+File URL::fileFromFileSchemeURL (const URL& fileURL)
+{
+    if (! fileURL.isLocalFile())
+    {
+        jassertfalse;
+        return {};
+    }
+
+    auto path = removeEscapeChars (fileURL.getDomainInternal (true)).replace ("+", "%2B");
+
+   #if JUCE_WINDOWS
+    bool isUncPath = (! fileURL.url.startsWith ("file:///"));
+   #else
+    path = File::getSeparatorString() + path;
+   #endif
+
+    auto urlElements = StringArray::fromTokens (fileURL.getSubPath(), "/", "");
+
+    for (auto urlElement : urlElements)
+        path += File::getSeparatorString() + removeEscapeChars (urlElement.replace ("+", "%2B"));
+
+   #if JUCE_WINDOWS
+    if (isUncPath)
+        path = "\\\\" + path;
+   #endif
+
+    return path;
+}
+
 int URL::getPort() const
 {
     auto colonPos = url.indexOfChar (URLHelpers::findStartOfNetLocation (url), ':');
@@ -340,10 +422,10 @@ URL URL::withNewDomainAndPath (const String& newURL) const
 
 URL URL::withNewSubPath (const String& newPath) const
 {
-    const int startOfPath = URLHelpers::findStartOfPath (url);
-
     URL u (*this);
 
+    auto startOfPath = URLHelpers::findStartOfPath (url);
+
     if (startOfPath > 0)
         u.url = url.substring (0, startOfPath);
 
@@ -351,6 +433,13 @@ URL URL::withNewSubPath (const String& newPath) const
     return u;
 }
 
+URL URL::getParentURL() const
+{
+    URL u (*this);
+    u.url = URLHelpers::removeLastPathSection (u.url);
+    return u;
+}
+
 URL URL::getChildURL (const String& subPath) const
 {
     URL u (*this);
@@ -416,18 +505,15 @@ void URL::createHeadersAndPostData (String& headers, MemoryBlock& postDataToWrit
 //==============================================================================
 bool URL::isProbablyAWebsiteURL (const String& possibleURL)
 {
-    static const char* validProtocols[] = { "http:", "ftp:", "https:" };
-
-    for (auto* protocol : validProtocols)
+    for (auto* protocol : { "http:", "https:", "ftp:" })
         if (possibleURL.startsWithIgnoreCase (protocol))
             return true;
 
-    if (possibleURL.containsChar ('@')
-        || possibleURL.containsChar (' '))
+    if (possibleURL.containsChar ('@') || possibleURL.containsChar (' '))
         return false;
 
-    const String topLevelDomain (possibleURL.upToFirstOccurrenceOf ("/", false, false)
-                                 .fromLastOccurrenceOf (".", false, false));
+    auto topLevelDomain = possibleURL.upToFirstOccurrenceOf ("/", false, false)
+                                     .fromLastOccurrenceOf (".", false, false);
 
     return topLevelDomain.isNotEmpty() && topLevelDomain.length() <= 3;
 }
@@ -441,40 +527,175 @@ bool URL::isProbablyAnEmailAddress (const String& possibleEmailAddress)
         && ! possibleEmailAddress.endsWithChar ('.');
 }
 
-//==============================================================================
-WebInputStream* URL::createInputStream (const bool usePostCommand,
-                                        OpenStreamProgressCallback* const progressCallback,
-                                        void* const progressCallbackContext,
-                                        String headers,
-                                        const int timeOutMs,
-                                        StringPairArray* const responseHeaders,
-                                        int* statusCode,
-                                        const int numRedirectsToFollow,
-                                        String httpRequestCmd) const
+String URL::getDomainInternal (bool ignorePort) const
 {
-    ScopedPointer<WebInputStream> wi (new WebInputStream (*this, usePostCommand));
+    auto start = URLHelpers::findStartOfNetLocation (url);
+    auto end1 = url.indexOfChar (start, '/');
+    auto end2 = ignorePort ? -1 : url.indexOfChar (start, ':');
 
-    struct ProgressCallbackCaller : WebInputStream::Listener
+    auto end = (end1 < 0 && end2 < 0) ? std::numeric_limits<int>::max()
+                                      : ((end1 < 0 || end2 < 0) ? jmax (end1, end2)
+                                                                : jmin (end1, end2));
+    return url.substring (start, end);
+}
+
+#if JUCE_IOS
+URL::Bookmark::Bookmark (void* bookmarkToUse) : data (bookmarkToUse)
+{
+}
+
+URL::Bookmark::~Bookmark()
+{
+    [(NSData*) data release];
+}
+
+void setURLBookmark (URL& u, void* bookmark)
+{
+    u.bookmark = new URL::Bookmark (bookmark);
+}
+
+void* getURLBookmark (URL& u)
+{
+    if (u.bookmark.get() == nullptr)
+        return nullptr;
+
+    return u.bookmark.get()->data;
+}
+
+template <typename Stream> struct iOSFileStreamWrapperFlush    { static void flush (Stream*) {} };
+template <> struct iOSFileStreamWrapperFlush<FileOutputStream> { static void flush (OutputStream* o) { o->flush(); } };
+
+template <typename Stream>
+class iOSFileStreamWrapper : public Stream
+{
+public:
+    iOSFileStreamWrapper (URL& urlToUse)
+        : Stream (getLocalFileAccess (urlToUse)),
+          url (urlToUse)
+    {}
+
+    ~iOSFileStreamWrapper()
     {
-        ProgressCallbackCaller (OpenStreamProgressCallback* const progressCallbackToUse,
-                                void* const progressCallbackContextToUse)
+        iOSFileStreamWrapperFlush<Stream>::flush (this);
+
+        if (NSData* bookmark = (NSData*) getURLBookmark (url))
+        {
+            BOOL isBookmarkStale = false;
+            NSError* error = nil;
+
+            auto nsURL = [NSURL URLByResolvingBookmarkData: bookmark
+                                                   options: 0
+                                             relativeToURL: nil
+                                       bookmarkDataIsStale: &isBookmarkStale
+                                                     error: &error];
+
+            if (error == nil)
+            {
+                if (isBookmarkStale)
+                    updateStaleBookmark (nsURL, url);
+
+                [nsURL stopAccessingSecurityScopedResource];
+            }
+            else
+            {
+                auto desc = [error localizedDescription];
+                ignoreUnused (desc);
+                jassertfalse;
+            }
+        }
+    }
+
+private:
+    URL url;
+    bool securityAccessSucceeded = false;
+
+    File getLocalFileAccess (URL& urlToUse)
+    {
+        if (NSData* bookmark = (NSData*) getURLBookmark (urlToUse))
+        {
+            BOOL isBookmarkStale = false;
+            NSError* error = nil;
+
+            auto nsURL = [NSURL URLByResolvingBookmarkData: bookmark
+                                                   options: 0
+                                             relativeToURL: nil
+                                       bookmarkDataIsStale: &isBookmarkStale
+                                                      error: &error];
+
+            if (error == nil)
+            {
+                securityAccessSucceeded = [nsURL startAccessingSecurityScopedResource];
+
+                if (isBookmarkStale)
+                    updateStaleBookmark (nsURL, urlToUse);
+
+                return urlToUse.getLocalFile();
+            }
+
+            auto desc = [error localizedDescription];
+            ignoreUnused (desc);
+            jassertfalse;
+        }
+
+        return urlToUse.getLocalFile();
+    }
+
+    void updateStaleBookmark (NSURL* nsURL, URL& juceUrl)
+    {
+        NSError* error = nil;
+
+        NSData* bookmark = [nsURL bookmarkDataWithOptions: NSURLBookmarkCreationSuitableForBookmarkFile
+                           includingResourceValuesForKeys: nil
+                                            relativeToURL: nil
+                                                    error: &error];
+
+        if (error == nil)
+            setURLBookmark (juceUrl, (void*) bookmark);
+        else
+            jassertfalse;
+    }
+};
+#endif
+
+//==============================================================================
+std::unique_ptr<InputStream> URL::createInputStream (bool usePostCommand,
+                                                     OpenStreamProgressCallback* progressCallback,
+                                                     void* progressCallbackContext,
+                                                     String headers,
+                                                     int timeOutMs,
+                                                     StringPairArray* responseHeaders,
+                                                     int* statusCode,
+                                                     int numRedirectsToFollow,
+                                                     String httpRequestCmd) const
+{
+    if (isLocalFile())
+    {
+       #if JUCE_IOS
+        // We may need to refresh the embedded bookmark.
+        return std::make_unique<iOSFileStreamWrapper<FileInputStream>> (const_cast<URL&>(*this));
+       #else
+        return getLocalFile().createInputStream();
+       #endif
+    }
+
+    auto wi = std::make_unique<WebInputStream> (*this, usePostCommand);
+
+    struct ProgressCallbackCaller  : public WebInputStream::Listener
+    {
+        ProgressCallbackCaller (OpenStreamProgressCallback* progressCallbackToUse, void* progressCallbackContextToUse)
             : callback (progressCallbackToUse), data (progressCallbackContextToUse)
         {}
 
         bool postDataSendProgress (WebInputStream&, int bytesSent, int totalBytes) override
         {
-            return callback(data, bytesSent, totalBytes);
+            return callback (data, bytesSent, totalBytes);
         }
 
-        OpenStreamProgressCallback* const callback;
+        OpenStreamProgressCallback* callback;
         void* const data;
-
-        // workaround a MSVC 2013 compiler warning
-        ProgressCallbackCaller (const ProgressCallbackCaller& o) : callback (o.callback), data (o.data) { jassertfalse; }
-        ProgressCallbackCaller& operator= (const ProgressCallbackCaller&) { jassertfalse; return *this; }
     };
 
-    ScopedPointer<ProgressCallbackCaller> callbackCaller =
+    std::unique_ptr<ProgressCallbackCaller> callbackCaller
         (progressCallback != nullptr ? new ProgressCallbackCaller (progressCallback, progressCallbackContext) : nullptr);
 
     if (headers.isNotEmpty())
@@ -488,7 +709,7 @@ WebInputStream* URL::createInputStream (const bool usePostCommand,
 
     wi->withNumRedirectsToFollow (numRedirectsToFollow);
 
-    bool success = wi->connect (callbackCaller);
+    bool success = wi->connect (callbackCaller.get());
 
     if (statusCode != nullptr)
         *statusCode = wi->getStatusCode();
@@ -499,13 +720,39 @@ WebInputStream* URL::createInputStream (const bool usePostCommand,
     if (! success || wi->isError())
         return nullptr;
 
-    return wi.release();
+    // Older GCCs complain about binding unique_ptr<WebInputStream>&& to unique_ptr<InputStream>
+    // if we just `return wi` here.
+    return std::unique_ptr<InputStream> (std::move (wi));
+}
+
+#if JUCE_ANDROID
+OutputStream* juce_CreateContentURIOutputStream (const URL&);
+#endif
+
+std::unique_ptr<OutputStream> URL::createOutputStream() const
+{
+    if (isLocalFile())
+    {
+       #if JUCE_IOS
+        // We may need to refresh the embedded bookmark.
+        return std::make_unique<iOSFileStreamWrapper<FileOutputStream>> (const_cast<URL&> (*this));
+       #else
+        return std::make_unique<FileOutputStream> (getLocalFile());
+       #endif
+    }
+
+   #if JUCE_ANDROID
+    return std::unique_ptr<OutputStream> (juce_CreateContentURIOutputStream (*this));
+   #else
+    return nullptr;
+   #endif
 }
 
 //==============================================================================
 bool URL::readEntireBinaryStream (MemoryBlock& destData, bool usePostCommand) const
 {
-    const ScopedPointer<InputStream> in (createInputStream (usePostCommand));
+    const std::unique_ptr<InputStream> in (isLocalFile() ? getLocalFile().createInputStream()
+                                                         : createInputStream (usePostCommand));
 
     if (in != nullptr)
     {
@@ -518,7 +765,8 @@ bool URL::readEntireBinaryStream (MemoryBlock& destData, bool usePostCommand) co
 
 String URL::readEntireTextStream (bool usePostCommand) const
 {
-    const ScopedPointer<InputStream> in (createInputStream (usePostCommand));
+    const std::unique_ptr<InputStream> in (isLocalFile() ? getLocalFile().createInputStream()
+                                                         : createInputStream (usePostCommand));
 
     if (in != nullptr)
         return in->readEntireStreamAsString();
@@ -526,23 +774,23 @@ String URL::readEntireTextStream (bool usePostCommand) const
     return {};
 }
 
-XmlElement* URL::readEntireXmlStream (bool usePostCommand) const
+std::unique_ptr<XmlElement> URL::readEntireXmlStream (bool usePostCommand) const
 {
-    return XmlDocument::parse (readEntireTextStream (usePostCommand));
+    return parseXML (readEntireTextStream (usePostCommand));
 }
 
 //==============================================================================
 URL URL::withParameter (const String& parameterName,
                         const String& parameterValue) const
 {
-    URL u (*this);
+    auto u = *this;
     u.addParameter (parameterName, parameterValue);
     return u;
 }
 
 URL URL::withParameters (const StringPairArray& parametersToAdd) const
 {
-    URL u (*this);
+    auto u = *this;
 
     for (int i = 0; i < parametersToAdd.size(); ++i)
         u.addParameter (parametersToAdd.getAllKeys()[i],
@@ -558,7 +806,7 @@ URL URL::withPOSTData (const String& newPostData) const
 
 URL URL::withPOSTData (const MemoryBlock& newPostData) const
 {
-    URL u (*this);
+    auto u = *this;
     u.postData = newPostData;
     return u;
 }
@@ -572,7 +820,7 @@ URL::Upload::Upload (const String& param, const String& name,
 
 URL URL::withUpload (Upload* const f) const
 {
-    URL u (*this);
+    auto u = *this;
 
     for (int i = u.filesToUpload.size(); --i >= 0;)
         if (u.filesToUpload.getObjectPointerUnchecked(i)->parameterName == f->parameterName)
@@ -612,8 +860,8 @@ String URL::removeEscapeChars (const String& s)
     {
         if (utf8.getUnchecked(i) == '%')
         {
-            const int hexDigit1 = CharacterFunctions::getHexDigitValue ((juce_wchar) (uint8) utf8 [i + 1]);
-            const int hexDigit2 = CharacterFunctions::getHexDigitValue ((juce_wchar) (uint8) utf8 [i + 2]);
+            auto hexDigit1 = CharacterFunctions::getHexDigitValue ((juce_wchar) (uint8) utf8 [i + 1]);
+            auto hexDigit2 = CharacterFunctions::getHexDigitValue ((juce_wchar) (uint8) utf8 [i + 2]);
 
             if (hexDigit1 >= 0 && hexDigit2 >= 0)
             {
