@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -31,6 +31,8 @@ class WebInputStream;
 
     This class can be used to launch URLs in browsers, and also to create
     InputStreams that can read from remote http or ftp sources.
+
+    @tags{Core}
 */
 class JUCE_API  URL
 {
@@ -46,14 +48,16 @@ public:
     */
     URL (const String& url);
 
-    /** Creates a copy of another URL. */
-    URL (const URL&);
+    URL (const URL&) = default;
+    URL& operator= (const URL&) = default;
+    URL (URL&&) = default;
+    URL& operator= (URL&&) = default;
+
+    /** Creates URL referring to a local file on your disk using the file:// scheme. */
+    explicit URL (File);
 
     /** Destructor. */
-    ~URL();
-
-    /** Copies this URL from another one. */
-    URL& operator= (const URL&);
+    ~URL() = default;
 
     /** Compares two URLs.
         All aspects of the URLs must be identical for them to match, including any parameters,
@@ -84,8 +88,17 @@ public:
 
     /** Returns the path part of the URL.
         E.g. for "http://www.xyz.com/foo/bar?x=1", this will return "foo/bar".
+
+        If includeGetParameters is true and any parameters have been set with the
+        withParameter() method, then the string will have these appended on the
+        end and url-encoded.
     */
-    String getSubPath() const;
+    String getSubPath (bool includeGetParameters = false) const;
+
+    /** If any parameters are set, returns these URL encoded, including the "?"
+     *  prefix.
+    */
+    String getQueryString() const;
 
     /** Returns the scheme of the URL.
         E.g. for "http://www.xyz.com/foobar", this will return "http". (It won't
@@ -93,6 +106,29 @@ public:
     */
     String getScheme() const;
 
+    /** Returns true if this URL refers to a local file. */
+    bool isLocalFile() const;
+
+    /** Returns the file path of the local file to which this URL refers to.
+        If the URL does not represent a local file URL (i.e. the URL's scheme is not 'file')
+        then this method will assert.
+
+        This method also supports converting Android's content:// URLs to
+        local file paths.
+
+        @see isLocalFile
+    */
+    File getLocalFile() const;
+
+    /** Returns the file name. For all but Android's content:// scheme, it will
+        simply return the last segment of the URL.
+        E.g. for "http://www.xyz.com/foo/bar.txt", this will return "bar.txt".
+
+        For Android's content:// scheme, it will attempt to resolve the filename
+        located under the URL.
+    */
+    String getFileName() const;
+
     /** Attempts to read a port number from the URL.
         @returns the port number, or 0 if none is explicitly specified.
     */
@@ -112,6 +148,11 @@ public:
     */
     URL withNewSubPath (const String& newPath) const;
 
+    /** Attempts to return a URL which is the parent folder containing this URL.
+        If there isn't a parent, this method will just return a copy of this URL.
+    */
+    URL getParentURL() const;
+
     /** Returns a new URL that refers to a sub-path relative to this one.
         E.g. if the URL is "http://www.xyz.com/foo" and you call this with
         "bar", it'll return "http://www.xyz.com/foo/bar". Note that there's no way for
@@ -227,7 +268,7 @@ public:
     URL withPOSTData (const MemoryBlock& postData) const;
 
     /** Returns the data that was set using withPOSTData(). */
-    String getPostData() const noexcept                             { return postData.toString(); }
+    String getPostData() const                                      { return postData.toString(); }
 
     /** Returns the data that was set using withPOSTData() as MemoryBlock. */
     const MemoryBlock& getPostDataAsMemoryBlock() const noexcept    { return postData; }
@@ -255,20 +296,22 @@ public:
         It allows your app to receive progress updates during a lengthy POST operation. If you
         want to continue the operation, this should return true, or false to abort.
     */
-    typedef bool (OpenStreamProgressCallback) (void* context, int bytesSent, int totalBytes);
+    using OpenStreamProgressCallback = bool (void* context, int bytesSent, int totalBytes);
 
     /** Attempts to open a stream that can read from this URL.
 
-        This method is a convenience wrapper for creating a new WebInputStream and setting some
-        commonly used options. The returned WebInputStream will have already been connected and
-        reading can start instantly.
-
         Note that this method will block until the first byte of data has been received or an
         error has occurred.
 
         Note that on some platforms (Android, for example) it's not permitted to do any network
         action from the message thread, so you must only call it from a background thread.
 
+        Unless the URL represents a local file, this method returns an instance of a
+        WebInputStream. You can use dynamic_cast to cast the return value to a WebInputStream
+        which allows you more fine-grained control of the transfer process.
+
+        If the URL represents a local file, then this method simply returns a FileInputStream.
+
         @param doPostLikeRequest if true, the parameters added to this class will be transferred
                                  via the HTTP headers which is typical for POST requests. Otherwise
                                  the parameters will be added to the URL address. Additionally,
@@ -294,41 +337,47 @@ public:
                                  returning a response (ignored for Android which follows up to 5 redirects)
         @param httpRequestCmd    Specify which HTTP Request to use. If this is empty, then doPostRequest
                                  will determine the HTTP request.
-        @returns    an input stream that the caller must delete, or a null pointer if there was an
-                    error trying to open it.
+        @returns    a valid input stream, or nullptr if there was an error trying to open it.
      */
-    WebInputStream* createInputStream (bool doPostLikeRequest,
-                                       OpenStreamProgressCallback* progressCallback = nullptr,
-                                       void* progressCallbackContext = nullptr,
-                                       String extraHeaders = String(),
-                                       int connectionTimeOutMs = 0,
-                                       StringPairArray* responseHeaders = nullptr,
-                                       int* statusCode = nullptr,
-                                       int numRedirectsToFollow = 5,
-                                       String httpRequestCmd = String()) const;
+    std::unique_ptr<InputStream> createInputStream (bool doPostLikeRequest,
+                                                    OpenStreamProgressCallback* progressCallback = nullptr,
+                                                    void* progressCallbackContext = nullptr,
+                                                    String extraHeaders = {},
+                                                    int connectionTimeOutMs = 0,
+                                                    StringPairArray* responseHeaders = nullptr,
+                                                    int* statusCode = nullptr,
+                                                    int numRedirectsToFollow = 5,
+                                                    String httpRequestCmd = {}) const;
+
+    /** Attempts to open an output stream to a URL for writing
+
+        This method can only be used for certain scheme types such as local files
+        and content:// URIs on Android.
+    */
+    std::unique_ptr<OutputStream> createOutputStream() const;
 
     //==============================================================================
     /** Represents a download task.
-        Returned by downloadToFile to allow querying and controling the download task.
+        Returned by downloadToFile to allow querying and controlling the download task.
     */
-    class DownloadTask
+    class JUCE_API  DownloadTask
     {
     public:
-        struct Listener
+        /** Used to receive callbacks for download progress */
+        struct JUCE_API  Listener
         {
             virtual ~Listener();
 
             /** Called when the download has finished. Be aware that this callback may
                 come on an arbitrary thread. */
-            virtual void finished (DownloadTask* task, bool success) = 0;
+            virtual void finished (URL::DownloadTask* task, bool success) = 0;
 
             /** Called periodically by the OS to indicate download progress.
                 Beware that this callback may come on an arbitrary thread.
             */
-            virtual void progress (DownloadTask* task, int64 bytesDownloaded, int64 totalLength);
+            virtual void progress (URL::DownloadTask* task, int64 bytesDownloaded, int64 totalLength);
         };
 
-
         /** Releases the resources of the download task, unregisters the listener
             and cancels the download if necessary. */
         virtual ~DownloadTask();
@@ -352,16 +401,20 @@ public:
         /** Returns true if there was an error. */
         inline bool hadError() const                      { return error; }
 
+        /** Returns the target file location that was provided in URL::downloadToFile. */
+        File getTargetLocation() const                    { return targetLocation; }
+
     protected:
         int64 contentLength = -1, downloaded = 0;
         bool finished = false, error = false;
         int httpCode = -1;
+        File targetLocation;
 
         DownloadTask();
 
     private:
         friend class URL;
-        static DownloadTask* createFallbackDownloader (const URL&, const File&, const String&, Listener*, bool);
+        static std::unique_ptr<DownloadTask> createFallbackDownloader (const URL&, const File&, const String&, Listener*, bool);
 
     public:
        #if JUCE_IOS
@@ -382,10 +435,10 @@ public:
         using a native OS background network task. Such tasks automatically deal with
         network re-connections and continuing your download while your app is suspended.
     */
-    DownloadTask* downloadToFile (const File& targetLocation,
-                                  String extraHeaders = String(),
-                                  DownloadTask::Listener* listener = nullptr,
-                                  bool usePostCommand = false);
+    std::unique_ptr<DownloadTask> downloadToFile (const File& targetLocation,
+                                                  String extraHeaders = String(),
+                                                  DownloadTask::Listener* listener = nullptr,
+                                                  bool usePostCommand = false);
 
     //==============================================================================
     /** Tries to download the entire contents of this URL into a binary data block.
@@ -425,9 +478,6 @@ public:
         If it fails, or if the text that it reads can't be parsed as XML, this will
         return nullptr.
 
-        When it returns a valid XmlElement object, the caller is responsibile for deleting
-        this object when no longer needed.
-
         Note that on some platforms (Android, for example) it's not permitted to do any network
         action from the message thread, so you must only call it from a background thread.
 
@@ -436,7 +486,7 @@ public:
 
         @see readEntireBinaryStream, readEntireTextStream
     */
-    XmlElement* readEntireXmlStream (bool usePostCommand = false) const;
+    std::unique_ptr<XmlElement> readEntireXmlStream (bool usePostCommand = false) const;
 
     //==============================================================================
     /** Adds escape sequences to a string to encode any characters that aren't
@@ -486,20 +536,40 @@ private:
     MemoryBlock postData;
     StringArray parameterNames, parameterValues;
 
+    static File fileFromFileSchemeURL (const URL&);
+    String getDomainInternal (bool) const;
+
     struct Upload  : public ReferenceCountedObject
     {
         Upload (const String&, const String&, const String&, const File&, MemoryBlock*);
         String parameterName, filename, mimeType;
         File file;
-        ScopedPointer<MemoryBlock> data;
+        std::unique_ptr<MemoryBlock> data;
 
         JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Upload)
     };
 
-    friend struct ContainerDeletePolicy<Upload>;
     ReferenceCountedArray<Upload> filesToUpload;
 
+  #if JUCE_IOS
+    struct Bookmark : public ReferenceCountedObject
+    {
+        using Ptr = ReferenceCountedObjectPtr<Bookmark>;
+
+        Bookmark (void*);
+        ~Bookmark();
+
+        void* data;
+    };
+
+    Bookmark::Ptr bookmark;
+
+    friend void setURLBookmark (URL&, void*);
+    friend void* getURLBookmark (URL&);
+  #endif
+
     URL (const String&, int);
+    void init();
     void addParameter (const String&, const String&);
     void createHeadersAndPostData (String&, MemoryBlock&) const;
     URL withUpload (Upload*) const;
