@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -20,6 +20,20 @@
   ==============================================================================
 */
 
+#if ! DOXYGEN && (JUCE_MAC || JUCE_IOS)
+ // Annoyingly we can only forward-declare a typedef by forward-declaring the
+ // aliased type
+ #if __has_attribute(objc_bridge)
+  #define JUCE_CF_BRIDGED_TYPE(T) __attribute__((objc_bridge(T)))
+ #else
+  #define JUCE_CF_BRIDGED_TYPE(T)
+ #endif
+
+ typedef const struct JUCE_CF_BRIDGED_TYPE(NSString) __CFString * CFStringRef;
+
+ #undef JUCE_CF_BRIDGED_TYPE
+#endif
+
 namespace juce
 {
 
@@ -32,8 +46,10 @@ namespace juce
     dream of.
 
     @see StringArray, StringPairArray
+
+    @tags{Core}
 */
-class JUCE_API  String
+class JUCE_API  String  final
 {
 public:
     //==============================================================================
@@ -43,10 +59,10 @@ public:
     String() noexcept;
 
     /** Creates a copy of another string. */
-    String (const String& other) noexcept;
+    String (const String&) noexcept;
 
     /** Move constructor */
-    String (String&& other) noexcept;
+    String (String&&) noexcept;
 
     /** Creates a string from a zero-terminated ascii text string.
 
@@ -93,37 +109,37 @@ public:
 
     //==============================================================================
     /** Creates a string from a UTF-8 character string */
-    String (const CharPointer_UTF8 text);
+    String (CharPointer_UTF8 text);
 
     /** Creates a string from a UTF-8 character string */
-    String (const CharPointer_UTF8 text, size_t maxChars);
+    String (CharPointer_UTF8 text, size_t maxChars);
 
     /** Creates a string from a UTF-8 character string */
-    String (const CharPointer_UTF8 start, const CharPointer_UTF8 end);
+    String (CharPointer_UTF8 start, CharPointer_UTF8 end);
 
     //==============================================================================
     /** Creates a string from a UTF-16 character string */
-    String (const CharPointer_UTF16 text);
+    String (CharPointer_UTF16 text);
 
     /** Creates a string from a UTF-16 character string */
-    String (const CharPointer_UTF16 text, size_t maxChars);
+    String (CharPointer_UTF16 text, size_t maxChars);
 
     /** Creates a string from a UTF-16 character string */
-    String (const CharPointer_UTF16 start, const CharPointer_UTF16 end);
+    String (CharPointer_UTF16 start, CharPointer_UTF16 end);
 
     //==============================================================================
     /** Creates a string from a UTF-32 character string */
-    String (const CharPointer_UTF32 text);
+    String (CharPointer_UTF32 text);
 
     /** Creates a string from a UTF-32 character string */
-    String (const CharPointer_UTF32 text, size_t maxChars);
+    String (CharPointer_UTF32 text, size_t maxChars);
 
     /** Creates a string from a UTF-32 character string */
-    String (const CharPointer_UTF32 start, const CharPointer_UTF32 end);
+    String (CharPointer_UTF32 start, CharPointer_UTF32 end);
 
     //==============================================================================
     /** Creates a string from an ASCII character string */
-    String (const CharPointer_ASCII text);
+    String (CharPointer_ASCII text);
 
     /** Creates a string from a UTF-8 encoded std::string. */
     String (const std::string&);
@@ -138,17 +154,6 @@ public:
     /** Destructor. */
     ~String() noexcept;
 
-    //==============================================================================
-   #if JUCE_ALLOW_STATIC_NULL_VARIABLES
-    /** This is a static empty string object that can be used if you need a reference to one.
-        The value of String::empty is exactly the same as String(), and in almost all cases
-        it's better to avoid String::empty and just use String() or {} instead, so that the compiler
-        only has to reason about locally-constructed objects, rather than taking into account
-        the fact that you're referencing a global shared static memory address.
-    */
-    static const String empty;
-   #endif
-
     /** This is the character encoding type used internally to store the string.
 
         By setting the value of JUCE_STRING_UTF_TYPE to 8, 16, or 32, you can change the
@@ -162,11 +167,11 @@ public:
         toUTF32() methods let you access the string's content in any of the other formats.
     */
    #if (JUCE_STRING_UTF_TYPE == 32)
-    typedef CharPointer_UTF32 CharPointerType;
+    using CharPointerType = CharPointer_UTF32;
    #elif (JUCE_STRING_UTF_TYPE == 16)
-    typedef CharPointer_UTF16 CharPointerType;
+    using CharPointerType = CharPointer_UTF16;
    #elif (DOXYGEN || JUCE_STRING_UTF_TYPE == 8)
-    typedef CharPointer_UTF8  CharPointerType;
+    using CharPointerType = CharPointer_UTF8;
    #else
     #error "You must set the value of JUCE_STRING_UTF_TYPE to be either 8, 16, or 32!"
    #endif
@@ -230,8 +235,8 @@ public:
         @param startOfTextToAppend  the start of the string to add. This must not be a nullptr
         @param endOfTextToAppend    the end of the string to add. This must not be a nullptr
     */
-    void appendCharPointer (const CharPointerType startOfTextToAppend,
-                            const CharPointerType endOfTextToAppend);
+    void appendCharPointer (CharPointerType startOfTextToAppend,
+                            CharPointerType endOfTextToAppend);
 
     /** Appends a string to the end of this one.
 
@@ -239,8 +244,8 @@ public:
         @param endOfTextToAppend    the end of the string to add. This must not be a nullptr
     */
     template <class CharPointer>
-    void appendCharPointer (const CharPointer startOfTextToAppend,
-                            const CharPointer endOfTextToAppend)
+    void appendCharPointer (CharPointer startOfTextToAppend,
+                            CharPointer endOfTextToAppend)
     {
         jassert (startOfTextToAppend.getAddress() != nullptr && endOfTextToAppend.getAddress() != nullptr);
 
@@ -260,7 +265,7 @@ public:
     }
 
     /** Appends a string to the end of this one. */
-    void appendCharPointer (const CharPointerType textToAppend);
+    void appendCharPointer (CharPointerType textToAppend);
 
     /** Appends a string to the end of this one.
 
@@ -268,7 +273,7 @@ public:
         @param maxCharsToTake   the maximum number of characters to take from the string passed in
     */
     template <class CharPointer>
-    void appendCharPointer (const CharPointer textToAppend, size_t maxCharsToTake)
+    void appendCharPointer (CharPointer textToAppend, size_t maxCharsToTake)
     {
         if (textToAppend.getAddress() != nullptr)
         {
@@ -290,7 +295,7 @@ public:
 
     /** Appends a string to the end of this one. */
     template <class CharPointer>
-    void appendCharPointer (const CharPointer textToAppend)
+    void appendCharPointer (CharPointer textToAppend)
     {
         appendCharPointer (textToAppend, std::numeric_limits<size_t>::max());
     }
@@ -302,13 +307,13 @@ public:
         Note that there's also an isNotEmpty() method to help write readable code.
         @see containsNonWhitespaceChars()
     */
-    inline bool isEmpty() const noexcept                    { return text.isEmpty(); }
+    bool isEmpty() const noexcept                           { return text.isEmpty(); }
 
     /** Returns true if the string contains at least one character.
         Note that there's also an isEmpty() method to help write readable code.
         @see containsNonWhitespaceChars()
     */
-    inline bool isNotEmpty() const noexcept                 { return ! text.isEmpty(); }
+    bool isNotEmpty() const noexcept                        { return ! text.isEmpty(); }
 
     /** Resets this string to be empty. */
     void clear() noexcept;
@@ -914,6 +919,35 @@ public:
     template <typename... Args>
     static String formatted (const String& formatStr, Args... args)      { return formattedRaw (formatStr.toRawUTF8(), args...); }
 
+    /** Returns an iterator pointing at the beginning of the string. */
+    CharPointerType begin() const                                        { return getCharPointer(); }
+
+    /** Returns an iterator pointing at the terminating null of the string.
+
+        Note that this has to find the terminating null before returning it, so prefer to
+        call this once before looping and then reuse the result, rather than calling 'end()'
+        each time through the loop.
+
+        @code
+        String str = ...;
+
+        // BEST
+        for (auto c : str)
+            DBG (c);
+
+        // GOOD
+        for (auto ptr = str.begin(), end = str.end(); ptr != end; ++ptr)
+            DBG (*ptr);
+
+        std::for_each (str.begin(), str.end(), [] (juce_wchar c) { DBG (c); });
+
+        // BAD
+        for (auto ptr = str.begin(); ptr != str.end(); ++ptr)
+            DBG (*ptr);
+        @endcode
+    */
+    CharPointerType end() const                                          { return begin().findTerminatingNull(); }
+
     //==============================================================================
     // Numeric conversions..
 
@@ -971,25 +1005,32 @@ public:
 
     /** Creates a string representing this floating-point number.
         @param floatValue               the value to convert to a string
-        @param numberOfDecimalPlaces    if this is > 0, it will format the number using that many
-                                        decimal places, and will not use exponent notation. If 0 or
-                                        less, it will use exponent notation if necessary.
+        @param numberOfDecimalPlaces    if this is > 0 the number will be formatted using that many
+                                        decimal places, adding trailing zeros as required. If 0 or
+                                        less the number will be formatted using the C++ standard
+                                        library default format, which uses scientific notation for
+                                        large and small numbers.
+        @param useScientificNotation    if the number should be formatted using scientific notation
         @see getDoubleValue, getIntValue
     */
-    String (float floatValue, int numberOfDecimalPlaces);
+    String (float floatValue, int numberOfDecimalPlaces, bool useScientificNotation = false);
 
     /** Creates a string representing this floating-point number.
         @param doubleValue              the value to convert to a string
         @param numberOfDecimalPlaces    if this is > 0, it will format the number using that many
-                                        decimal places, and will not use exponent notation. If 0 or
-                                        less, it will use exponent notation if necessary.
+                                        decimal places, adding trailing zeros as required, and
+                                        will not use exponent notation. If 0 or less, it will use
+                                        exponent notation if necessary.
+        @param useScientificNotation    if the number should be formatted using scientific notation
         @see getFloatValue, getIntValue
     */
-    String (double doubleValue, int numberOfDecimalPlaces);
+    String (double doubleValue, int numberOfDecimalPlaces, bool useScientificNotation = false);
 
+   #ifndef DOXYGEN
     // Automatically creating a String from a bool opens up lots of nasty type conversion edge cases.
     // If you want a String representation of a bool you can cast the bool to an int first.
     explicit String (bool) = delete;
+   #endif
 
     /** Reads the value of the string as a decimal number (up to 32 bits in size).
 
@@ -1065,6 +1106,131 @@ public:
     */
     static String toHexString (const void* data, int size, int groupSize = 1);
 
+    /** Returns a string containing a decimal with a set number of significant figures.
+
+        @param number                         the input number
+        @param numberOfSignificantFigures     the number of significant figures to use
+    */
+    template <typename DecimalType>
+    static String toDecimalStringWithSignificantFigures (DecimalType number, int numberOfSignificantFigures)
+    {
+        jassert (numberOfSignificantFigures > 0);
+
+        if (number == 0)
+        {
+            if (numberOfSignificantFigures > 1)
+            {
+                String result ("0.0");
+
+                for (int i = 2; i < numberOfSignificantFigures; ++i)
+                    result += "0";
+
+                return result;
+            }
+
+            return "0";
+        }
+
+        auto numDigitsBeforePoint = (int) std::ceil (std::log10 (number < 0 ? -number : number));
+
+       #if JUCE_PROJUCER_LIVE_BUILD
+        auto doubleNumber = (double) number;
+        constexpr int bufferSize = 311;
+        char buffer[bufferSize];
+        auto* ptr = &(buffer[0]);
+        auto* const safeEnd = ptr + (bufferSize - 1);
+        auto numSigFigsParsed = 0;
+
+        auto writeToBuffer = [safeEnd] (char* destination, char data)
+        {
+            *destination++ = data;
+
+            if (destination == safeEnd)
+            {
+                *destination = '\0';
+                return true;
+            }
+
+            return false;
+        };
+
+        auto truncateOrRound = [numberOfSignificantFigures] (double fractional, int sigFigsParsed)
+        {
+            return (sigFigsParsed == numberOfSignificantFigures - 1) ? (int) std::round (fractional)
+                                                                     : (int) fractional;
+        };
+
+        if (doubleNumber < 0)
+        {
+            doubleNumber *= -1;
+            *ptr++ = '-';
+        }
+
+        if (numDigitsBeforePoint > 0)
+        {
+            doubleNumber /= std::pow (10.0, numDigitsBeforePoint);
+
+            while (numDigitsBeforePoint-- > 0)
+            {
+                if (numSigFigsParsed == numberOfSignificantFigures)
+                {
+                    if (writeToBuffer (ptr++, '0'))
+                        return buffer;
+
+                    continue;
+                }
+
+                doubleNumber *= 10;
+                auto digit = truncateOrRound (doubleNumber, numSigFigsParsed);
+
+                if (writeToBuffer (ptr++, (char) ('0' + digit)))
+                    return buffer;
+
+                ++numSigFigsParsed;
+                doubleNumber -= digit;
+            }
+
+            if (numSigFigsParsed == numberOfSignificantFigures)
+            {
+                *ptr++ = '\0';
+                return buffer;
+            }
+        }
+        else
+        {
+            *ptr++ = '0';
+        }
+
+        if (writeToBuffer (ptr++, '.'))
+            return buffer;
+
+        while (numSigFigsParsed < numberOfSignificantFigures)
+        {
+            doubleNumber *= 10;
+            auto digit = truncateOrRound (doubleNumber, numSigFigsParsed);
+
+            if (writeToBuffer (ptr++, (char) ('0' + digit)))
+                return buffer;
+
+            if (numSigFigsParsed != 0 || digit != 0)
+                ++numSigFigsParsed;
+
+            doubleNumber -= digit;
+        }
+
+        *ptr++ = '\0';
+        return buffer;
+       #else
+        auto shift = numberOfSignificantFigures - numDigitsBeforePoint;
+        auto factor = std::pow (10.0, shift);
+        auto rounded = std::round (number * factor) / factor;
+
+        std::stringstream ss;
+        ss << std::fixed << std::setprecision (std::max (shift, 0)) << rounded;
+        return ss.str();
+       #endif
+    }
+
     //==============================================================================
     /** Returns the character pointer currently being used to store this string.
 
@@ -1072,7 +1238,7 @@ public:
         that is returned must not be stored anywhere, as it can be deleted whenever the
         string changes.
     */
-    inline CharPointerType getCharPointer() const noexcept      { return text; }
+    CharPointerType getCharPointer() const noexcept             { return text; }
 
     /** Returns a pointer to a UTF-8 version of this string.
 
@@ -1247,6 +1413,17 @@ public:
     */
     int getReferenceCount() const noexcept;
 
+    //==============================================================================
+    /*  This was a static empty string object, but is now deprecated as it's too easy to accidentally
+        use it indirectly during a static constructor, leading to hard-to-find order-of-initialisation
+        problems.
+        @deprecated If you need an empty String object, just use String() or {}.
+        The only time you might miss having String::empty available might be if you need to return an
+        empty string from a function by reference, but if you need to do that, it's easy enough to use
+        a function-local static String object and return that, avoiding any order-of-initialisation issues.
+    */
+    JUCE_DEPRECATED_STATIC (static const String empty;)
+
 private:
     //==============================================================================
     CharPointerType text;
@@ -1276,7 +1453,7 @@ private:
     static String createHex (uint64);
 
     template <typename Type>
-    static String createHex (Type n)  { return createHex (static_cast<typename TypeHelpers::UnsignedTypeWithSize<sizeof (n)>::type> (n)); }
+    static String createHex (Type n)  { return createHex (static_cast<typename TypeHelpers::UnsignedTypeWithSize<(int) sizeof (n)>::type> (n)); }
 };
 
 //==============================================================================
@@ -1300,6 +1477,8 @@ JUCE_API String JUCE_CALLTYPE operator+ (String string1, const char* string2);
 /** Concatenates two strings. */
 JUCE_API String JUCE_CALLTYPE operator+ (String string1, const wchar_t* string2);
 /** Concatenates two strings. */
+JUCE_API String JUCE_CALLTYPE operator+ (String string1, const std::string& string2);
+/** Concatenates two strings. */
 JUCE_API String JUCE_CALLTYPE operator+ (String string1, char characterToAppend);
 /** Concatenates two strings. */
 JUCE_API String JUCE_CALLTYPE operator+ (String string1, wchar_t characterToAppend);
@@ -1326,6 +1505,8 @@ JUCE_API String& JUCE_CALLTYPE operator<< (String& string1, const wchar_t* strin
 JUCE_API String& JUCE_CALLTYPE operator<< (String& string1, const String& string2);
 /** Appends a string to the end of the first one. */
 JUCE_API String& JUCE_CALLTYPE operator<< (String& string1, StringRef string2);
+/** Appends a string to the end of the first one. */
+JUCE_API String& JUCE_CALLTYPE operator<< (String& string1, const std::string& string2);
 
 /** Appends a decimal number to the end of a string. */
 JUCE_API String& JUCE_CALLTYPE operator<< (String& string1, uint8 number);
@@ -1346,9 +1527,11 @@ JUCE_API String& JUCE_CALLTYPE operator<< (String& string1, float number);
 /** Appends a decimal number to the end of a string. */
 JUCE_API String& JUCE_CALLTYPE operator<< (String& string1, double number);
 
+#ifndef DOXYGEN
 // Automatically creating a String from a bool opens up lots of nasty type conversion edge cases.
 // If you want a String representation of a bool you can cast the bool to an int first.
 String& JUCE_CALLTYPE operator<< (String&, bool) = delete;
+#endif
 
 //==============================================================================
 /** Case-sensitive comparison of two strings. */
@@ -1377,15 +1560,6 @@ JUCE_API bool JUCE_CALLTYPE operator!= (const String& string1, CharPointer_UTF16
 /** Case-sensitive comparison of two strings. */
 JUCE_API bool JUCE_CALLTYPE operator!= (const String& string1, CharPointer_UTF32 string2) noexcept;
 
-/** Case-sensitive comparison of two strings. */
-JUCE_API bool JUCE_CALLTYPE operator>  (const String& string1, const String& string2) noexcept;
-/** Case-sensitive comparison of two strings. */
-JUCE_API bool JUCE_CALLTYPE operator<  (const String& string1, const String& string2) noexcept;
-/** Case-sensitive comparison of two strings. */
-JUCE_API bool JUCE_CALLTYPE operator>= (const String& string1, const String& string2) noexcept;
-/** Case-sensitive comparison of two strings. */
-JUCE_API bool JUCE_CALLTYPE operator<= (const String& string1, const String& string2) noexcept;
-
 //==============================================================================
 /** This operator allows you to write a juce String directly to std output streams.
     This is handy for writing strings to std::cout, std::cerr, etc.
@@ -1413,7 +1587,7 @@ JUCE_API OutputStream& JUCE_CALLTYPE operator<< (OutputStream& stream, StringRef
 
 } // namespace juce
 
-#if JUCE_COMPILER_SUPPORTS_INITIALIZER_LISTS // just used to avoid compiling this under compilers that lack libc++
+#if ! DOXYGEN
 namespace std
 {
     template <> struct hash<juce::String>
