@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -28,6 +28,8 @@ namespace juce
     A special array for holding a list of strings.
 
     @see String, StringPairArray
+
+    @tags{Core}
 */
 class JUCE_API  StringArray
 {
@@ -43,7 +45,24 @@ public:
     StringArray (StringArray&&) noexcept;
 
     /** Creates an array containing a single string. */
-    explicit StringArray (const String& firstValue);
+    StringArray (const String& firstValue);
+
+    /** Creates an array containing a list of strings. */
+    template <typename... OtherElements>
+    StringArray (StringRef firstValue, OtherElements... otherValues) : strings (firstValue, otherValues...) {}
+
+    /** Creates an array containing a list of strings. */
+    StringArray (const std::initializer_list<const char*>& strings);
+
+    /** Creates a StringArray by moving from an Array<String> */
+    StringArray (Array<String>&&) noexcept;
+
+    /** Creates a StringArray from an array of objects which can be implicitly converted to Strings. */
+    template <typename Type>
+    StringArray (const Array<Type>& stringArray)
+    {
+        addArray (stringArray.begin(), stringArray.end());
+    }
 
     /** Creates an array from a raw array of strings.
         @param strings          an array of strings to add
@@ -78,10 +97,6 @@ public:
     */
     StringArray (const wchar_t* const* strings, int numberOfStrings);
 
-   #if JUCE_COMPILER_SUPPORTS_INITIALIZER_LISTS
-    StringArray (const std::initializer_list<const char*>& strings);
-   #endif
-
     /** Destructor. */
     ~StringArray();
 
@@ -91,6 +106,14 @@ public:
     /** Move assignment operator */
     StringArray& operator= (StringArray&&) noexcept;
 
+    /** Copies a StringArray from an array of objects which can be implicitly converted to Strings. */
+    template <typename Type>
+    StringArray& operator= (const Array<Type>& stringArray)
+    {
+        addArray (stringArray.begin(), stringArray.end());
+        return *this;
+    }
+
     /** Swaps the contents of this and another StringArray. */
     void swapWith (StringArray&) noexcept;
 
@@ -132,12 +155,23 @@ public:
     /** Returns a pointer to the first String in the array.
         This method is provided for compatibility with standard C++ iteration mechanisms.
     */
-    inline String* begin() const noexcept       { return strings.begin(); }
+    inline String* begin() noexcept                 { return strings.begin(); }
+
+    /** Returns a pointer to the first String in the array.
+        This method is provided for compatibility with standard C++ iteration mechanisms.
+    */
+    inline const String* begin() const noexcept     { return strings.begin(); }
 
     /** Returns a pointer to the String which follows the last element in the array.
         This method is provided for compatibility with standard C++ iteration mechanisms.
     */
-    inline String* end() const noexcept         { return strings.end(); }
+    inline String* end() noexcept                  { return strings.end(); }
+
+    /** Returns a pointer to the String which follows the last element in the array.
+        This method is provided for compatibility with standard C++ iteration mechanisms.
+    */
+    inline const String* end() const noexcept       { return strings.end(); }
+
 
     /** Searches for a string in the array.
 
@@ -164,10 +198,7 @@ public:
 
     //==============================================================================
     /** Appends a string at the end of the array. */
-    void add (const String& stringToAdd);
-
-    /** Appends a string at the end of the array. */
-    void add (String&& stringToAdd);
+    void add (String stringToAdd);
 
     /** Inserts a string into the array.
 
@@ -176,7 +207,7 @@ public:
         If the index is less than zero or greater than the size of the array,
         the new string will be added to the end of the array.
     */
-    void insert (int index, const String& stringToAdd);
+    void insert (int index, String stringToAdd);
 
     /** Adds a string to the array as long as it's not already in there.
         The search can optionally be case-insensitive.
@@ -190,7 +221,7 @@ public:
         If the index is higher than the array's size, the new string will be
         added to the end of the array; if it's less than zero nothing happens.
     */
-    void set (int index, const String& newString);
+    void set (int index, String newString);
 
     /** Appends some strings from another array to the end of this one.
 
@@ -203,6 +234,18 @@ public:
                    int startIndex = 0,
                    int numElementsToAdd = -1);
 
+    /** Adds items from a range of start/end iterators of some kind of objects which
+        can be implicitly converted to Strings.
+    */
+    template <typename Iterator>
+    void addArray (Iterator&& start, Iterator&& end)
+    {
+        ensureStorageAllocated (size() + (int) static_cast<size_t> (end - start));
+
+        while (start != end)
+            strings.add (*start++);
+    }
+
     /** Merges the strings from another array into this one.
         This will not add a string that already exists.
 
@@ -388,7 +431,7 @@ public:
 
     //==============================================================================
     /** Sorts the array into alphabetical order.
-        @param ignoreCase       if true, the comparisons used will be case-sensitive.
+        @param ignoreCase       if true, the comparisons used will not be case-sensitive.
     */
     void sort (bool ignoreCase);
 
