@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -23,55 +23,64 @@
 namespace juce
 {
 
-XmlDocument::XmlDocument (const String& documentText)
-    : originalText (documentText)
+XmlDocument::XmlDocument (const String& text)  : originalText (text) {}
+XmlDocument::XmlDocument (const File& file)  : inputSource (new FileInputSource (file)) {}
+
+XmlDocument::~XmlDocument() {}
+
+std::unique_ptr<XmlElement> XmlDocument::parse (const File& file)
 {
+    return XmlDocument (file).getDocumentElement();
 }
 
-XmlDocument::XmlDocument (const File& file)
-    : inputSource (new FileInputSource (file))
+std::unique_ptr<XmlElement> XmlDocument::parse (const String& textToParse)
 {
+    return XmlDocument (textToParse).getDocumentElement();
 }
 
-XmlDocument::~XmlDocument()
+std::unique_ptr<XmlElement> parseXML (const String& textToParse)
 {
+    return XmlDocument (textToParse).getDocumentElement();
 }
 
-XmlElement* XmlDocument::parse (const File& file)
+std::unique_ptr<XmlElement> parseXML (const File& file)
 {
-    XmlDocument doc (file);
-    return doc.getDocumentElement();
+    return XmlDocument (file).getDocumentElement();
 }
 
-XmlElement* XmlDocument::parse (const String& xmlData)
+std::unique_ptr<XmlElement> parseXMLIfTagMatches (const String& textToParse, StringRef requiredTag)
 {
-    XmlDocument doc (xmlData);
-    return doc.getDocumentElement();
+    return XmlDocument (textToParse).getDocumentElementIfTagMatches (requiredTag);
 }
 
-void XmlDocument::setInputSource (InputSource* const newSource) noexcept
+std::unique_ptr<XmlElement> parseXMLIfTagMatches (const File& file, StringRef requiredTag)
 {
-    inputSource = newSource;
+    return XmlDocument (file).getDocumentElementIfTagMatches (requiredTag);
 }
 
-void XmlDocument::setEmptyTextElementsIgnored (const bool shouldBeIgnored) noexcept
+void XmlDocument::setInputSource (InputSource* newSource) noexcept
+{
+    inputSource.reset (newSource);
+}
+
+void XmlDocument::setEmptyTextElementsIgnored (bool shouldBeIgnored) noexcept
 {
     ignoreEmptyTextElements = shouldBeIgnored;
 }
 
 namespace XmlIdentifierChars
 {
-    static bool isIdentifierCharSlow (const juce_wchar c) noexcept
+    static bool isIdentifierCharSlow (juce_wchar c) noexcept
     {
         return CharacterFunctions::isLetterOrDigit (c)
                  || c == '_' || c == '-' || c == ':' || c == '.';
     }
 
-    static bool isIdentifierChar (const juce_wchar c) noexcept
+    static bool isIdentifierChar (juce_wchar c) noexcept
     {
         static const uint32 legalChars[] = { 0, 0x7ff6000, 0x87fffffe, 0x7fffffe, 0 };
 
-        return ((int) c < (int) numElementsInArray (legalChars) * 32) ? ((legalChars [c >> 5] & (1 << (c & 31))) != 0)
+        return ((int) c < (int) numElementsInArray (legalChars) * 32) ? ((legalChars [c >> 5] & (uint32) (1 << (c & 31))) != 0)
                                                                       : isIdentifierCharSlow (c);
     }
 
@@ -89,7 +98,7 @@ namespace XmlIdentifierChars
         DBG (s);
     }*/
 
-    static String::CharPointerType findEndOfToken (String::CharPointerType p)
+    static String::CharPointerType findEndOfToken (String::CharPointerType p) noexcept
     {
         while (isIdentifierChar (*p))
             ++p;
@@ -98,11 +107,11 @@ namespace XmlIdentifierChars
     }
 }
 
-XmlElement* XmlDocument::getDocumentElement (const bool onlyReadOuterDocumentElement)
+std::unique_ptr<XmlElement> XmlDocument::getDocumentElement (const bool onlyReadOuterDocumentElement)
 {
     if (originalText.isEmpty() && inputSource != nullptr)
     {
-        ScopedPointer<InputStream> in (inputSource->createInputStream());
+        std::unique_ptr<InputStream> in (inputSource->createInputStream());
 
         if (in != nullptr)
         {
@@ -113,7 +122,7 @@ XmlElement* XmlDocument::getDocumentElement (const bool onlyReadOuterDocumentEle
             if (data.getDataSize() > 2)
             {
                 data.writeByte (0);
-                const char* text = static_cast<const char*> (data.getData());
+                auto* text = static_cast<const char*> (data.getData());
 
                 if (CharPointer_UTF16::isByteOrderMarkBigEndian (text)
                       || CharPointer_UTF16::isByteOrderMarkLittleEndian (text))
@@ -138,6 +147,15 @@ XmlElement* XmlDocument::getDocumentElement (const bool onlyReadOuterDocumentEle
     return parseDocumentElement (originalText.getCharPointer(), onlyReadOuterDocumentElement);
 }
 
+std::unique_ptr<XmlElement> XmlDocument::getDocumentElementIfTagMatches (StringRef requiredTag)
+{
+    if (auto xml = getDocumentElement (true))
+        if (xml->hasTagName (requiredTag))
+            return getDocumentElement (false);
+
+    return {};
+}
+
 const String& XmlDocument::getLastParseError() const noexcept
 {
     return lastError;
@@ -153,7 +171,7 @@ String XmlDocument::getFileContents (const String& filename) const
 {
     if (inputSource != nullptr)
     {
-        const ScopedPointer<InputStream> in (inputSource->createInputStreamFor (filename.trim().unquoted()));
+        std::unique_ptr<InputStream> in (inputSource->createInputStreamFor (filename.trim().unquoted()));
 
         if (in != nullptr)
             return in->readEntireStreamAsString();
@@ -164,7 +182,7 @@ String XmlDocument::getFileContents (const String& filename) const
 
 juce_wchar XmlDocument::readNextChar() noexcept
 {
-    const juce_wchar c = input.getAndAdvance();
+    auto c = input.getAndAdvance();
 
     if (c == 0)
     {
@@ -175,8 +193,8 @@ juce_wchar XmlDocument::readNextChar() noexcept
     return c;
 }
 
-XmlElement* XmlDocument::parseDocumentElement (String::CharPointerType textToParse,
-                                               const bool onlyReadOuterDocumentElement)
+std::unique_ptr<XmlElement> XmlDocument::parseDocumentElement (String::CharPointerType textToParse,
+                                                               bool onlyReadOuterDocumentElement)
 {
     input = textToParse;
     errorOccurred = false;
@@ -198,14 +216,13 @@ XmlElement* XmlDocument::parseDocumentElement (String::CharPointerType textToPar
     else
     {
         lastError.clear();
-
-        ScopedPointer<XmlElement> result (readNextElement (! onlyReadOuterDocumentElement));
+        std::unique_ptr<XmlElement> result (readNextElement (! onlyReadOuterDocumentElement));
 
         if (! errorOccurred)
-            return result.release();
+            return result;
     }
 
-    return nullptr;
+    return {};
 }
 
 bool XmlDocument::parseHeader()
@@ -214,17 +231,18 @@ bool XmlDocument::parseHeader()
 
     if (CharacterFunctions::compareUpTo (input, CharPointer_ASCII ("<?xml"), 5) == 0)
     {
-        const String::CharPointerType headerEnd (CharacterFunctions::find (input, CharPointer_ASCII ("?>")));
+        auto headerEnd = CharacterFunctions::find (input, CharPointer_ASCII ("?>"));
 
         if (headerEnd.isEmpty())
             return false;
 
        #if JUCE_DEBUG
-        const String encoding (String (input, headerEnd)
-                                 .fromFirstOccurrenceOf ("encoding", false, true)
-                                 .fromFirstOccurrenceOf ("=", false, false)
-                                 .fromFirstOccurrenceOf ("\"", false, false)
-                                 .upToFirstOccurrenceOf ("\"", false, false).trim());
+        auto encoding = String (input, headerEnd)
+                          .fromFirstOccurrenceOf ("encoding", false, true)
+                          .fromFirstOccurrenceOf ("=", false, false)
+                          .fromFirstOccurrenceOf ("\"", false, false)
+                          .upToFirstOccurrenceOf ("\"", false, false)
+                          .trim();
 
         /* If you load an XML document with a non-UTF encoding type, it may have been
            loaded wrongly.. Since all the files are read via the normal juce file streams,
@@ -248,11 +266,11 @@ bool XmlDocument::parseDTD()
     if (CharacterFunctions::compareUpTo (input, CharPointer_ASCII ("<!DOCTYPE"), 9) == 0)
     {
         input += 9;
-        const String::CharPointerType dtdStart (input);
+        auto dtdStart = input;
 
         for (int n = 1; n > 0;)
         {
-            const juce_wchar c = readNextChar();
+            auto c = readNextChar();
 
             if (outOfData)
                 return false;
@@ -288,7 +306,7 @@ void XmlDocument::skipNextWhiteSpace()
                  && input[3] == '-')
             {
                 input += 4;
-                const int closeComment = input.indexOf (CharPointer_ASCII ("-->"));
+                auto closeComment = input.indexOf (CharPointer_ASCII ("-->"));
 
                 if (closeComment < 0)
                 {
@@ -303,7 +321,7 @@ void XmlDocument::skipNextWhiteSpace()
             if (input[1] == '?')
             {
                 input += 2;
-                const int closeBracket = input.indexOf (CharPointer_ASCII ("?>"));
+                auto closeBracket = input.indexOf (CharPointer_ASCII ("?>"));
 
                 if (closeBracket < 0)
                 {
@@ -322,11 +340,11 @@ void XmlDocument::skipNextWhiteSpace()
 
 void XmlDocument::readQuotedString (String& result)
 {
-    const juce_wchar quote = readNextChar();
+    auto quote = readNextChar();
 
     while (! outOfData)
     {
-        const juce_wchar c = readNextChar();
+        auto c = readNextChar();
 
         if (c == quote)
             break;
@@ -339,11 +357,11 @@ void XmlDocument::readQuotedString (String& result)
         }
         else
         {
-            const String::CharPointerType start (input);
+            auto start = input;
 
             for (;;)
             {
-                const juce_wchar character = *input;
+                auto character = *input;
 
                 if (character == quote)
                 {
@@ -351,12 +369,14 @@ void XmlDocument::readQuotedString (String& result)
                     ++input;
                     return;
                 }
-                else if (character == '&')
+
+                if (character == '&')
                 {
                     result.appendCharPointer (start, input);
                     break;
                 }
-                else if (character == 0)
+
+                if (character == 0)
                 {
                     setLastError ("unmatched quotes", false);
                     outOfData = true;
@@ -372,15 +392,15 @@ void XmlDocument::readQuotedString (String& result)
 XmlElement* XmlDocument::readNextElement (const bool alsoParseSubElements)
 {
     XmlElement* node = nullptr;
-
     skipNextWhiteSpace();
+
     if (outOfData)
         return nullptr;
 
     if (*input == '<')
     {
         ++input;
-        String::CharPointerType endOfToken (XmlIdentifierChars::findEndOfToken (input));
+        auto endOfToken = XmlIdentifierChars::findEndOfToken (input);
 
         if (endOfToken == input)
         {
@@ -403,8 +423,7 @@ XmlElement* XmlDocument::readNextElement (const bool alsoParseSubElements)
         for (;;)
         {
             skipNextWhiteSpace();
-
-            const juce_wchar c = *input;
+            auto c = *input;
 
             // empty tag..
             if (c == '/' && input[1] == '>')
@@ -427,26 +446,22 @@ XmlElement* XmlDocument::readNextElement (const bool alsoParseSubElements)
             // get an attribute..
             if (XmlIdentifierChars::isIdentifierChar (c))
             {
-                String::CharPointerType attNameEnd (XmlIdentifierChars::findEndOfToken (input));
+                auto attNameEnd = XmlIdentifierChars::findEndOfToken (input);
 
                 if (attNameEnd != input)
                 {
-                    const String::CharPointerType attNameStart (input);
+                    auto attNameStart = input;
                     input = attNameEnd;
-
                     skipNextWhiteSpace();
 
                     if (readNextChar() == '=')
                     {
                         skipNextWhiteSpace();
-
-                        const juce_wchar nextChar = *input;
+                        auto nextChar = *input;
 
                         if (nextChar == '"' || nextChar == '\'')
                         {
-                            XmlElement::XmlAttributeNode* const newAtt
-                                = new XmlElement::XmlAttributeNode (attNameStart, attNameEnd);
-
+                            auto* newAtt = new XmlElement::XmlAttributeNode (attNameStart, attNameEnd);
                             readQuotedString (newAtt->value);
                             attributeAppender.append (newAtt);
                             continue;
@@ -479,7 +494,7 @@ void XmlDocument::readChildElements (XmlElement& parent)
 
     for (;;)
     {
-        const String::CharPointerType preWhitespaceInput (input);
+        auto preWhitespaceInput = input;
         skipNextWhiteSpace();
 
         if (outOfData)
@@ -490,12 +505,12 @@ void XmlDocument::readChildElements (XmlElement& parent)
 
         if (*input == '<')
         {
-            const juce_wchar c1 = input[1];
+            auto c1 = input[1];
 
             if (c1 == '/')
             {
                 // our close tag..
-                const int closeTag = input.indexOf ((juce_wchar) '>');
+                auto closeTag = input.indexOf ((juce_wchar) '>');
 
                 if (closeTag >= 0)
                     input += closeTag + 1;
@@ -506,11 +521,11 @@ void XmlDocument::readChildElements (XmlElement& parent)
             if (c1 == '!' && CharacterFunctions::compareUpTo (input + 2, CharPointer_ASCII ("[CDATA["), 7) == 0)
             {
                 input += 9;
-                const String::CharPointerType inputStart (input);
+                auto inputStart = input;
 
                 for (;;)
                 {
-                    const juce_wchar c0 = *input;
+                    auto c0 = *input;
 
                     if (c0 == 0)
                     {
@@ -518,9 +533,8 @@ void XmlDocument::readChildElements (XmlElement& parent)
                         outOfData = true;
                         break;
                     }
-                    else if (c0 == ']'
-                              && input[1] == ']'
-                              && input[2] == '>')
+
+                    if (c0 == ']' && input[1] == ']' && input[2] == '>')
                     {
                         childAppender.append (XmlElement::createTextElement (String (inputStart, input)));
                         input += 3;
@@ -533,7 +547,7 @@ void XmlDocument::readChildElements (XmlElement& parent)
             else
             {
                 // this is some other element, so parse and add it..
-                if (XmlElement* const n = readNextElement (true))
+                if (auto* n = readNextElement (true))
                     childAppender.append (n);
                 else
                     break;
@@ -547,14 +561,14 @@ void XmlDocument::readChildElements (XmlElement& parent)
 
             for (;;)
             {
-                const juce_wchar c = *input;
+                auto c = *input;
 
                 if (c == '<')
                 {
                     if (input[1] == '!' && input[2] == '-' && input[3] == '-')
                     {
                         input += 4;
-                        const int closeComment = input.indexOf (CharPointer_ASCII ("-->"));
+                        auto closeComment = input.indexOf (CharPointer_ASCII ("-->"));
 
                         if (closeComment < 0)
                         {
@@ -584,13 +598,13 @@ void XmlDocument::readChildElements (XmlElement& parent)
 
                     if (entity.startsWithChar ('<') && entity [1] != 0)
                     {
-                        const String::CharPointerType oldInput (input);
-                        const bool oldOutOfData = outOfData;
+                        auto oldInput = input;
+                        auto oldOutOfData = outOfData;
 
                         input = entity.getCharPointer();
                         outOfData = false;
 
-                        while (XmlElement* n = readNextElement (true))
+                        while (auto* n = readNextElement (true))
                             childAppender.append (n);
 
                         input = oldInput;
@@ -606,7 +620,7 @@ void XmlDocument::readChildElements (XmlElement& parent)
                 {
                     for (;; ++input)
                     {
-                        juce_wchar nextChar = *input;
+                        auto nextChar = *input;
 
                         if (nextChar == '\r')
                         {
@@ -680,7 +694,7 @@ void XmlDocument::readEntity (String& result)
 
             while (input[0] != ';')
             {
-                const int hexValue = CharacterFunctions::getHexDigitValue (input[0]);
+                auto hexValue = CharacterFunctions::getHexDigitValue (input[0]);
 
                 if (hexValue < 0 || ++numChars > 8)
                 {
@@ -723,8 +737,8 @@ void XmlDocument::readEntity (String& result)
     }
     else
     {
-        const String::CharPointerType entityNameStart (input);
-        const int closingSemiColon = input.indexOf ((juce_wchar) ';');
+        auto entityNameStart = input;
+        auto closingSemiColon = input.indexOf ((juce_wchar) ';');
 
         if (closingSemiColon < 0)
         {
@@ -734,7 +748,6 @@ void XmlDocument::readEntity (String& result)
         else
         {
             input += closingSemiColon + 1;
-
             result += expandExternalEntity (String (entityNameStart, (size_t) closingSemiColon));
         }
     }
@@ -750,7 +763,7 @@ String XmlDocument::expandEntity (const String& ent)
 
     if (ent[0] == '#')
     {
-        const juce_wchar char1 = ent[1];
+        auto char1 = ent[1];
 
         if (char1 == 'x' || char1 == 'X')
             return String::charToString (static_cast<juce_wchar> (ent.substring (2).getHexValue32()));
@@ -774,10 +787,10 @@ String XmlDocument::expandExternalEntity (const String& entity)
             dtdText = dtdText.trimCharactersAtEnd (">");
             tokenisedDTD.addTokens (dtdText, true);
 
-            if (tokenisedDTD [tokenisedDTD.size() - 2].equalsIgnoreCase ("system")
-                 && tokenisedDTD [tokenisedDTD.size() - 1].isQuotedString())
+            if (tokenisedDTD[tokenisedDTD.size() - 2].equalsIgnoreCase ("system")
+                 && tokenisedDTD[tokenisedDTD.size() - 1].isQuotedString())
             {
-                const String fn (tokenisedDTD [tokenisedDTD.size() - 1]);
+                auto fn = tokenisedDTD[tokenisedDTD.size() - 1];
 
                 tokenisedDTD.clear();
                 tokenisedDTD.addTokens (getFileContents (fn), true);
@@ -785,11 +798,11 @@ String XmlDocument::expandExternalEntity (const String& entity)
             else
             {
                 tokenisedDTD.clear();
-                const int openBracket = dtdText.indexOfChar ('[');
+                auto openBracket = dtdText.indexOfChar ('[');
 
                 if (openBracket > 0)
                 {
-                    const int closeBracket = dtdText.lastIndexOfChar (']');
+                    auto closeBracket = dtdText.lastIndexOfChar (']');
 
                     if (closeBracket > openBracket)
                         tokenisedDTD.addTokens (dtdText.substring (openBracket + 1,
@@ -802,7 +815,7 @@ String XmlDocument::expandExternalEntity (const String& entity)
                 if (tokenisedDTD[i].startsWithChar ('%')
                      && tokenisedDTD[i].endsWithChar (';'))
                 {
-                    const String parsed (getParameterEntity (tokenisedDTD[i].substring (1, tokenisedDTD[i].length() - 1)));
+                    auto parsed = getParameterEntity (tokenisedDTD[i].substring (1, tokenisedDTD[i].length() - 1));
                     StringArray newToks;
                     newToks.addTokens (parsed, true);
 
@@ -823,14 +836,14 @@ String XmlDocument::expandExternalEntity (const String& entity)
         {
             if (tokenisedDTD[i - 1].equalsIgnoreCase ("<!entity"))
             {
-                String ent (tokenisedDTD [i + 1].trimCharactersAtEnd (">").trim().unquoted());
+                auto ent = tokenisedDTD [i + 1].trimCharactersAtEnd (">").trim().unquoted();
 
                 // check for sub-entities..
-                int ampersand = ent.indexOfChar ('&');
+                auto ampersand = ent.indexOfChar ('&');
 
                 while (ampersand >= 0)
                 {
-                    const int semiColon = ent.indexOf (i + 1, ";");
+                    auto semiColon = ent.indexOf (i + 1, ";");
 
                     if (semiColon < 0)
                     {
@@ -838,7 +851,7 @@ String XmlDocument::expandExternalEntity (const String& entity)
                         break;
                     }
 
-                    const String resolved (expandEntity (ent.substring (i + 1, semiColon)));
+                    auto resolved = expandEntity (ent.substring (i + 1, semiColon));
 
                     ent = ent.substring (0, ampersand)
                            + resolved
@@ -853,7 +866,6 @@ String XmlDocument::expandExternalEntity (const String& entity)
     }
 
     setLastError ("unknown entity", true);
-
     return entity;
 }
 
@@ -865,7 +877,7 @@ String XmlDocument::getParameterEntity (const String& entity)
              && tokenisedDTD [i - 1] == "%"
              && tokenisedDTD [i - 2].equalsIgnoreCase ("<!entity"))
         {
-            const String ent (tokenisedDTD [i + 1].trimCharactersAtEnd (">"));
+            auto ent = tokenisedDTD [i + 1].trimCharactersAtEnd (">");
 
             if (ent.equalsIgnoreCase ("system"))
                 return getFileContents (tokenisedDTD [i + 2].trimCharactersAtEnd (">"));
