@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -32,6 +31,8 @@ namespace juce
     Manages a rectangle and allows geometric operations to be performed on it.
 
     @see RectangleList, Path, Line, Point
+
+    @tags{Graphics}
 */
 template <typename ValueType>
 class Rectangle
@@ -41,16 +42,10 @@ public:
     /** Creates a rectangle of zero size.
         The default coordinates will be (0, 0, 0, 0).
     */
-    Rectangle() noexcept
-      : w(), h()
-    {
-    }
+    Rectangle() = default;
 
     /** Creates a copy of another rectangle. */
-    Rectangle (const Rectangle& other) noexcept
-      : pos (other.pos), w (other.w), h (other.h)
-    {
-    }
+    Rectangle (const Rectangle&) = default;
 
     /** Creates a rectangle with a given position and size. */
     Rectangle (ValueType initialX, ValueType initialY,
@@ -87,15 +82,11 @@ public:
         return { left, top, right - left, bottom - top };
     }
 
-    Rectangle& operator= (const Rectangle& other) noexcept
-    {
-        pos = other.pos;
-        w = other.w; h = other.h;
-        return *this;
-    }
+    /** Creates a copy of another rectangle. */
+    Rectangle& operator= (const Rectangle&) = default;
 
     /** Destructor. */
-    ~Rectangle() noexcept {}
+    ~Rectangle() = default;
 
     //==============================================================================
     /** Returns true if the rectangle's width or height are zero or less */
@@ -223,13 +214,13 @@ public:
                                                                                                                newCentre.y - h / (ValueType) 2, w, h }; }
 
     /** Returns a rectangle which has the same position and height as this one, but with a different width. */
-    Rectangle withWidth (ValueType newWidth) const noexcept                                         { return { pos.x, pos.y, newWidth, h }; }
+    Rectangle withWidth (ValueType newWidth) const noexcept                                         { return { pos.x, pos.y, jmax (ValueType(), newWidth), h }; }
 
     /** Returns a rectangle which has the same position and width as this one, but with a different height. */
-    Rectangle withHeight (ValueType newHeight) const noexcept                                       { return { pos.x, pos.y, w, newHeight }; }
+    Rectangle withHeight (ValueType newHeight) const noexcept                                       { return { pos.x, pos.y, w, jmax (ValueType(), newHeight) }; }
 
     /** Returns a rectangle with the same top-left position as this one, but a new size. */
-    Rectangle withSize (ValueType newWidth, ValueType newHeight) const noexcept                     { return { pos.x, pos.y, newWidth, newHeight }; }
+    Rectangle withSize (ValueType newWidth, ValueType newHeight) const noexcept                     { return { pos.x, pos.y, jmax (ValueType(), newWidth), jmax (ValueType(), newHeight) }; }
 
     /** Returns a rectangle with the same centre position as this one, but a new size. */
     Rectangle withSizeKeepingCentre (ValueType newWidth, ValueType newHeight) const noexcept        { return { pos.x + (w - newWidth)  / (ValueType) 2,
@@ -358,10 +349,10 @@ public:
     template <typename FloatType>
     Rectangle operator*= (FloatType scaleFactor) noexcept
     {
-        Rectangle<FloatType> (pos.x * scaleFactor,
-                              pos.y * scaleFactor,
-                              w * scaleFactor,
-                              h * scaleFactor).copyWithRounding (*this);
+        Rectangle<FloatType> ((FloatType) pos.x * scaleFactor,
+                              (FloatType) pos.y * scaleFactor,
+                              (FloatType) w * scaleFactor,
+                              (FloatType) h * scaleFactor).copyWithRounding (*this);
         return *this;
     }
 
@@ -373,10 +364,10 @@ public:
     template <typename FloatType>
     Rectangle operator*= (Point<FloatType> scaleFactor) noexcept
     {
-        Rectangle<FloatType> (pos.x * scaleFactor.x,
-                              pos.y * scaleFactor.y,
-                              w * scaleFactor.x,
-                              h * scaleFactor.y).copyWithRounding (*this);
+        Rectangle<FloatType> ((FloatType) pos.x * scaleFactor.x,
+                              (FloatType) pos.y * scaleFactor.y,
+                              (FloatType) w * scaleFactor.x,
+                              (FloatType) h * scaleFactor.y).copyWithRounding (*this);
         return *this;
     }
 
@@ -393,10 +384,10 @@ public:
     template <typename FloatType>
     Rectangle operator/= (FloatType scaleFactor) noexcept
     {
-        Rectangle<FloatType> (pos.x / scaleFactor,
-                              pos.y / scaleFactor,
-                              w / scaleFactor,
-                              h / scaleFactor).copyWithRounding (*this);
+        Rectangle<FloatType> ((FloatType) pos.x / scaleFactor,
+                              (FloatType) pos.y / scaleFactor,
+                              (FloatType) w / scaleFactor,
+                              (FloatType) h / scaleFactor).copyWithRounding (*this);
         return *this;
     }
 
@@ -404,10 +395,10 @@ public:
     template <typename FloatType>
     Rectangle operator/= (Point<FloatType> scaleFactor) noexcept
     {
-        Rectangle<FloatType> (pos.x / scaleFactor.x,
-                              pos.y / scaleFactor.y,
-                              w / scaleFactor.x,
-                              h / scaleFactor.y).copyWithRounding (*this);
+        Rectangle<FloatType> ((FloatType) pos.x / scaleFactor.x,
+                              (FloatType) pos.y / scaleFactor.y,
+                              (FloatType) w / scaleFactor.x,
+                              (FloatType) h / scaleFactor.y).copyWithRounding (*this);
         return *this;
     }
 
@@ -515,7 +506,7 @@ public:
         by the specified amount and returning the section that was removed.
 
         E.g. if this rectangle is (100, 100, 300, 300) and amountToRemove is 50, this will
-        return (250, 100, 50, 300) and leave this rectangle as (100, 100, 250, 300).
+        return (350, 100, 50, 300) and leave this rectangle as (100, 100, 250, 300).
 
         If amountToRemove is greater than the width of this rectangle, it'll be clipped to
         that value.
@@ -532,7 +523,7 @@ public:
         by the specified amount and returning the section that was removed.
 
         E.g. if this rectangle is (100, 100, 300, 300) and amountToRemove is 50, this will
-        return (100, 250, 300, 50) and leave this rectangle as (100, 100, 300, 250).
+        return (100, 350, 300, 50) and leave this rectangle as (100, 100, 300, 250).
 
         If amountToRemove is greater than the height of this rectangle, it'll be clipped to
         that value.
@@ -561,22 +552,22 @@ public:
     template <typename FloatType>
     Point<ValueType> getRelativePoint (FloatType relativeX, FloatType relativeY) const noexcept
     {
-        return { pos.x + static_cast<ValueType> (w * relativeX),
-                 pos.y + static_cast<ValueType> (h * relativeY) };
+        return { pos.x + static_cast<ValueType> ((FloatType) w * relativeX),
+                 pos.y + static_cast<ValueType> ((FloatType) h * relativeY) };
     }
 
     /** Returns a proportion of the width of this rectangle. */
     template <typename FloatType>
     ValueType proportionOfWidth (FloatType proportion) const noexcept
     {
-        return static_cast<ValueType> (w * proportion);
+        return static_cast<ValueType> ((FloatType) w * proportion);
     }
 
     /** Returns a proportion of the height of this rectangle. */
     template <typename FloatType>
     ValueType proportionOfHeight (FloatType proportion) const noexcept
     {
-        return static_cast<ValueType> (h * proportion);
+        return static_cast<ValueType> ((FloatType) h * proportion);
     }
 
     /** Returns a rectangle based on some proportional coordinates relative to this one.
@@ -759,10 +750,11 @@ public:
 
         switch (inside)
         {
-            case 1 + 2 + 8:     w = r - otherR; pos.x = otherR; return true;
-            case 1 + 2 + 4:     h = b - otherB; pos.y = otherB; return true;
-            case 2 + 4 + 8:     w = other.pos.x - pos.x; return true;
-            case 1 + 4 + 8:     h = other.pos.y - pos.y; return true;
+            case 1 + 2 + 8:  w = r - otherR; pos.x = otherR; return true;
+            case 1 + 2 + 4:  h = b - otherB; pos.y = otherB; return true;
+            case 2 + 4 + 8:  w = other.pos.x - pos.x;        return true;
+            case 1 + 4 + 8:  h = other.pos.y - pos.y;        return true;
+            default:         break;
         }
 
         return false;
@@ -777,12 +769,13 @@ public:
     */
     Rectangle constrainedWithin (Rectangle areaToFitWithin) const noexcept
     {
-        auto newW = jmin (w, areaToFitWithin.getWidth());
-        auto newH = jmin (h, areaToFitWithin.getHeight());
+        auto newPos = areaToFitWithin.withSize (areaToFitWithin.getWidth() - w,
+                                                areaToFitWithin.getHeight() - h)
+                        .getConstrainedPoint (pos);
 
-        return { jlimit (areaToFitWithin.getX(), areaToFitWithin.getRight()  - newW, pos.x),
-                 jlimit (areaToFitWithin.getY(), areaToFitWithin.getBottom() - newH, pos.y),
-                 newW, newH };
+        return { newPos.x, newPos.y,
+                 jmin (w, areaToFitWithin.getWidth()),
+                 jmin (h, areaToFitWithin.getHeight()) };
     }
 
     /** Returns the smallest rectangle that can contain the shape created by applying
@@ -792,7 +785,7 @@ public:
     */
     Rectangle transformedBy (const AffineTransform& transform) const noexcept
     {
-        typedef typename TypeHelpers::SmallestFloatType<ValueType>::type FloatType;
+        using FloatType = typename TypeHelpers::SmallestFloatType<ValueType>::type;
 
         auto x1 = static_cast<FloatType> (pos.x),     y1 = static_cast<FloatType> (pos.y);
         auto x2 = static_cast<FloatType> (pos.x + w), y2 = static_cast<FloatType> (pos.y);
@@ -814,7 +807,7 @@ public:
 
     /** Returns the smallest integer-aligned rectangle that completely contains this one.
         This is only relevant for floating-point rectangles, of course.
-        @see toFloat(), toNearestInt()
+        @see toFloat(), toNearestInt(), toNearestIntEdges()
     */
     Rectangle<int> getSmallestIntegerContainer() const noexcept
     {
@@ -827,7 +820,7 @@ public:
     /** Casts this rectangle to a Rectangle<int>.
         This uses roundToInt to snap x, y, width and height to the nearest integer (losing precision).
         If the rectangle already uses integers, this will simply return a copy.
-        @see getSmallestIntegerContainer()
+        @see getSmallestIntegerContainer(), toNearestIntEdges()
     */
     Rectangle<int> toNearestInt() const noexcept
     {
@@ -835,6 +828,17 @@ public:
                  roundToInt (w),     roundToInt (h) };
     }
 
+    /** Casts this rectangle to a Rectangle<int>.
+        This uses roundToInt to snap top, left, right and bottom to the nearest integer (losing precision).
+        If the rectangle already uses integers, this will simply return a copy.
+        @see getSmallestIntegerContainer(), toNearestInt()
+    */
+    Rectangle<int> toNearestIntEdges() const noexcept
+    {
+        return Rectangle<int>::leftTopRightBottom (roundToInt (pos.x),       roundToInt (pos.y),
+                                                   roundToInt (getRight()),  roundToInt (getBottom()));
+    }
+
     /** Casts this rectangle to a Rectangle<float>.
         @see getSmallestIntegerContainer
     */
@@ -961,7 +965,7 @@ private:
     template <typename OtherType> friend class Rectangle;
 
     Point<ValueType> pos;
-    ValueType w, h;
+    ValueType w {}, h {};
 
     static ValueType parseIntAfterSpace (StringRef s) noexcept
         { return static_cast<ValueType> (s.text.findEndOfWhitespace().getIntValue32()); }
