@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -36,16 +35,18 @@ namespace juce
     adjacent rectangles.
 
     @see Rectangle
+
+    @tags{Graphics}
 */
 template <typename ValueType>
-class RectangleList
+class RectangleList  final
 {
 public:
-    typedef Rectangle<ValueType> RectangleType;
+    using RectangleType = Rectangle<ValueType>;
 
     //==============================================================================
     /** Creates an empty RectangleList */
-    RectangleList() noexcept {}
+    RectangleList() = default;
 
     /** Creates a copy of another list */
     RectangleList (const RectangleList& other)  : rects (other.rects)
@@ -53,7 +54,7 @@ public:
     }
 
     /** Creates a list containing just one rectangle. */
-    RectangleList (const RectangleType& rect)
+    RectangleList (RectangleType rect)
     {
         addWithoutMerging (rect);
     }
@@ -67,14 +68,14 @@ public:
 
     /** Move constructor */
     RectangleList (RectangleList&& other) noexcept
-        : rects (static_cast<Array<RectangleType>&&> (other.rects))
+        : rects (std::move (other.rects))
     {
     }
 
     /** Move assignment operator */
     RectangleList& operator= (RectangleList&& other) noexcept
     {
-        rects = static_cast<Array<RectangleType>&&> (other.rects);
+        rects = std::move (other.rects);
         return *this;
     }
 
@@ -106,7 +107,7 @@ public:
         The rectangle can have any size and may be empty, but if it's floating point
         then it's expected to not contain any INF values.
     */
-    void add (const RectangleType& rect)
+    void add (RectangleType rect)
     {
         jassert (rect.isFinite()); // You must provide a valid rectangle to this method!
 
@@ -176,7 +177,7 @@ public:
         The rectangle can have any size and may be empty, but if it's floating point
         then it's expected to not contain any INF values.
     */
-    void addWithoutMerging (const RectangleType& rect)
+    void addWithoutMerging (RectangleType rect)
     {
         jassert (rect.isFinite()); // You must provide a valid rectangle to this method!
 
@@ -192,7 +193,7 @@ public:
     void add (const RectangleList& other)
     {
         for (auto& r : other)
-            add (*r);
+            add (r);
     }
 
     /** Removes a rectangular region from the list.
@@ -200,7 +201,7 @@ public:
         Any rectangles in the list which overlap this will be clipped and subdivided
         if necessary.
     */
-    void subtract (const RectangleType& rect)
+    void subtract (RectangleType rect)
     {
         if (auto numRects = rects.size())
         {
@@ -310,7 +311,7 @@ public:
 
         @see getIntersectionWith
     */
-    bool clipTo (const RectangleType& rect)
+    bool clipTo (RectangleType rect)
     {
         jassert (rect.isFinite()); // You must provide a valid rectangle to this method!
 
@@ -377,14 +378,14 @@ public:
 
         @see clipTo
     */
-    bool getIntersectionWith (const RectangleType& rect, RectangleList& destRegion) const
+    bool getIntersectionWith (RectangleType rect, RectangleList& destRegion) const
     {
         jassert (rect.isFinite()); // You must provide a valid rectangle to this method!
 
         destRegion.clear();
 
         if (! rect.isEmpty())
-            for (auto& r : rects)
+            for (auto r : rects)
                 if (rect.intersectRectangle (r))
                     destRegion.rects.add (r);
 
@@ -428,7 +429,7 @@ public:
                     defined by this object
         @see intersectsRectangle, containsPoint
     */
-    bool containsRectangle (const RectangleType& rectangleToCheck) const
+    bool containsRectangle (RectangleType rectangleToCheck) const
     {
         if (rects.size() > 1)
         {
@@ -456,7 +457,7 @@ public:
                     defined by this object
         @see containsRectangle
     */
-    bool intersectsRectangle (const RectangleType& rectangleToCheck) const noexcept
+    bool intersectsRectangle (RectangleType rectangleToCheck) const noexcept
     {
         for (auto& r : rects)
             if (r.intersects (rectangleToCheck))
@@ -466,7 +467,6 @@ public:
     }
 
     /** Checks whether this region intersects any part of another one.
-
         @see intersectsRectangle
     */
     bool intersects (const RectangleList& other) const noexcept
