@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -27,20 +26,16 @@
 namespace juce
 {
 
-#if JUCE_MSVC
- #pragma warning (push)
- #pragma warning (disable: 4390 4611 4365 4267)
- #ifdef __INTEL_COMPILER
-  #pragma warning (disable: 2544 2545)
- #endif
-#endif
+JUCE_BEGIN_IGNORE_WARNINGS_MSVC (4390 4611 4365 4267 4616 2544 2545)
 
 namespace zlibNamespace
 {
 #if JUCE_INCLUDE_ZLIB_CODE
   #undef OS_CODE
   #undef fdopen
-  #include "../../juce_core/zip/zlib/zlib.h"
+  #define ZLIB_INTERNAL
+  #define NO_DUMMY_DECL
+  #include <juce_core/zip/zlib/zlib.h>
   #undef OS_CODE
 #else
   #include JUCE_ZLIB_INCLUDE_PATH
@@ -63,16 +58,11 @@ namespace pnglibNamespace
    using std::free;
   #endif
 
-  #if JUCE_CLANG
-   #pragma clang diagnostic push
-   #pragma clang diagnostic ignored "-Wsign-conversion"
-   #if __has_warning("-Wcomma")
-    #pragma clang diagnostic ignored "-Wcomma"
-   #endif
-  #elif defined(__GNUC__) && (__GNUC__ >= 7)
-   #pragma GCC diagnostic push
-   #pragma GCC diagnostic ignored "-Wimplicit-fallthrough"
-  #endif
+   JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wsign-conversion",
+                                        "-Wimplicit-fallthrough",
+                                        "-Wtautological-constant-out-of-range-compare",
+                                        "-Wzero-as-null-pointer-constant",
+                                        "-Wcomma")
 
   #undef check
   using std::abs;
@@ -87,7 +77,6 @@ namespace pnglibNamespace
   #define PNG_ALIGNED_MEMORY_SUPPORTED
   #define PNG_BENIGN_ERRORS_SUPPORTED
   #define PNG_BENIGN_READ_ERRORS_SUPPORTED
-  #define PNG_BUILD_GRAYSCALE_PALETTE_SUPPORTED
   #define PNG_CHECK_FOR_INVALID_INDEX_SUPPORTED
   #define PNG_COLORSPACE_SUPPORTED
   #define PNG_CONSOLE_IO_SUPPORTED
@@ -103,7 +92,6 @@ namespace pnglibNamespace
   #define PNG_INCH_CONVERSIONS_SUPPORTED
   #define PNG_INFO_IMAGE_SUPPORTED
   #define PNG_IO_STATE_SUPPORTED
-  #define PNG_MNG_FEATURES_SUPPORTED
   #define PNG_POINTER_INDEXING_SUPPORTED
   #define PNG_PROGRESSIVE_READ_SUPPORTED
   #define PNG_READ_16BIT_SUPPORTED
@@ -267,6 +255,19 @@ namespace pnglibNamespace
   #define PNG_sCAL_PRECISION 5
   #define PNG_sRGB_PROFILE_CHECKS 2
 
+  #define PNG_LINKAGE_API
+  #define PNG_LINKAGE_FUNCTION
+
+  #define PNG_ARM_NEON_OPT 0
+
+  #if ! defined (PNG_USER_WIDTH_MAX)
+   #define PNG_USER_WIDTH_MAX 1000000
+  #endif
+
+  #if ! defined (PNG_USER_HEIGHT_MAX)
+   #define PNG_USER_HEIGHT_MAX 1000000
+  #endif
+
   #define png_debug(a, b)
   #define png_debug1(a, b, c)
   #define png_debug2(a, b, c, d)
@@ -282,6 +283,24 @@ namespace pnglibNamespace
   #include "pnglib/pngread.c"
   #include "pnglib/pngpread.c"
   #include "pnglib/pngrio.c"
+
+  void png_do_expand_palette (png_row_infop, png_bytep, png_const_colorp, png_const_bytep, int);
+  void png_do_expand (png_row_infop, png_bytep, png_const_color_16p);
+  void png_do_chop (png_row_infop, png_bytep);
+  void png_do_quantize (png_row_infop, png_bytep, png_const_bytep, png_const_bytep);
+  void png_do_gray_to_rgb (png_row_infop, png_bytep);
+  void png_do_unshift (png_row_infop, png_bytep, png_const_color_8p);
+  void png_do_unpack (png_row_infop, png_bytep);
+  int png_do_rgb_to_gray (png_structrp, png_row_infop, png_bytep);
+  void png_do_compose (png_row_infop, png_bytep, png_structrp);
+  void png_do_gamma (png_row_infop, png_bytep, png_structrp);
+  void png_do_encode_alpha (png_row_infop, png_bytep, png_structrp);
+  void png_do_scale_16_to_8 (png_row_infop, png_bytep);
+  void png_do_expand_16 (png_row_infop, png_bytep);
+  void png_do_read_filler (png_row_infop, png_bytep, png_uint_32, png_uint_32);
+  void png_do_read_invert_alpha (png_row_infop, png_bytep);
+  void png_do_read_swap_alpha (png_row_infop, png_bytep);
+
   #include "pnglib/pngrtran.c"
   #include "pnglib/pngrutil.c"
   #include "pnglib/pngset.c"
@@ -291,11 +310,7 @@ namespace pnglibNamespace
   #include "pnglib/pngwtran.c"
   #include "pnglib/pngwutil.c"
 
-  #if JUCE_CLANG
-   #pragma clang diagnostic pop
-  #elif defined(__GNUC__) && (__GNUC__ >= 7)
-   #pragma GCC diagnostic pop
-  #endif
+  JUCE_END_IGNORE_WARNINGS_GCC_LIKE
 
 #else
   extern "C"
@@ -310,9 +325,7 @@ namespace pnglibNamespace
 #undef min
 #undef fdopen
 
-#if JUCE_MSVC
- #pragma warning (pop)
-#endif
+JUCE_END_IGNORE_WARNINGS_MSVC
 
 //==============================================================================
 namespace PNGHelpers
@@ -343,10 +356,7 @@ namespace PNGHelpers
 
     static void JUCE_CDECL warningCallback (png_structp, png_const_charp) {}
 
-   #if JUCE_MSVC
-    #pragma warning (push)
-    #pragma warning (disable: 4611) // (warning about setjmp)
-   #endif
+    JUCE_BEGIN_IGNORE_WARNINGS_MSVC (4611)
 
     static bool readHeader (InputStream& in, png_structp pngReadStruct, png_infop pngInfoStruct, jmp_buf& errorJumpBuf,
                             png_uint_32& width, png_uint_32& height, int& bitDepth, int& colorType, int& interlaceType) noexcept
@@ -361,7 +371,7 @@ namespace PNGHelpers
             png_get_IHDR (pngReadStruct, pngInfoStruct,
                           &width, &height,
                           &bitDepth, &colorType,
-                          &interlaceType, 0, 0);
+                          &interlaceType, nullptr, nullptr);
 
             if (bitDepth == 16)
                 png_set_strip_16 (pngReadStruct);
@@ -398,9 +408,7 @@ namespace PNGHelpers
         return false;
     }
 
-   #if JUCE_MSVC
-    #pragma warning (pop)
-   #endif
+    JUCE_END_IGNORE_WARNINGS_MSVC
 
     static Image createImageFromData (bool hasAlphaChan, int width, int height, png_bytepp rows)
     {
@@ -475,16 +483,16 @@ namespace PNGHelpers
 
     static Image readImage (InputStream& in)
     {
-        if (png_structp pngReadStruct = png_create_read_struct (PNG_LIBPNG_VER_STRING, 0, 0, 0))
+        if (png_structp pngReadStruct = png_create_read_struct (PNG_LIBPNG_VER_STRING, nullptr, nullptr, nullptr))
         {
             if (png_infop pngInfoStruct = png_create_info_struct (pngReadStruct))
             {
                 Image image (readImage (in, pngReadStruct, pngInfoStruct));
-                png_destroy_read_struct (&pngReadStruct, &pngInfoStruct, 0);
+                png_destroy_read_struct (&pngReadStruct, &pngInfoStruct, nullptr);
                 return image;
             }
 
-            png_destroy_read_struct (&pngReadStruct, 0, 0);
+            png_destroy_read_struct (&pngReadStruct, nullptr, nullptr);
         }
 
         return Image();
@@ -526,23 +534,23 @@ Image PNGImageFormat::decodeImage (InputStream& in)
 bool PNGImageFormat::writeImageToStream (const Image& image, OutputStream& out)
 {
     using namespace pnglibNamespace;
-    const int width = image.getWidth();
-    const int height = image.getHeight();
+    auto width = image.getWidth();
+    auto height = image.getHeight();
 
-    png_structp pngWriteStruct = png_create_write_struct (PNG_LIBPNG_VER_STRING, 0, 0, 0);
+    auto pngWriteStruct = png_create_write_struct (PNG_LIBPNG_VER_STRING, nullptr, nullptr, nullptr);
 
     if (pngWriteStruct == nullptr)
         return false;
 
-    png_infop pngInfoStruct = png_create_info_struct (pngWriteStruct);
+    auto pngInfoStruct = png_create_info_struct (pngWriteStruct);
 
     if (pngInfoStruct == nullptr)
     {
-        png_destroy_write_struct (&pngWriteStruct, (png_infopp) nullptr);
+        png_destroy_write_struct (&pngWriteStruct, nullptr);
         return false;
     }
 
-    png_set_write_fn (pngWriteStruct, &out, PNGHelpers::writeDataCallback, 0);
+    png_set_write_fn (pngWriteStruct, &out, PNGHelpers::writeDataCallback, nullptr);
 
     png_set_IHDR (pngWriteStruct, pngInfoStruct, (png_uint_32) width, (png_uint_32) height, 8,
                   image.hasAlphaChannel() ? PNG_COLOR_TYPE_RGB_ALPHA
@@ -551,7 +559,7 @@ bool PNGImageFormat::writeImageToStream (const Image& image, OutputStream& out)
                   PNG_COMPRESSION_TYPE_BASE,
                   PNG_FILTER_TYPE_BASE);
 
-    HeapBlock<uint8> rowData ((size_t) width * 4);
+    HeapBlock<uint8> rowData (width * 4);
 
     png_color_8 sig_bit;
     sig_bit.red   = 8;
