@@ -29,7 +29,7 @@ namespace juce
 
     struct AppInactivityCallback // NB: careful, this declaration is duplicated in other modules
     {
-        virtual ~AppInactivityCallback() {}
+        virtual ~AppInactivityCallback() = default;
         virtual void appBecomingInactive() = 0;
     };
 
@@ -434,30 +434,6 @@ void LookAndFeel::playAlertSound()
 }
 
 //==============================================================================
-class iOSMessageBox;
-
-#if defined (__IPHONE_8_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_8_0
- #define JUCE_USE_NEW_IOS_ALERTWINDOW 1
-#endif
-
-#if ! JUCE_USE_NEW_IOS_ALERTWINDOW
-    } // (juce namespace)
-
-    @interface JuceAlertBoxDelegate  : NSObject <UIAlertViewDelegate>
-    {
-    @public
-        iOSMessageBox* owner;
-    }
-
-    - (void) alertView: (UIAlertView*) alertView clickedButtonAtIndex: (NSInteger) buttonIndex;
-
-    @end
-
-    namespace juce
-    {
-#endif
-
-
 class iOSMessageBox
 {
 public:
@@ -466,7 +442,6 @@ public:
                    ModalComponentManager::Callback* cb, const bool async)
         : result (0), resultReceived (false), callback (cb), isAsync (async)
     {
-       #if JUCE_USE_NEW_IOS_ALERTWINDOW
         if (currentlyFocusedPeer != nullptr)
         {
             UIAlertController* alert = [UIAlertController alertControllerWithTitle: juceStringToNS (title)
@@ -486,27 +461,6 @@ public:
             // have at least one window on screen when you use this
             jassertfalse;
         }
-
-       #else
-        delegate = [[JuceAlertBoxDelegate alloc] init];
-        delegate->owner = this;
-
-        alert = [[UIAlertView alloc] initWithTitle: juceStringToNS (title)
-                                           message: juceStringToNS (message)
-                                          delegate: delegate
-                                 cancelButtonTitle: button1
-                                 otherButtonTitles: button2, button3, nil];
-        [alert retain];
-        [alert show];
-       #endif
-    }
-
-    ~iOSMessageBox()
-    {
-       #if ! JUCE_USE_NEW_IOS_ALERTWINDOW
-        [alert release];
-        [delegate release];
-       #endif
     }
 
     int getResult()
@@ -515,11 +469,7 @@ public:
 
         JUCE_AUTORELEASEPOOL
         {
-           #if JUCE_USE_NEW_IOS_ALERTWINDOW
             while (! resultReceived)
-           #else
-            while (! (alert.hidden || resultReceived))
-           #endif
                 [[NSRunLoop mainRunLoop] runUntilDate: [NSDate dateWithTimeIntervalSinceNow: 0.01]];
         }
 
@@ -544,7 +494,6 @@ private:
     std::unique_ptr<ModalComponentManager::Callback> callback;
     const bool isAsync;
 
-   #if JUCE_USE_NEW_IOS_ALERTWINDOW
     void addButton (UIAlertController* alert, NSString* text, int index)
     {
         if (text != nil)
@@ -552,33 +501,10 @@ private:
                                                        style: UIAlertActionStyleDefault
                                                      handler: ^(UIAlertAction*) { this->buttonClicked (index); }]];
     }
-   #else
-    UIAlertView* alert;
-    JuceAlertBoxDelegate* delegate;
-   #endif
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (iOSMessageBox)
 };
 
-
-#if ! JUCE_USE_NEW_IOS_ALERTWINDOW
-    } // (juce namespace)
-
-    @implementation JuceAlertBoxDelegate
-
-    - (void) alertView: (UIAlertView*) alertView clickedButtonAtIndex: (NSInteger) buttonIndex
-    {
-        owner->buttonClicked ((int) buttonIndex);
-        alertView.hidden = true;
-    }
-
-    @end
-
-    namespace juce
-    {
-#endif
-
-
 //==============================================================================
 #if JUCE_MODAL_LOOPS_PERMITTED
 void JUCE_CALLTYPE NativeMessageBox::showMessageBox (AlertWindow::AlertIconType /*iconType*/,
@@ -742,13 +668,9 @@ void Displays::findDisplays (float masterScale)
         UIScreen* s = [UIScreen mainScreen];
 
         Display d;
-        d.userArea = d.totalArea = UIViewComponentPeer::realScreenPosToRotated (convertToRectInt ([s bounds])) / masterScale;
+        d.userArea = d.totalArea = convertToRectInt ([s bounds]) / masterScale;
         d.isMain = true;
-        d.scale = masterScale;
-
-        if ([s respondsToSelector: @selector (scale)])
-            d.scale *= s.scale;
-
+        d.scale = masterScale * s.scale;
         d.dpi = 160 * d.scale;
 
         displays.add (d);
