@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -58,6 +57,16 @@ AlertWindow::AlertWindow (const String& title,
 
 AlertWindow::~AlertWindow()
 {
+    // Ensure that the focus does not jump to another TextEditor while we
+    // remove children.
+    for (auto* t : textBoxes)
+        t->setWantsKeyboardFocus (false);
+
+    // Give away focus before removing the editors, so that any TextEditor
+    // with focus has a chance to dismiss native keyboard if shown.
+    if (hasKeyboardFocus (true))
+        Component::unfocusAllComponents();
+
     removeAllChildren();
 }
 
@@ -81,7 +90,7 @@ void AlertWindow::setMessage (const String& message)
 }
 
 //==============================================================================
-void AlertWindow::buttonClicked (Button* button)
+void AlertWindow::exitAlert (Button* button)
 {
     if (auto* parent = button->getParentComponent())
         parent->exitModalState (button->getCommandID());
@@ -98,10 +107,10 @@ void AlertWindow::addButton (const String& name,
 
     b->setWantsKeyboardFocus (true);
     b->setMouseClickGrabsKeyboardFocus (false);
-    b->setCommandToTrigger (0, returnValue, false);
+    b->setCommandToTrigger (nullptr, returnValue, false);
     b->addShortcut (shortcutKey1);
     b->addShortcut (shortcutKey2);
-    b->addListener (this);
+    b->onClick = [this, b] { exitAlert (b); };
 
     Array<TextButton*> buttonsArray (buttons.begin(), buttons.size());
     auto& lf = getLookAndFeel();
@@ -230,7 +239,7 @@ public:
         setFont (font);
         setText (message, false);
 
-        bestWidth = 2 * (int) std::sqrt (font.getHeight() * font.getStringWidth (message));
+        bestWidth = 2 * (int) std::sqrt (font.getHeight() * (float) font.getStringWidth (message));
     }
 
     void updateLayout (const int width)
@@ -240,7 +249,7 @@ public:
         s.append (getText(), getFont());
 
         TextLayout text;
-        text.createLayoutWithBalancedLineLengths (s, width - 8.0f);
+        text.createLayoutWithBalancedLineLengths (s, (float) width - 8.0f);
         setSize (width, jmin (width, (int) (text.getHeight() + getFont().getHeight())));
     }
 
@@ -346,8 +355,8 @@ void AlertWindow::updateLayout (const bool onlyIncreaseSize)
     auto wid = jmax (messageFont.getStringWidth (text),
                      messageFont.getStringWidth (getName()));
 
-    auto sw = (int) std::sqrt (messageFont.getHeight() * wid);
-    auto w = jmin (300 + sw * 2, (int) (getParentWidth() * 0.7f));
+    auto sw = (int) std::sqrt (messageFont.getHeight() * (float) wid);
+    auto w = jmin (300 + sw * 2, (int) ((float) getParentWidth() * 0.7f));
     const int edgeGap = 10;
     const int labelHeight = 18;
     int iconSpace = 0;
@@ -373,7 +382,7 @@ void AlertWindow::updateLayout (const bool onlyIncreaseSize)
     }
 
     w = jmax (350, (int) textLayout.getWidth() + iconSpace + edgeGap * 4);
-    w = jmin (w, (int) (getParentWidth() * 0.7f));
+    w = jmin (w, (int) ((float) getParentWidth() * 0.7f));
 
     auto textLayoutH = (int) textLayout.getHeight();
     auto textBottom = 16 + titleH + textLayoutH;
@@ -403,12 +412,12 @@ void AlertWindow::updateLayout (const bool onlyIncreaseSize)
     for (auto* tb : textBlocks)
         w = jmax (w, static_cast<const AlertTextComp*> (tb)->bestWidth);
 
-    w = jmin (w, (int) (getParentWidth() * 0.7f));
+    w = jmin (w, (int) ((float) getParentWidth() * 0.7f));
 
     for (auto* tb : textBlocks)
     {
         auto* ac = static_cast<AlertTextComp*> (tb);
-        ac->updateLayout ((int) (w * 0.8f));
+        ac->updateLayout ((int) ((float) w * 0.8f));
         h += ac->getHeight() + 10;
     }
 
@@ -434,7 +443,7 @@ void AlertWindow::updateLayout (const bool onlyIncreaseSize)
         totalWidth += b->getWidth() + spacer;
 
     auto x = (w - totalWidth) / 2;
-    auto y = (int) (getHeight() * 0.95f);
+    auto y = (int) ((float) getHeight() * 0.95f);
 
     for (auto* c : buttons)
     {
@@ -513,7 +522,7 @@ bool AlertWindow::keyPressed (const KeyPress& key)
         }
     }
 
-    if (key.isKeyCode (KeyPress::escapeKey) && escapeKeyCancels && buttons.size() == 0)
+    if (key.isKeyCode (KeyPress::escapeKey) && escapeKeyCancels)
     {
         exitModalState (0);
         return true;
@@ -574,8 +583,8 @@ private:
         auto& lf = associatedComponent != nullptr ? associatedComponent->getLookAndFeel()
                                                   : LookAndFeel::getDefaultLookAndFeel();
 
-        ScopedPointer<Component> alertBox (lf.createAlertWindow (title, message, button1, button2, button3,
-                                                                 iconType, numButtons, associatedComponent));
+        std::unique_ptr<Component> alertBox (lf.createAlertWindow (title, message, button1, button2, button3,
+                                                                   iconType, numButtons, associatedComponent));
 
         jassert (alertBox != nullptr); // you have to return one of these!
 
