@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
+   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-6-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -86,8 +85,8 @@ public:
     void getHighlightArea (RectangleList<float>& area, float x, int y, int lineH, float characterWidth) const
     {
         if (highlightColumnStart < highlightColumnEnd)
-            area.add (Rectangle<float> (x + highlightColumnStart * characterWidth - 1.0f, y - 0.5f,
-                                        (highlightColumnEnd - highlightColumnStart) * characterWidth + 1.5f, lineH + 1.0f));
+            area.add (Rectangle<float> (x + (float) highlightColumnStart * characterWidth - 1.0f, (float) y - 0.5f,
+                                        (float) (highlightColumnEnd - highlightColumnStart) * characterWidth + 1.5f, (float) lineH + 1.0f));
     }
 
     void draw (CodeEditorComponent& owner, Graphics& g, const Font& fontToUse,
@@ -101,7 +100,7 @@ public:
 
         for (auto& token : tokens)
         {
-            const float tokenX = x + column * characterWidth;
+            const float tokenX = x + (float) column * characterWidth;
             if (tokenX > rightClip)
                 break;
 
@@ -109,7 +108,7 @@ public:
             column += token.length;
         }
 
-        as.draw (g, { x, (float) y, column * characterWidth + 10.0f, (float) lineH });
+        as.draw (g, { x, (float) y, (float) column * characterWidth + 10.0f, (float) lineH });
     }
 
 private:
@@ -195,8 +194,9 @@ private:
     {
         jassert (index <= line.length());
 
-        String::CharPointerType t (line.getCharPointer());
+        auto t = line.getCharPointer();
         int col = 0;
+
         for (int i = 0; i < index; ++i)
         {
             if (t.getAndAdvance() != '\t')
@@ -208,8 +208,7 @@ private:
         return col;
     }
 
-    static void addToken (Array<SyntaxToken>& dest, const String& text,
-                          const int length, const int type)
+    static void addToken (Array<SyntaxToken>& dest, const String& text, int length, int type)
     {
         if (length > 1000)
         {
@@ -289,7 +288,7 @@ public:
     void paint (Graphics& g) override
     {
         jassert (dynamic_cast<CodeEditorComponent*> (getParentComponent()) != nullptr);
-        const CodeEditorComponent& editor = *static_cast<CodeEditorComponent*> (getParentComponent());
+        auto& editor = *static_cast<CodeEditorComponent*> (getParentComponent());
 
         g.fillAll (editor.findColour (CodeEditorComponent::backgroundColourId)
                     .overlaidWith (editor.findColour (lineNumberBackgroundId)));
@@ -301,10 +300,10 @@ public:
         const int lastLineToDraw = jmin (editor.lines.size(), clip.getBottom() / lineH + 1,
                                          lastNumLines - editor.firstLineOnScreen);
 
-        const Font lineNumberFont (editor.getFont().withHeight (jmin (13.0f, lineHeightFloat * 0.8f)));
-        const float w = getWidth() - 2.0f;
-
+        auto lineNumberFont = editor.getFont().withHeight (jmin (13.0f, lineHeightFloat * 0.8f));
+        auto w = (float) getWidth() - 2.0f;
         GlyphArrangement ga;
+
         for (int i = firstLineToDraw; i < lastLineToDraw; ++i)
             ga.addFittedText (lineNumberFont, String (editor.firstLineOnScreen + i + 1),
                               0, (float) (lineH * i), w, lineHeightFloat,
@@ -316,7 +315,7 @@ public:
 
     void documentChanged (CodeDocument& doc, int newFirstLine)
     {
-        const int newNumLines = doc.getNumLines();
+        auto newNumLines = doc.getNumLines();
 
         if (newNumLines != lastNumLines || firstLine != newFirstLine)
         {
@@ -339,7 +338,7 @@ CodeEditorComponent::CodeEditorComponent (CodeDocument& doc, CodeTokeniser* cons
       selectionEnd (doc, 0, 0),
       codeTokeniser (tokeniser)
 {
-    pimpl = new Pimpl (*this);
+    pimpl.reset (new Pimpl (*this));
 
     caretPos.setPositionMaintained (true);
     selectionStart.setPositionMaintained (true);
@@ -349,7 +348,8 @@ CodeEditorComponent::CodeEditorComponent (CodeDocument& doc, CodeTokeniser* cons
     setMouseCursor (MouseCursor::IBeamCursor);
     setWantsKeyboardFocus (true);
 
-    addAndMakeVisible (caret = getLookAndFeel().createCaretComponent (this));
+    lookAndFeelChanged();
+    addAndMakeVisible (caret.get());
 
     addAndMakeVisible (verticalScrollBar);
     verticalScrollBar.setSingleStepSize (1.0);
@@ -366,14 +366,14 @@ CodeEditorComponent::CodeEditorComponent (CodeDocument& doc, CodeTokeniser* cons
 
     setLineNumbersShown (true);
 
-    verticalScrollBar.addListener (pimpl);
-    horizontalScrollBar.addListener (pimpl);
-    document.addListener (pimpl);
+    verticalScrollBar.addListener (pimpl.get());
+    horizontalScrollBar.addListener (pimpl.get());
+    document.addListener (pimpl.get());
 }
 
 CodeEditorComponent::~CodeEditorComponent()
 {
-    document.removeListener (pimpl);
+    document.removeListener (pimpl.get());
 }
 
 int CodeEditorComponent::getGutterSize() const noexcept
@@ -398,14 +398,14 @@ bool CodeEditorComponent::isTextInputActive() const
     return true;
 }
 
-void CodeEditorComponent::setTemporaryUnderlining (const Array<Range<int> >&)
+void CodeEditorComponent::setTemporaryUnderlining (const Array<Range<int>>&)
 {
     jassertfalse; // TODO Windows IME not yet supported for this comp..
 }
 
 Rectangle<int> CodeEditorComponent::getCaretRectangle()
 {
-    return getLocalArea (caret, caret->getLocalBounds());
+    return getLocalArea (caret.get(), caret->getLocalBounds());
 }
 
 void CodeEditorComponent::setLineNumbersShown (const bool shouldBeShown)
@@ -413,10 +413,13 @@ void CodeEditorComponent::setLineNumbersShown (const bool shouldBeShown)
     if (showLineNumbers != shouldBeShown)
     {
         showLineNumbers = shouldBeShown;
-        gutter = nullptr;
+        gutter.reset();
 
         if (shouldBeShown)
-            addAndMakeVisible (gutter = new GutterComponent());
+        {
+            gutter.reset (new GutterComponent());
+            addAndMakeVisible (gutter.get());
+        }
 
         resized();
     }
@@ -429,18 +432,18 @@ void CodeEditorComponent::setReadOnly (bool b) noexcept
         readOnly = b;
 
         if (b)
-            removeChildComponent (caret);
+            removeChildComponent (caret.get());
         else
-            addAndMakeVisible (caret);
+            addAndMakeVisible (caret.get());
     }
 }
 
 //==============================================================================
 void CodeEditorComponent::resized()
 {
-    const int visibleWidth = getWidth() - scrollbarThickness - getGutterSize();
+    auto visibleWidth = getWidth() - scrollbarThickness - getGutterSize();
     linesOnScreen   = jmax (1, (getHeight() - scrollbarThickness) / lineHeight);
-    columnsOnScreen = jmax (1, (int) (visibleWidth / charWidth));
+    columnsOnScreen = jmax (1, (int) ((float) visibleWidth / charWidth));
     lines.clear();
     rebuildLineTokens();
     updateCaretPosition();
@@ -461,7 +464,10 @@ void CodeEditorComponent::paint (Graphics& g)
     g.fillAll (findColour (CodeEditorComponent::backgroundColourId));
 
     auto gutterSize = getGutterSize();
-    g.reduceClipRegion (gutterSize, 0, verticalScrollBar.getX() - gutterSize, horizontalScrollBar.getY());
+    auto bottom = horizontalScrollBar.isVisible() ? horizontalScrollBar.getY() : getHeight();
+    auto right  = verticalScrollBar.isVisible()   ? verticalScrollBar.getX()   : getWidth();
+
+    g.reduceClipRegion (gutterSize, 0, right - gutterSize, bottom);
 
     g.setFont (font);
 
@@ -503,9 +509,8 @@ void CodeEditorComponent::rebuildLineTokens()
 {
     pimpl->cancelPendingUpdate();
 
-    const int numNeeded = linesOnScreen + 1;
-
-    int minLineToRepaint = numNeeded;
+    auto numNeeded = linesOnScreen + 1;
+    auto minLineToRepaint = numNeeded;
     int maxLineToRepaint = 0;
 
     if (numNeeded != lines.size())
@@ -547,9 +552,7 @@ void CodeEditorComponent::codeDocumentChanged (const int startIndex, const int e
     const CodeDocument::Position affectedTextStart (document, startIndex);
     const CodeDocument::Position affectedTextEnd (document, endIndex);
 
-    clearCachedIterators (affectedTextStart.getLineNumber());
-
-    rebuildLineTokensAsync();
+    retokenise (startIndex, endIndex);
 
     updateCaretPosition();
     columnToTryToMaintain = -1;
@@ -566,6 +569,16 @@ void CodeEditorComponent::codeDocumentChanged (const int startIndex, const int e
     updateScrollBars();
 }
 
+void CodeEditorComponent::retokenise (int startIndex, int endIndex)
+{
+    const CodeDocument::Position affectedTextStart (document, startIndex);
+    juce::ignoreUnused (endIndex); // Leave room for more efficient impl in future.
+
+    clearCachedIterators (affectedTextStart.getLineNumber());
+
+    rebuildLineTokensAsync();
+}
+
 //==============================================================================
 void CodeEditorComponent::updateCaretPosition()
 {
@@ -582,8 +595,8 @@ void CodeEditorComponent::moveCaretTo (const CodeDocument::Position& newPos, con
     {
         if (dragType == notDragging)
         {
-            if (abs (caretPos.getPosition() - selectionStart.getPosition())
-                  < abs (caretPos.getPosition() - selectionEnd.getPosition()))
+            if (std::abs (caretPos.getPosition() - selectionStart.getPosition())
+                  < std::abs (caretPos.getPosition() - selectionEnd.getPosition()))
                 dragType = draggingSelectionStart;
             else
                 dragType = draggingSelectionEnd;
@@ -595,7 +608,7 @@ void CodeEditorComponent::moveCaretTo (const CodeDocument::Position& newPos, con
 
             if (selectionEnd.getPosition() < selectionStart.getPosition())
             {
-                const CodeDocument::Position temp (selectionStart);
+                auto temp = selectionStart;
                 selectionStart = selectionEnd;
                 selectionEnd = temp;
 
@@ -608,7 +621,7 @@ void CodeEditorComponent::moveCaretTo (const CodeDocument::Position& newPos, con
 
             if (selectionEnd.getPosition() < selectionStart.getPosition())
             {
-                const CodeDocument::Position temp (selectionStart);
+                auto temp = selectionStart;
                 selectionStart = selectionEnd;
                 selectionEnd = temp;
 
@@ -626,6 +639,7 @@ void CodeEditorComponent::moveCaretTo (const CodeDocument::Position& newPos, con
     updateCaretPosition();
     scrollToKeepCaretOnScreen();
     updateScrollBars();
+    caretPositionMoved();
 
     if (appCommandManager != nullptr && selectionWasActive != isHighlightActive())
         appCommandManager->commandStatusChanged();
@@ -723,7 +737,7 @@ void CodeEditorComponent::scrollToKeepCaretOnScreen()
 
 Rectangle<int> CodeEditorComponent::getCharacterBounds (const CodeDocument::Position& pos) const
 {
-    return { roundToInt ((getGutterSize() - xOffset * charWidth) + indexToColumn (pos.getLineNumber(), pos.getIndexInLine()) * charWidth),
+    return { roundToInt ((getGutterSize() - xOffset * charWidth) + (float) indexToColumn (pos.getLineNumber(), pos.getIndexInLine()) * charWidth),
              (pos.getLineNumber() - firstLineOnScreen) * lineHeight,
              roundToInt (charWidth),
              lineHeight };
@@ -754,6 +768,7 @@ void CodeEditorComponent::insertText (const String& newText)
             document.insertText (caretPos, newText);
 
         scrollToKeepCaretOnScreen();
+        caretPositionMoved();
     }
 }
 
@@ -1221,6 +1236,10 @@ void CodeEditorComponent::editorViewportPositionChanged()
 {
 }
 
+void CodeEditorComponent::caretPositionMoved()
+{
+}
+
 //==============================================================================
 ApplicationCommandTarget* CodeEditorComponent::getNextCommandTarget()
 {
@@ -1296,6 +1315,11 @@ bool CodeEditorComponent::perform (const InvocationInfo& info)
     return performCommand (info.commandID);
 }
 
+void CodeEditorComponent::lookAndFeelChanged()
+{
+    caret.reset (getLookAndFeel().createCaretComponent (this));
+}
+
 bool CodeEditorComponent::performCommand (const CommandID commandID)
 {
     switch (commandID)
@@ -1533,7 +1557,7 @@ void CodeEditorComponent::clearCachedIterators (const int firstLineToBeInvalid)
 {
     int i;
     for (i = cachedIterators.size(); --i >= 0;)
-        if (cachedIterators.getUnchecked (i)->getLine() < firstLineToBeInvalid)
+        if (cachedIterators.getUnchecked (i).getLine() < firstLineToBeInvalid)
             break;
 
     cachedIterators.removeRange (jmax (0, i - 1), cachedIterators.size());
@@ -1545,29 +1569,29 @@ void CodeEditorComponent::updateCachedIterators (int maxLineNum)
     const int linesBetweenCachedSources = jmax (10, document.getNumLines() / maxNumCachedPositions);
 
     if (cachedIterators.size() == 0)
-        cachedIterators.add (new CodeDocument::Iterator (document));
+        cachedIterators.add (CodeDocument::Iterator (document));
 
     if (codeTokeniser != nullptr)
     {
         for (;;)
         {
-            auto& last = *cachedIterators.getLast();
+            const auto last = cachedIterators.getLast();
 
             if (last.getLine() >= maxLineNum)
                 break;
 
-            auto* t = new CodeDocument::Iterator (last);
-            cachedIterators.add (t);
+            cachedIterators.add (CodeDocument::Iterator (last));
+            auto& t = cachedIterators.getReference (cachedIterators.size() - 1);
             const int targetLine = jmin (maxLineNum, last.getLine() + linesBetweenCachedSources);
 
             for (;;)
             {
-                codeTokeniser->readNextToken (*t);
+                codeTokeniser->readNextToken (t);
 
-                if (t->getLine() >= targetLine)
+                if (t.getLine() >= targetLine)
                     break;
 
-                if (t->isEOF())
+                if (t.isEOF())
                     return;
             }
         }
@@ -1580,7 +1604,7 @@ void CodeEditorComponent::getIteratorForPosition (int position, CodeDocument::It
     {
         for (int i = cachedIterators.size(); --i >= 0;)
         {
-            auto& t = *cachedIterators.getUnchecked (i);
+            auto& t = cachedIterators.getReference (i);
 
             if (t.getPosition() <= position)
             {
