@@ -23,6 +23,8 @@
   ==============================================================================
 */
 
+JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wdeprecated-declarations")
+
 namespace juce
 {
 
@@ -562,3 +564,5 @@ private:
 };
 
 } // namespace juce
+
+JUCE_END_IGNORE_WARNINGS_GCC_LIKE
