@@ -118,6 +118,14 @@ public:
     */
     bool isDirectory() const;
 
+    /** Checks whether the file is invalid (empty path).
+    */
+    bool isNull() const;
+
+    /** Checks whether the file is valid (non-empty path).
+    */
+    bool isNotNull() const;
+
     /** Returns the size of the file in bytes.
 
         @returns    the number of bytes in the file, or 0 if it doesn't exist.
