@@ -167,6 +167,13 @@ void MemoryBlock::swapWith (MemoryBlock& other) noexcept
     data.swapWith (other.data);
 }
 
+//==============================================================================
+void* MemoryBlock::release () noexcept
+{
+    size = 0;
+    return data.release();
+}
+
 //==============================================================================
 void MemoryBlock::fillWith (const uint8 value) noexcept
 {
