@@ -33,12 +33,14 @@ HINSTANCE getCurrentModuleInstanceHandle() noexcept
     return currentModuleHandle;
 }
 
+# ifndef STATIC_PLUGIN_TARGET
 CARLA_EXPORT
 BOOL WINAPI DllMain(HINSTANCE hInst, DWORD, LPVOID)
 {
     currentModuleHandle = hInst;
     return 1;
 }
+# endif
 #endif
 
 }
@@ -69,6 +71,7 @@ BOOL WINAPI DllMain(HINSTANCE hInst, DWORD, LPVOID)
 
 #include "streams/FileInputSource.cpp"
 #include "streams/InputStream.cpp"
+#include "streams/MemoryInputStream.cpp"
 #include "streams/MemoryOutputStream.cpp"
 #include "streams/OutputStream.cpp"
 
