@@ -87,6 +87,7 @@ public:
           fVolume(1.0f),
           fPool(),
           fReader(),
+          fFilename(),
           fPrograms(hostGetFilePath("audio"), audiofilesWildcard),
           fPreviewData()
 #ifndef __MOD_DEVICES__
@@ -126,7 +127,7 @@ protected:
         {
         case kParameterLooping:
             param.name  = "Loop Mode";
-            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMABLE|
+            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMATABLE|
                                                             NATIVE_PARAMETER_IS_ENABLED|
                                                             NATIVE_PARAMETER_IS_BOOLEAN);
             param.ranges.def = 1.0f;
@@ -135,7 +136,7 @@ protected:
             break;
         case kParameterHostSync:
             param.name  = "Host Sync";
-            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMABLE|
+            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMATABLE|
                                                             NATIVE_PARAMETER_IS_ENABLED|
                                                             NATIVE_PARAMETER_IS_BOOLEAN);
 #ifdef __MOD_DEVICES__
@@ -148,7 +149,7 @@ protected:
             break;
         case kParameterVolume:
             param.name  = "Volume";
-            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMABLE|
+            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMATABLE|
                                                             NATIVE_PARAMETER_IS_ENABLED);
             param.ranges.def = 100.0f;
             param.ranges.min = 0.0f;
@@ -159,7 +160,7 @@ protected:
             break;
         case kParameterEnabled:
             param.name  = "Enabled";
-            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMABLE|
+            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMATABLE|
                                                             NATIVE_PARAMETER_IS_ENABLED|
                                                             NATIVE_PARAMETER_IS_BOOLEAN|
                                                             NATIVE_PARAMETER_USES_DESIGNATION);
@@ -170,7 +171,7 @@ protected:
             break;
         case kParameterInfoChannels:
             param.name  = "Num Channels";
-            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMABLE|
+            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMATABLE|
                                                             NATIVE_PARAMETER_IS_ENABLED|
                                                             NATIVE_PARAMETER_IS_INTEGER|
                                                             NATIVE_PARAMETER_IS_OUTPUT);
@@ -180,7 +181,7 @@ protected:
             break;
         case kParameterInfoBitRate:
             param.name  = "Bit Rate";
-            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMABLE|
+            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMATABLE|
                                                             NATIVE_PARAMETER_IS_ENABLED|
                                                             NATIVE_PARAMETER_IS_INTEGER|
                                                             NATIVE_PARAMETER_IS_OUTPUT);
@@ -190,7 +191,7 @@ protected:
             break;
         case kParameterInfoBitDepth:
             param.name  = "Bit Depth";
-            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMABLE|
+            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMATABLE|
                                                             NATIVE_PARAMETER_IS_ENABLED|
                                                             NATIVE_PARAMETER_IS_INTEGER|
                                                             NATIVE_PARAMETER_IS_OUTPUT);
@@ -200,7 +201,7 @@ protected:
             break;
         case kParameterInfoSampleRate:
             param.name  = "Sample Rate";
-            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMABLE|
+            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMATABLE|
                                                             NATIVE_PARAMETER_IS_ENABLED|
                                                             NATIVE_PARAMETER_IS_INTEGER|
                                                             NATIVE_PARAMETER_IS_OUTPUT);
@@ -210,7 +211,7 @@ protected:
             break;
         case kParameterInfoLength:
             param.name  = "Length";
-            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMABLE|
+            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMATABLE|
                                                             NATIVE_PARAMETER_IS_ENABLED|
                                                             NATIVE_PARAMETER_IS_OUTPUT);
             param.ranges.def = 0.0f;
@@ -220,7 +221,7 @@ protected:
             break;
         case kParameterInfoPosition:
             param.name  = "Position";
-            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMABLE|
+            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMATABLE|
                                                             NATIVE_PARAMETER_IS_ENABLED|
                                                             NATIVE_PARAMETER_IS_OUTPUT);
             param.ranges.def = 0.0f;
@@ -230,7 +231,7 @@ protected:
             break;
         case kParameterInfoPoolFill:
             param.name  = "Pool Fill";
-            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMABLE|
+            param.hints = static_cast<NativeParameterHints>(NATIVE_PARAMETER_IS_AUTOMATABLE|
                                                             NATIVE_PARAMETER_IS_ENABLED|
                                                             NATIVE_PARAMETER_IS_OUTPUT);
             param.ranges.def = 0.0f;
@@ -686,6 +687,15 @@ protected:
     }
 #endif
 
+    void sampleRateChanged(double) override
+    {
+        if (char* const filename = fFilename.releaseBufferPointer())
+        {
+            loadFilename(filename);
+            std::free(filename);
+        }
+    }
+
     // -------------------------------------------------------------------
 
 private:
@@ -705,6 +715,7 @@ private:
 
     AudioFilePool   fPool;
     AudioFileReader fReader;
+    CarlaString     fFilename;
 
     NativeMidiPrograms fPrograms;
     float fPreviewData[108];
@@ -755,6 +766,7 @@ private:
         fInternalTransportFrame = 0;
         fPool.destroy();
         fReader.destroy();
+        fFilename.clear();
 
         if (filename == nullptr || *filename == '\0')
         {
@@ -781,6 +793,7 @@ private:
             }
 
             fDoProcess = true;
+            fFilename = filename;
             hostSendPreviewBufferData('f', previewDataSize, fPreviewData);
         }
         else
