@@ -23,6 +23,15 @@
 
 #include "water/files/File.h"
 
+// --------------------------------------------------------------------------------------------------------------------
+// Expose info functions as needed
+
+#ifndef CARLA_PLUGIN_EXPORT
+# include "utils/Information.cpp"
+#endif
+
+// --------------------------------------------------------------------------------------------------------------------
+
 CARLA_BACKEND_USE_NAMESPACE
 
 // --------------------------------------------------------------------------------------------------------------------
