@@ -48,6 +48,14 @@ typedef void (*EventProcPtr)(XEvent* ev);
 
 static const uint X11Key_Escape = 9;
 static bool gErrorTriggered = false;
+# if defined(__GNUC__) && (__GNUC__ >= 5) && ! defined(__clang__)
+#  pragma GCC diagnostic push
+#  pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
+# endif
+static pthread_mutex_t gErrorMutex = PTHREAD_MUTEX_INITIALIZER;
+# if defined(__GNUC__) && (__GNUC__ >= 5) && ! defined(__clang__)
+#  pragma GCC diagnostic pop
+# endif
 
 static int temporaryErrorHandler(Display*, XErrorEvent*)
 {
@@ -123,6 +131,9 @@ public:
     {
         CARLA_SAFE_ASSERT(! fIsVisible);
 
+        if (fDisplay == nullptr)
+            return;
+
         if (fIsVisible)
         {
             XUnmapWindow(fDisplay, fHostWindow);
@@ -135,11 +146,8 @@ public:
             fHostWindow = 0;
         }
 
-        if (fDisplay != nullptr)
-        {
-            XCloseDisplay(fDisplay);
-            fDisplay = nullptr;
-        }
+        XCloseDisplay(fDisplay);
+        fDisplay = nullptr;
     }
 
     void show() override
@@ -153,47 +161,65 @@ public:
             {
                 if (! fSetSizeCalledAtLeastOnce)
                 {
-                    XSizeHints hints;
-                    carla_zeroStruct(hints);
+                    int width = 0;
+                    int height = 0;
 
-                    if (XGetNormalHints(fDisplay, childWindow, &hints))
+                    XWindowAttributes attrs;
+                    carla_zeroStruct(attrs);
+
+                    pthread_mutex_lock(&gErrorMutex);
+                    const XErrorHandler oldErrorHandler = XSetErrorHandler(temporaryErrorHandler);
+                    gErrorTriggered = false;
+
+                    if (XGetWindowAttributes(fDisplay, childWindow, &attrs))
                     {
-                        int width = 0;
-                        int height = 0;
-
-                        if (hints.flags & PSize)
-                        {
-                            width = hints.width;
-                            height = hints.height;
-                        }
-                        else if (hints.flags & PBaseSize)
-                        {
-                            width = hints.base_width;
-                            height = hints.base_height;
-                        }
-                        else if (hints.flags & PMinSize)
-                        {
-                            width = hints.min_width;
-                            height = hints.min_height;
-                        }
-
-                        if (width > 0 && height > 0)
-                            setSize(static_cast<uint>(width), static_cast<uint>(height), false);
+                        width = attrs.width;
+                        height = attrs.height;
                     }
+
+                    XSetErrorHandler(oldErrorHandler);
+                    pthread_mutex_unlock(&gErrorMutex);
+
+                    if (width == 0 && height == 0)
+                    {
+                        XSizeHints sizeHints;
+                        carla_zeroStruct(sizeHints);
+
+                        if (XGetNormalHints(fDisplay, childWindow, &sizeHints))
+                        {
+                            if (sizeHints.flags & PSize)
+                            {
+                                width = sizeHints.width;
+                                height = sizeHints.height;
+                            }
+                            else if (sizeHints.flags & PBaseSize)
+                            {
+                                width = sizeHints.base_width;
+                                height = sizeHints.base_height;
+                            }
+                        }
+                    }
+
+                    if (width > 1 && height > 1)
+                        setSize(static_cast<uint>(width), static_cast<uint>(height), false);
                 }
 
                 const Atom _xevp = XInternAtom(fDisplay, "_XEventProc", False);
 
-                gErrorTriggered = false;
+                pthread_mutex_lock(&gErrorMutex);
                 const XErrorHandler oldErrorHandler(XSetErrorHandler(temporaryErrorHandler));
+                gErrorTriggered = false;
 
                 Atom actualType;
                 int actualFormat;
                 ulong nitems, bytesAfter;
                 uchar* data = nullptr;
 
-                XGetWindowProperty(fDisplay, childWindow, _xevp, 0, 1, False, AnyPropertyType, &actualType, &actualFormat, &nitems, &bytesAfter, &data);
+                XGetWindowProperty(fDisplay, childWindow, _xevp, 0, 1, False, AnyPropertyType,
+                                   &actualType, &actualFormat, &nitems, &bytesAfter, &data);
+
                 XSetErrorHandler(oldErrorHandler);
+                pthread_mutex_unlock(&gErrorMutex);
 
                 if (nitems == 1 && ! gErrorTriggered)
                 {
@@ -255,8 +281,9 @@ public:
                         {
                             if (! fChildWindowConfigured)
                             {
-                                gErrorTriggered = false;
+                                pthread_mutex_lock(&gErrorMutex);
                                 const XErrorHandler oldErrorHandler = XSetErrorHandler(temporaryErrorHandler);
+                                gErrorTriggered = false;
 
                                 XSizeHints sizeHints;
                                 carla_zeroStruct(sizeHints);
@@ -273,6 +300,7 @@ public:
 
                                 fChildWindowConfigured = true;
                                 XSetErrorHandler(oldErrorHandler);
+                                pthread_mutex_unlock(&gErrorMutex);
                             }
 
                             if (fChildWindow != 0)
@@ -313,7 +341,13 @@ public:
                 if (fChildWindow == 0)
                     fChildWindow = getChildWindow();
                 if (fChildWindow != 0)
-                    XSetInputFocus(fDisplay, fChildWindow, RevertToPointerRoot, CurrentTime);
+                {
+                    XWindowAttributes wa;
+                    carla_zeroStruct(wa);
+
+                    if (XGetWindowAttributes(fDisplay, fChildWindow, &wa) && wa.map_state == IsViewable)
+                        XSetInputFocus(fDisplay, fChildWindow, RevertToPointerRoot, CurrentTime);
+                }
                 break;
             }
 
@@ -323,7 +357,7 @@ public:
                 fEventProc(&event);
         }
 
-        if (nextWidth != 0 && nextHeight != 0)
+        if (nextWidth != 0 && nextHeight != 0 && fChildWindow != 0)
         {
             XSizeHints sizeHints;
             carla_zeroStruct(sizeHints);
@@ -462,11 +496,14 @@ private:
 #ifdef CARLA_OS_MAC
 
 #if defined(BUILD_BRIDGE_ALTERNATIVE_ARCH)
-# define CarlaPluginWindow CARLA_JOIN_MACRO(CarlaPluginWindowBridgedArch, CARLA_PLUGIN_UI_CLASS_PREFIX)
+# define CarlaPluginWindow CARLA_JOIN_MACRO3(CarlaPluginWindowBridgedArch, CARLA_BACKEND_NAMESPACE, CARLA_PLUGIN_UI_CLASS_PREFIX)
+# define CarlaPluginWindowDelegate CARLA_JOIN_MACRO3(CarlaPluginWindowDelegateBridgedArch, CARLA_BACKEND_NAMESPACE, CARLA_PLUGIN_UI_CLASS_PREFIX)
 #elif defined(BUILD_BRIDGE)
-# define CarlaPluginWindow CARLA_JOIN_MACRO(CarlaPluginWindowBridged, CARLA_PLUGIN_UI_CLASS_PREFIX)
+# define CarlaPluginWindow CARLA_JOIN_MACRO3(CarlaPluginWindowBridged, CARLA_BACKEND_NAMESPACE, CARLA_PLUGIN_UI_CLASS_PREFIX)
+# define CarlaPluginWindowDelegate CARLA_JOIN_MACRO3(CarlaPluginWindowDelegateBridged, CARLA_BACKEND_NAMESPACE, CARLA_PLUGIN_UI_CLASS_PREFIX)
 #else
-# define CarlaPluginWindow CARLA_JOIN_MACRO(CarlaPluginWindow, CARLA_PLUGIN_UI_CLASS_PREFIX)
+# define CarlaPluginWindow CARLA_JOIN_MACRO3(CarlaPluginWindow, CARLA_BACKEND_NAMESPACE, CARLA_PLUGIN_UI_CLASS_PREFIX)
+# define CarlaPluginWindowDelegate CARLA_JOIN_MACRO3(CarlaPluginWindowDelegate, CARLA_BACKEND_NAMESPACE, CARLA_PLUGIN_UI_CLASS_PREFIX)
 #endif
 
 @interface CarlaPluginWindow : NSWindow
@@ -518,6 +555,7 @@ private:
 - (instancetype)initWithWindowAndCallback:(CarlaPluginWindow*)window
                                  callback:(CarlaPluginUI::Callback*)callback2;
 - (BOOL)windowShouldClose:(id)sender;
+- (NSSize)windowWillResize:(NSWindow*)sender toSize:(NSSize)frameSize;
 @end
 
 @implementation CarlaPluginWindowDelegate
@@ -545,27 +583,37 @@ private:
     (void)sender;
 }
 
-@end
-
-@interface CarlaPluginView : NSView
-- (void)resizeWithOldSuperviewSize:(NSSize)oldSize;
-@end
-
-@implementation CarlaPluginView
-
-- (void)resizeWithOldSuperviewSize:(NSSize)oldSize
+- (NSSize)windowWillResize:(NSWindow*)sender toSize:(NSSize)frameSize
 {
-    [super resizeWithOldSuperviewSize:oldSize];
-
-    /*
-    for (NSView* subview in [self subviews])
+    for (NSView* subview in [[window contentView] subviews])
     {
-        [subview setFrame:NSMakeRect(0, 0, oldSize.width, oldSize.height)];
+        const NSSize minSize = [subview fittingSize];
+        if (frameSize.width < minSize.width)
+            frameSize.width = minSize.width;
+        if (frameSize.height < minSize.height)
+            frameSize.height = minSize.height;
         break;
     }
-    */
+
+    return frameSize;
 }
 
+/*
+- (void)windowDidResize:(NSWindow*)sender
+{
+    carla_stdout("window did resize %p %f %f", sender, [window frame].size.width, [window frame].size.height);
+
+    const NSSize size = [window frame].size;
+    NSView* const view = [window contentView];
+
+    for (NSView* subview in [view subviews])
+    {
+        [subview setFrameSize:size];
+        break;
+    }
+}
+*/
+
 @end
 
 class CocoaPluginUI : public CarlaPluginUI
@@ -578,10 +626,10 @@ public:
           fWindow(nullptr)
     {
         carla_debug("CocoaPluginUI::CocoaPluginUI(%p, " P_UINTPTR, "%s)", callback, parentId, bool2str(isResizable));
-        const CarlaBackend::AutoNSAutoreleasePool arp;
+        const CARLA_BACKEND_NAMESPACE::AutoNSAutoreleasePool arp;
         [NSApplication sharedApplication];
 
-        fView = [[CarlaPluginView new]retain];
+        fView = [[NSView new]retain];
         CARLA_SAFE_ASSERT_RETURN(fView != nullptr,)
 
         uint style = NSClosableWindowMask | NSTitledWindowMask;
@@ -606,13 +654,26 @@ public:
             return;
         }
 
-        ((NSWindow*)fWindow).delegate = [[[CarlaPluginWindowDelegate alloc] 
+        ((NSWindow*)fWindow).delegate = [[[CarlaPluginWindowDelegate alloc]
             initWithWindowAndCallback:fWindow
                              callback:callback] retain];
 
-        // if (! isResizable)
+        /*
+        if (isResizable)
+        {
+            [fView setAutoresizingMask:(NSViewWidthSizable |
+                                        NSViewHeightSizable |
+                                        NSViewMinXMargin |
+                                        NSViewMaxXMargin |
+                                        NSViewMinYMargin |
+                                        NSViewMaxYMargin)];
+            [fView setAutoresizesSubviews:YES];
+        }
+        else
+        */
         {
             [fView setAutoresizingMask:NSViewNotSizable];
+            [fView setAutoresizesSubviews:NO];
             [[fWindow standardWindowButton:NSWindowZoomButton] setHidden:YES];
         }
 
@@ -668,6 +729,19 @@ public:
     void idle() override
     {
         // carla_debug("CocoaPluginUI::idle()");
+
+        for (NSView* subview in [fView subviews])
+        {
+            const NSSize viewSize = [fView frame].size;
+            const NSSize subviewSize = [subview frame].size;
+
+            if (viewSize.width != subviewSize.width || viewSize.height != subviewSize.height)
+            {
+                [fView setFrameSize:subviewSize];
+                [fWindow setContentSize:subviewSize];
+            }
+            break;
+        }
     }
 
     void focus() override
@@ -686,17 +760,21 @@ public:
         CARLA_SAFE_ASSERT_RETURN(fWindow != nullptr,);
         CARLA_SAFE_ASSERT_RETURN(fView != nullptr,);
 
-        [fView setFrame:NSMakeRect(0, 0, width, height)];
-
-        for (NSView* subview in [fView subviews])
-        {
-            [subview setFrame:NSMakeRect(0, 0, width, height)];
-            break;
-        }
-
         const NSSize size = NSMakeSize(width, height);
+
+        [fView setFrameSize:size];
         [fWindow setContentSize:size];
 
+        // this is needed for a few plugins
+        if (forceUpdate)
+        {
+            for (NSView* subview in [fView subviews])
+            {
+                [subview setFrame:[fView frame]];
+                break;
+            }
+        }
+
         /*
         if (fIsResizable)
         {
