@@ -29,7 +29,7 @@ public:
     public:
         virtual ~Callback() {}
         virtual void handlePluginUIClosed() = 0;
-        virtual void handlePluginUIResized(const uint width, const uint height) = 0;
+        virtual void handlePluginUIResized(uint width, uint height) = 0;
     };
 
     virtual ~CarlaPluginUI() {}
@@ -37,17 +37,17 @@ public:
     virtual void hide() = 0;
     virtual void focus() = 0;
     virtual void idle() = 0;
-    virtual void setSize(const uint with, const uint height, const bool forceUpdate) = 0;
-    virtual void setTitle(const char* const title) = 0;
-    virtual void setChildWindow(void* const ptr) = 0;
-    virtual void setTransientWinId(const uintptr_t winId) = 0;
+    virtual void setSize(uint with, uint height, bool forceUpdate) = 0;
+    virtual void setTitle(const char* title) = 0;
+    virtual void setChildWindow(void* ptr) = 0;
+    virtual void setTransientWinId(uintptr_t winId) = 0;
     virtual void* getPtr() const noexcept = 0;
 #ifdef HAVE_X11
     virtual void* getDisplay() const noexcept = 0;
 #endif
 
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
-    static bool tryTransientWinIdMatch(const uintptr_t pid, const char* const uiTitle, const uintptr_t winId, const bool centerUI);
+    static bool tryTransientWinIdMatch(uintptr_t pid, const char* uiTitle, uintptr_t winId, bool centerUI);
 #endif
 
 #ifdef CARLA_OS_MAC
